C
C**   UNIT NAME: INSPBUFR - INPECTION AND DECODING OF BUFR MESSAGES
C
C     UNIT TYPE: MAIN PROGRAM
C
C     PROGRAMMING LANGUAGE: FORTRAN 77
C
C     PURPOSE:
C        COUNTS/LISTS/DECODES BUFR MESSAGES CORRECTLY DELIMITED, I.E.
C        STARTING WITH "BUFR" AND ENDING WITH "7777". 
C
C     SUMMARY/METHOD:
C  
C        ASKS USER FOR INPUT FILE PARAMETERS (TITLE AND RECORDTYPE) 
C          THROUGH A CALL TO SUBROUTINE WHICFL.
C        ASKS USER FOR PARAMETERS OF PROCESSING (WHAT IS TO BE DONE
C          BY THIS RUN (THE USER CAN SELECT WHICH INFORMATION IS TO BE  
C          PRINTED ON OUTPUT). THIS IS DONE THROUGH A CALL TO SUBROUTINE
C          OPTIONS.     
C        PERFORMS A SERIE OF CALLS TO BURDVM/BURDVMSS SUBROUTINE TO GET
C           ONE BUFR MESSAGE AT A TIME.             
C         
C          
C     USAGE:
C        inspbufr    (WITHOUT PARAMETERS, FROM UNIX-SHELL SCRIPTS)
C
C     INTERFACES:
C        GLOBAL VARIABLES: NONE.
C        FILES:
C           (5)    - STANDARD INPUT FOR INPUT FILE PARAMETERS
C           (6)    - STANDARD OUTPUT WITH REPORT OF THE RUN
C           (8)    - INPUT FILE WITH BUFR MESSAGES
C
C     EXTERNAL CODES:
C        CALLED MODULES: SUBROUTINE WSCCS
C                             "     WHICFL
C                             "     OPTIONS
C                             "     BURDVMPB
C                             "     BUFREX
C                             "     BUPRS0
C                             "     BUPRS1
C                             "     BUPRS2
C                             "     BUSEL
C                             "     BUPRT
C
C        RELATED MODULES: NONE
C 
C     REMARKS: BASED ON TEST.FOR PROGRAM FROM ECMWF.
C              USES THE BUFRDC.F PACKAGE FROM ECMWF.
C   
C     KEYWORDS:
C        WMO CODE FM 94 BUFR.
C
C     ORIGIN/AUTHOR:
C        CPTEC - ENIVALDO.
C  
C     VERSIONS:
C        ORIGINAL: 
C                       
C                 
C     MODIFICATIONS:
C              
C    
C**
C     
      PROGRAM inspbufr
C
C
      INCLUDE 'PARBUF'
      PARAMETER (KELEM=2000,KMAXSB=200,KVALS=KMAXSB*KELEM)
C
C     KELEM - MAXIMUM NUMBER OF DATA VALUES PER DATA SUBSET
C     KMAXSB- MAXIMUM NUMBER OF DATA SUBSETS
C  
C 
      LOGICAL OPRTMSG,OPRTS0,OPRTS1,OPRTS2,OPRTS3,OPRTS4,
     1        ODECODE,OCTONLY
      DIMENSION KBUFF(JBUFL)
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
      CHARACTER*80 CVALS(KVALS)
C
      DIMENSION KSUP(JSUP)  ,KSEC0(JSEC0),KSEC1(JSEC1)
      DIMENSION KSEC2(JSEC2),KSEC3(JSEC3),KSEC4(JSEC4)
      DIMENSION KEY  (JKEY)
C
      DIMENSION VALUES(KVALS)
      DIMENSION KTDLST(KELEM),KTDEXP(KELEM)
C                                                                       
      DATA CNAMES/KELEM*' '/,CUNITS/KELEM*' '/
C
C     INITIALIZATION OF VARIABLES
C
      N=0
      MESSWRIT=0
      CALL WSCCS
C
C     WHICH FILE AND WHICH OPTIONS FOR PROCESSING 
C
      CALL WHICFL(IUN)
      CALL OPTIONS(OPRTMSG,OPRTS0,OPRTS1,OPRTS2,OPRTS3,
     1             OPRTS4,ODECODE,OCTONLY,INITMSG,IENDMSG,
     2             INITDSUB,IENDDSUB)
      IF (INITMSG .EQ. 0 .AND. IENDMSG .EQ. 0) THEN
           INITMSG = 1
           IENDMSG = 90000
      ENDIF

 100  CONTINUE
C
C     READ IN ONE BUFR MESSAGE AT TIME.
C
      CALL BURDVMPB(IUN,JBUFL,KBUFF,KBUFL,IERR) 
      IF(IERR.NE.0) GO TO 200
C
C     MESSAGE WAS CORRECTLY IDENTIFIED BY INPUT ROUTINE 
C     CHECK IF IT IS TO BE PROCESSED (RANGE INITMSG -> IENDMSG)
C
      N=N+1
      IF (N .LT. INITMSG) GO TO 100
      IF (N .GT. IENDMSG) THEN
          IERR = -1
          GO TO 200
      ENDIF
C
C     MESSAGE IS TO BE PROCESSED
C
      MESSWRIT=MESSWRIT+1
      IF (.NOT. OCTONLY) WRITE(*,90998),N,KBUFL
      IF (OPRTMSG) WRITE(*,90999)(KBUFF(I),I=1,KBUFL)
      IF (.NOT. ODECODE) GO TO 100
C
C     MESSAGE IS TO BE DECODED
C
      CALL BUFREX(KBUFL,KBUFF,KSUP,KSEC0 ,KSEC1,KSEC2 ,KSEC3 ,KSEC4,
     1            KELEM,CNAMES,CUNITS,KVALS,VALUES,CVALS,IERR)
C
      IF(IERR.NE.0) GO TO 200
C
C     PRINT SECTIONS OF BUFR MESSAGE
C
C     PRINT SECTION 0
      IF (OPRTS0) CALL BUPRS0(KSEC0,KSUP)
C     PRINT SECTION 1
      IF (OPRTS1) CALL BUPRS1(KSEC1)
C     PRINT SECTION 2
      IF (OPRTS2) then
         CALL BUUKEY(KSEC1,KSEC2,KEY,KSUP,KERR)
         CALL BUPRS2(KSUP ,KEY)
      endif     
C
C     GET DATA DESCRIPTORS FROM COMMON BLOCKS FROM SECTION 3
C
      CALL BUSEL(KTDLEN,KTDLST,KTDEXL,KTDEXP,KERR)
C
      IF (OPRTS3) THEN
C        PRINT SECTION 3 DATA DESCRIPTORS(UNEXPANDED AND EXPANDED)
         CALL BUPRS3(KSEC3,KTDLEN,KTDLST,KTDEXL,KTDEXP,KELEM,CNAMES)
      ENDIF
C
      IF (OPRTS4) THEN
C        PRINT DATA FROM SECTION 4 
         IF (INITDSUB .EQ. 0 .AND. IENDDSUB .EQ. 0) THEN
            IST=1
            IEND=KSUP(6) 
         ELSE
            IST=INITDSUB
            IEND=IENDDSUB
         ENDIF
         CALL BUPRT(0,IST,IEND,KELEM,CNAMES,CUNITS,CVALS,
     1              KVALS,VALUES,KSUP,KSEC1,ksec4(1),IERR)
      ENDIF
      GO TO 100
C
 200  CONTINUE
C
C     AN ERROR OCCURRED:
C   
C         IF IERR = -1 : STOP PROCESSING (END OF FILE OR RANGE OF 
C                                         MESSAGES EXHAUSTED);
C         OTHERWISE, IT WAS A SERIOUS ERROR IN THE BUFR MESSAGE, BUT 
C                                      TRIES TO CONTINUE PROCESSING.    
C
      IF(IERR.EQ.-1) THEN
         WRITE(6,90000)MESSWRIT
         STOP
      ELSE
         write(0,'(a,i3)')' inspbufr : error ',ierr
         GO TO 100
      END IF
C
90000 FORMAT(/'Number of messages processed =',I5/)
90998 FORMAT(/'==> Message # ',I4,'   ===> length (words) = ',I4)
90999 FORMAT('Message Contents ='/8(Z8.8,1X))
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE WHICFL(IUNIT1)
      IMPLICIT NONE
      INTEGER IUNIT1
C
      CHARACTER*60 FL1
      INTEGER IRET
C
      WRITE(6,9000)
C
120   CONTINUE
      WRITE(6,9020)
      READ(5,9030)FL1
      IF (FL1 .EQ.' ' ) GO TO 120 
123   CONTINUE
      CALL PBOPEN(IUNIT1,FL1,'r',IRET)
      IF (IRET.EQ.-1 .OR. IRET.EQ.-2) THEN
           WRITE(6,9080)FL1
           GO TO 120
      ENDIF
      WRITE(6,9050)FL1
C         
      RETURN         
C         
9000  FORMAT(//'   INSPBUFR - INTERACTIVE BUFR DECODING PROGRAM'//)
9020  FORMAT('INPUT BUFR FILENAME: '$)
9030  FORMAT(A60)
9050  FORMAT(/'BUFR FILE IS  ======> ',A60)
9080  FORMAT(/'ERROR WHEN OPENING FILE ', A60/)
C         
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE OPTIONS(OPRTMSG,OPRTS0,OPRTS1,OPRTS2,OPRTS3,
     1                   OPRTS4,ODECODE,OCTONLY,INITMSG,IENDMSG,
     2                   INITDSUB,IENDDSUB)
      IMPLICIT NONE
      LOGICAL OPRTMSG,OPRTS0,OPRTS1,OPRTS2,OPRTS3,OPRTS4,
     1        ODECODE,OCTONLY
      INTEGER INITMSG,IENDMSG,INITDSUB,IENDDSUB
C
      CHARACTER*1 T
      LOGICAL ODEFINSP
C
      OPRTMSG=.FALSE.
      OPRTS0=.FALSE.
      OPRTS1=.TRUE.
      OPRTS2=.FALSE.
      OPRTS3=.FALSE.
      OPRTS4=.TRUE.
      ODECODE=.TRUE.
      OCTONLY=.FALSE.
C
      WRITE(6,90100)
      CALL ASKBOOL(
     1  'WANT TO USE DEFAULT INSPECTION (SECTIONS 1 AND 4)? (Y/N): ',
     2             ODEFINSP,.TRUE.)
      IF (ODEFINSP) GO TO 400
      CALL ASKBOOL('WANT TO DECODE MESSAGES? (Y/N): ',
     1             ODECODE,.TRUE.)
      CALL ASKBOOL('WANT TO PRINT RAW MESSAGES? (N/Y): ',
     1             OPRTMSG,.FALSE.)
      IF (.NOT. ODECODE) GO TO 300
      CALL ASKBOOL('WANT TO PRINT SECTION 0 INFORMATION? (N/Y): ',
     1             OPRTS0,.FALSE.)
      CALL ASKBOOL('WANT TO PRINT SECTION 1 INFORMATION? (Y/N): ',
     1             OPRTS1,.TRUE.)
      CALL ASKBOOL('WANT TO PRINT SECTION 2 INFORMATION? (N/Y: ',
     1             OPRTS2,.FALSE.)
      CALL ASKBOOL('WANT TO PRINT SECTION 3 INFORMATION? (N/Y): ',
     1             OPRTS3,.FALSE.)
      CALL ASKBOOL('WANT TO PRINT SECTION 4 INFORMATION? (Y/N): ',
     1             OPRTS4,.TRUE.)
300   CONTINUE
      IF (OPRTMSG .OR. 
     1   (ODECODE .AND. (OPRTS0 .OR. OPRTS1 .OR. OPRTS2 
     2     .OR. OPRTS3 .OR. OPRTS4))) GO TO 400
C   
      CALL ASKBOOL('WANT ONLY THE COUNT OF PROCESSED MESSAGES? (N/Y): ',
     1             OCTONLY,.FALSE.)
C
C     ASKS FOR RANGE OF MESSAGES WITHIN THE FILE
C
400   CONTINUE
      WRITE(6,90000)
      READ(5,*)INITMSG,IENDMSG
      IF (.NOT. (INITMSG .EQ. 0 .AND. IENDMSG .EQ. 0))THEN
         IF (INITMSG .LE. 0 .OR. IENDMSG .LE. 0 .OR.
     1       INITMSG .GT. IENDMSG)       GO TO 400
      ENDIF
500   CONTINUE
      IF (.NOT. (ODECODE .AND. OPRTS4))GO TO 700
600   CONTINUE
      WRITE(6,90200)
      READ(5,*)INITDSUB,IENDDSUB
      IF (INITDSUB .EQ. 0 .AND. IENDDSUB .EQ. 0) RETURN
      IF (INITDSUB .LE. 0 .OR. IENDDSUB .LE. 0 .OR.
     1           INITDSUB .GT. IENDDSUB)                     
     1        GO TO 600
700   CONTINUE
      RETURN
C         
90000 FORMAT(/'INPUT NUMBER OF INITIAL AND FINAL MESSAGES ', 
     1        'FOR ALL, TYPE 0 0): '$)
90100 FORMAT(/'HINT : in the following questions the DEFAULT option'
     1       /'       is the FIRST one, which may be chosen just by'
     2       /'       pressing the RETURN key'/)
90200 FORMAT(/'INPUT NUMBER OF INITIAL AND FINAL DATA-SUBSETS ', 
     1        '(FOR ALL, TYPE 0 0): '$)
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE ASKBOOL(CMSG,OBVAR,ODFLT)
      IMPLICIT NONE
      CHARACTER*(*) CMSG
      LOGICAL OBVAR,ODFLT
C
      CHARACTER*1 CRESP,CDFLT
      
C
      IF (ODFLT) THEN
          CDFLT = 'Y'
      ELSE
          CDFLT = 'N'
      ENDIF
C
100   CONTINUE
      WRITE(6,90000)CMSG
      READ(5,90100)CRESP
      IF (CRESP .EQ. ' ') CRESP = CDFLT
      IF (CRESP .EQ. 'Y' .OR. CRESP .EQ. 'y') THEN
           OBVAR = .TRUE.
      ELSE IF (CRESP .EQ. 'N' .OR. CRESP .EQ. 'n') THEN
           OBVAR = .FALSE.
      ELSE
           GO TO 100
      ENDIF
      RETURN
C
90000 FORMAT(/A,$)
90100 FORMAT(A1)
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE WSCCS
      CHARACTER*50 SCCSID
      SCCSID='SCCSID = @(#)inspbufr.f Version 2.2 (8/8/95)'
      WRITE(6,'(/A50//)')SCCSID
      END
