      SUBROUTINE BUFREX(KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KSEC2,KSEC3,KSEC4,
     2                  KELEM,CNAMES,CUNITS,KVALS,VALUES,CVALS,KERR)
C                                                                       
C**** *BUFREX*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Decode Bufr message into fully expanded form; returning
C     information relevant for all bufr sections, expanded values,
C     their names and units.
C
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUFREX(KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KSEC2,KSEC3,KSEC4,
C     1                   KELEM,CNAMES,CUNITS,KVALS,VALUES,CVALS,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *KVALS*   -  dimension of VALUES array
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC0*   -  array containing section 0 information
C                            KSEC0( 1)-- length of section 0 (bytes)
C                            KSEC0( 2)-- total length of Bufr message (bytes)
C                            KSEC0( 3)-- Bufr Edition number
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C                            KSEC1(16) - KSEC1(JSEC1) -- local ADP centre
C                                        information(BYTE by BYTE)
C               *KSEC2*   -  array containing section 2 information
C                            KSEC2( 1)-- length of section 2 (bytes)
C                            KSEC2( 2) to KSEC2(47) RDB key
C               *KSEC3*   -  array containing section 3 information
C                            KSEC3( 1)-- length of section 3 (bytes)
C                            KSEC3( 2)-- reserved
C                            KSEC3( 3)-- number of subsets
C                            KSEC3( 4)-- flag (data type,data compression)
C               *KSEC4*   -  array containing section 4 information
C                            KSEC4( 1)-- length of section 4 (bytes)
C                            KSEC4( 2)-- reserved
C               *CNAMES*  -  character array containing element names
C               *CUNITS*  -  character array containig units
C               *VALUES*  -  real array (expanded data values)
C               *CVALS*   -  character array containing bufr code table
C                            entries 
C               *KERR*    -  returned error code
C                           
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          Bufr message passed as argument to this routine is decoded
C     section by section. Suplementary information and expanded data
C     are returned as well as error code. During bit pattern unpacking 
C     GBYTE and GBYTES routines (VMS version) are used.
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUEXS0   -  expands section 0 of Bufr message
C          BUEXS1   -  expands section 1 of Bufr message 
C          BUEXS2   -  expands section 2 of Bufr message
C          BUEXS3   -  expands section 3 of Bufr message 
C          BUGBTS   -  load required Bufr tables 
C          BUEXS4   -  expands section 4 of Bufr message 
C          BUEXS5   -  expands section 5 of Bufr message
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF' 
      INCLUDE 'COMWORK' 
      INCLUDE 'COMWT'
      INCLUDE 'COMDEFC'
      INCLUDE 'COMREQ'
C
      DIMENSION KBUFF(KBUFL)
      DIMENSION KSUP(JSUP),KSEC0(JSEC0),KSEC1(JSEC1)
      DIMENSION KSEC2(JSEC2),KSEC3(JSEC3),KSEC4(JSEC4)
C
      DIMENSION  VALUES(KVALS)
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
      CHARACTER*80 CVALS(KVALS)                                      
      CHARACTER*4 CECMWF,CUSER
C
C     ------------------------------------------------------------------
C                                                                       
C*          1.   SET CONSTANTS.
C                --------------
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C
      IF(CECMWF.NE.'ECMF') THEN
         CALL BUEVAR(KERR)
         CECMWF='ECMF'
      END IF
C
      IF(CUSER.NE.'USER') THEN
         NREQ=0
         NRQL=0
      END IF
C     -----------------------------------------------------------------
C*          2.  EXPAND SECTION 0.
C               -----------------
 200  CONTINUE
C
      CALL BUEXS0( KBUFL,KBUFF,KSUP,KSEC0,KERR )
      IF(KERR.NE.0) RETURN
C                                                                       
C     ------------------------------------------------------------------
C*          3.   EXPAND SECTION 1.                                      
C                ------------------                                     
 300  CONTINUE                                                          
C                                                                       
      CALL BUEXS1( KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KERR )
      IF(KERR.NE.0) RETURN
C
C     ------------------------------------------------------------------
C*          4.  EXPAND SECTION 2.                                       
C               -----------------                                       
 400  CONTINUE                                                          
C                                                                       
      CALL BUEXS2( KBUFL,KBUFF,KSUP,KSEC1,KSEC2,KERR )
      IF(KERR.NE.0) RETURN
C                                                                       
C     ------------------------------------------------------------------
C*          5.   LOAD BUFR TABLES.                                      
C                -----------------                                      
 500  CONTINUE                                                          
C                                                                       
      CALL BUGBTS(KSEC1,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C     ------------------------------------------------------------------
C*          6.  EXPAND SECTION 3.                  
C               -----------------
 600  CONTINUE                                                          
C                                                                       
      CALL BUEXS3(KBUFL,KBUFF,KSUP,KSEC3,KELEM,CNAMES,CUNITS,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C     ------------------------------------------------------------------
C*          7.   EXPAND SECTION 4.                 
C                -----------------
 700  CONTINUE                                                          
C                                                                       
      CALL BUEXS4(KBUFL ,KBUFF ,KSUP  ,KSEC3,KSEC4,
     1            KELEM ,CNAMES,CUNITS,KVALS,VALUES,CVALS,KERR)
      IF(KERR.NE.0) RETURN
C
C                                                                       
C     ------------------------------------------------------------------
C*          8.   EXPAND SECTION 5.
C                -----------------
 800  CONTINUE
C
      CALL BUEXS5(KBUFL,KBUFF,KERR)
      IF(KERR.NE.0) RETURN
C
C     -----------------------------------------------------------------
C*          9.   SET TOTAL BUFR MESSAGE LENGTH.
C                ------------------------------
 900  CONTINUE
C
      KSUP(8)=KSEC0(1)+KSEC1(1)+KSEC2(1)+KSEC3(1)+KSEC4(1)+4
C
C     -----------------------------------------------------------------
      RETURN                                                            
C                                                                       
      END                                                               
      SUBROUTINE BUS012( KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KSEC2,KERR)
C
C**** *BUS012*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Expands section 0,1 AND 2 of Bufr message. 
C       
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUS012( KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KSEC2,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC0*   -  array containing section 0 information
C                            KSEC0( 1)-- length of section 0 (bytes)
C                            KSEC0( 2)-- total length of Bufr message (bytes)
C                            KSEC0( 3)-- Bufr Edition number
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C               *KSEC2*   -  array containing section 2 information
C                            KSEC2( 1)-- length of section 2 (bytes)
C                            KSEC2( 2) to KSEC2(47) RDB key
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     --------                                                          
C                                                                       
C          NONE.                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUEXS0          - unpack section 0
C          BUEXS1          - unpack section 1
C          BUEXS2          - unpack section 2
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/07/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
C                                                                       
      INCLUDE 'PARBUF' 
      INCLUDE 'COMWORK' 
      INCLUDE 'COMWT'
      INCLUDE 'COMDEFC'
C
      DIMENSION KBUFF(KBUFL)
      DIMENSION KSUP(JSUP),KSEC0(JSEC0),KSEC1(JSEC1)
      DIMENSION KSEC2(JSEC2)
C
      CHARACTER*4 CECMWF,CUSER
C
C     ------------------------------------------------------------------
C*          1.   SET CONSTANTS.
C                --------------
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C
      IF(CECMWF.NE.'ECMF') THEN
         CALL BUEVAR(KERR)
         CECMWF='ECMF'
      END IF
C
      IF(CUSER.NE.'USER') THEN
         NREQ=0
         NRQL=0
      END IF
C     -----------------------------------------------------------------
C
C*          2.  EXPAND SECTION 0.
C               -----------------
 200  CONTINUE
C
      CALL BUEXS0( KBUFL,KBUFF,KSUP,KSEC0,KERR )
      IF(KERR.NE.0) RETURN
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          3.   EXPAND SECTION 1.                                      
C                ------------------                                     
 300  CONTINUE                                                          
C                                                                       
      CALL BUEXS1( KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KERR )
      IF(KERR.NE.0) RETURN
C
C     ------------------------------------------------------------------
C                                                                       
C*          4.  EXPAND SECTION 2.                                       
C               -----------------                                       
 400  CONTINUE                                                          
C                                                                       
      CALL BUEXS2( KBUFL,KBUFF,KSUP,KSEC1,KSEC2,KERR )
      IF(KERR.NE.0) RETURN
C                                                                       
C*          5.  EXPAND SECTION 3 (PRELIMINARY ITEMS).
C               -------------------------------------
 500  CONTINUE
C
      CALL BUEXS3P(KBUFL,KBUFF,KSUP,KERR)
C
      RETURN
      END

      SUBROUTINE BURDVM(KUNIT,KLEN,KBUFF,KBUFL,KERR)
C                                                                       
C**** *BURDVM*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Read in one BUFR message ; returns length of message
C     and message itself.
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BURDVM(KUNIT,KLEN,KBUFF,KBUFL,KERR)*
C
C        INPUT :
C               *KUNIT*    -  unit number
C               *KLEN*     -  length of bufr message
C        OUTPUT:
C               *KBUFF*    -  array (containg message)
C               *KBUFL*    -  length of bufr message in computer words
C               *KERR*     -  return code
C                            -2  error during read of input file
C                            -1  end of file
C                             0  no errors detected
C                             1  start of bufr message NOT found
C                             2  end of bufr message not found
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          For the unit number, array and it dimension,
C     a bufr message is returned as well as its length and 
C     return error code.
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          NONE.
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*      JANUARY 1991.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      CHARACTER*32766 YBUFF,YBUFR
C
      DIMENSION KBUFF(KLEN)
      DIMENSION IBUFF(8192)
C
      EQUIVALENCE (IBUFF,YBUFF)
C                                                                       
C     ------------------------------------------------------------------
C*          1.  READ IN MESSAGE; STRIP OFF TELECOMMUNICATION HEADER.
C               ----------------------------------------------------
C
 100  CONTINUE                                                          
C                                                                       
      KERR = 0
      J    = 0
      YBUFF=' '
C
      IF(OMORE) GO TO 110
C
      YBUFR=' '
      READ(KUNIT,'(Q,A<ILEN>)',END=300,ERR=200,IOSTAT=IOS) ILEN,YBUFR
      IF(ILEN.EQ.0) GO TO 100
C
C*          1.1 FIND START OF BUFR MESSAGE.
C               -------------------------------
C
 110  CONTINUE
C
      IST=INDEX(YBUFR(1:ILEN),'BUFR')
      IF(IST.EQ.0) THEN
C         KERR=1
C         WRITE(0,'(A)')'BURDVM :'
C         CALL BUERR(KERR)
C         RETURN
          GO TO 100
      END IF
C
C*          1.2 FIND END OF BUFR MESSAGE.
C               -------------------------
 120  CONTINUE
C
      IEND=INDEX(YBUFR(1:ILEN),'7777')
      IF(IEND.NE.0) THEN
 1336     IE=IEND+4
          IF(YBUFR(IE:IE).EQ.'7') THEN
             IEND=IEND+1
             GO TO 1336
          END IF
      END IF
C
C*          1.3 CHECK IF END FOUND.
C               -------------------
 130  CONTINUE
C
      IF(IEND.EQ.0) THEN
C
C          1.3.1 FIND START OF NEXT BUFR MESSAGE.
C
         ISTNEXT=INDEX(YBUFR(IST+3:ILEN),'BUFR')
         IF(ISTNEXT.EQ.0) THEN
            DO 131 I=IST,ILEN
            J=J+1
            YBUFF(J:J)=YBUFR(I:I)
 131        CONTINUE
C
 132        CONTINUE
C
C          1.3.2 READ NEXT RECORD
C
            ybufr=' '
            READ(KUNIT,'(Q,A<ILEN>)',END=300,ERR=200,IOSTAT=IOS)
     1                               ILEN,YBUFR
C
            if(ilen.eq.0) then
               print*,' Length of bufr message is zero.'
               omore=.false.
               go to 100
            end if
c
            IST=INDEX(YBUFR(1:ILEN),'BUFR')
            IEND=INDEX(YBUFR(1:ILEN),'7777')
            IF(IEND.NE.0) THEN
 1333          IE=IEND+4
                IF(YBUFR(IE:IE).EQ.'7') THEN
                   IEND=IEND+1
                   GO TO 1333
               END IF
            END IF
            IF(IST.EQ.0.AND.IEND.EQ.0) THEN
C 
C              MOVE DATA TO WORK ARRAY
C
               DO 1321 I=1,ILEN
               J=J+1
               YBUFF(J:J)=YBUFR(I:I)
 1321          CONTINUE
               GO TO 132
            END IF
            IF(IST.EQ.0.AND.IEND.NE.0) THEN
C 
C              MOVE DATA TO WORK ARRAY
C
               IST=1
               GO TO 133
C
            END IF
C
            IF(IST.NE.0.AND.IEND.EQ.0) THEN
               KERR=2
               WRITE(0,'(A)')'BURDVM :'
               CALL BUERR(KERR)
               OMOR=.FALSE.
               GO TO 100
            END IF
            IF(IST.NE.0.AND.IEND.NE.0) THEN
               IF(IST.LT.IEND) THEN
                  KERR=2
                  WRITE(0,'(A)')'BURDVM :'
                  CALL BUERR(KERR)
                  OMORE=.FALSE.
                  GO TO 100
               ELSE
C
C              MOVE DATA TO WORK UNTIL THE END
C
                  IST=1
               END IF
            END IF
         ELSE
            KERR=2
            WRITE(0,'(A)')'BURDVM :'
            CALL BUERR(KERR)
            OMORE=.FALSE.
            GO TO 100
         END IF
      END IF
C
 133  CONTINUE
C
      DO 121 I=IST,IEND+3
      J=J+1
      YBUFF(J:J)=YBUFR(I:I)
 121  CONTINUE
C
      OMORE=.FALSE.
C
      IF(IEND+4.LT.ILEN) THEN
         JJ=0
         DO 123 I=IEND+4,ILEN
         JJ=JJ+1
         YBUFR(JJ:JJ)=YBUFR(I:I)
 123     CONTINUE
C
         ILEN=JJ
         IS=INDEX(YBUFR(1:ILEN),'BUFR')
         IF(IS.NE.0) OMORE=.TRUE.
      END IF
C
      IBYTES=J
      IWORD =IBYTES/4
      IOFF  =IBYTES-IWORD*4 
      IF(IOFF.NE.0) IWORD=IWORD+1
C
      IF(IWORD.GT.KLEN) THEN
         KERR=3
         WRITE(0,'(A)')'BURDVM :'
         CALL BUERR(KERR)
         WRITE(0,'(A,I8)')'Suggested size ',iword
         RETURN
      END IF
C
      KBUFL=IWORD 
C                
      DO 122 I=1,KBUFL
      KBUFF(I)=IBUFF(I)
 122  CONTINUE
C 
C     INVERTS BYTES IN MESSAGE (ENIVALDO, 28/06/93)
C
      CALL INVBYTES(IBYTES,KBUFF) 
     
      RETURN     
C                
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
 200  CONTINUE                                                          
C                                                                       
      KERR=-2
      WRITE(0,9902) KUNIT,IOS
      RETURN
C                                                                       
C     -----------------------------------------------------------------
C
 300  CONTINUE                                                          
C
      KERR= -1
C      WRITE(0,9901) KUNIT
      RETURN                                                            
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
 9901 FORMAT(1H ,' BURDVM : End of file on unit',i3)        
 9902 FORMAT(1H ,' BURDVM : Error during read unit',I3,'   Status=',I3)
C                                                                       
      END                                                               
      SUBROUTINE BUEXS0( KBUFL,KBUFF,KSUP,KSEC0,KERR)
C                                                                       
C**** *BUEXS0*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Expands section 0 of Bufr message. Saves Bufr Edition number
C     and total length of Bufr message (bytes).
C       
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUEXS0( KBUFL,KBUFF,KSUP,KSEC0,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC0*   -  array containing section 0 information
C                            KSEC0( 1)-- length of section 0 (bytes)
C                            KSEC0( 2)-- total length of Bufr message (bytes)
C                            KSEC0( 3)-- Bufr Edition number
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     --------                                                          
C                                                                       
C          NONE.                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPKS          - unpack bit pattern in repeated way
C          BUNPCK          - unpack bit pattern         
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/09/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C                                                                       
      DIMENSION KBUFF(KBUFL),KSUP(JSUP),KSEC0(JSEC0)
      DIMENSION IBUFR(4)                                                
C
      CHARACTER*4 YBUFR
C                                                                       
C     ------------------------------------------------------------------
C*          1.   EXPAND SECTION 0.                                      
C                -----------------                                      
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C
C                                                                       
C*          1.1  INITIALIZE WORKING POINTERS NWPT AND NBPT.             
C                ------------------------------------------             
      NWPT = 1                                                          
      NBPT = 0                                                          
C                                                                       
C*          1.2  UNPACK FIRST FOUR OCTETS CONTAINING *BUFR*.            
C                -------------------------------------------            
C
      CALL BUNPKS(NBPW,KBUFF(NWPT),IBUFR,NWPT,NBPT,8,0,4,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.3  CHECK IF FIRST FOUR OCTETS ARE 'BUFR'.
C                --------------------------------------
      YBUFR=CHAR(IBUFR(1))//CHAR(IBUFR(2))//
     1      CHAR(IBUFR(3))//CHAR(IBUFR(4))
      IF(YBUFR.NE.'BUFR') THEN
         KERR = 1                               
         WRITE(0,'(A)')' BUEXS0 :'
         CALL BUERR(KERR)
         RETURN                                 
      END IF                                    
C
C*          1.4 UNPACK BUFR EDITION NUMBER (IT IS 8TH BYTE ).
C               ---------------------------------------------
 140  CONTINUE
C
      IWPT=56/NBPW+1
      IBPT=56-(IWPT-1)*NBPW
C
      CALL BUNPCK(NBPW,KBUFF(IWPT),KSEC0(3),IWPT,IBPT,8,KERR)
      IF(KERR.NE.0) RETURN
C
      IF(KSEC0(3).LE.1) GO TO 170
C                                                                       
C*          1.5 UNPACK TOTAL LENGTH OF BUFR MESSAGE.
C               ------------------------------------
 150  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC0(2),NWPT,NBPT,24,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.6 UNPACK BUFR EDITION NUMBER.
C               ---------------------------
 160  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC0(3),NWPT,NBPT,8,KERR)
      IF(KERR.NE.0) RETURN
C
C
C*          1.7 SET LENGTH OF SECTION 0.
C               ------------------------
 170  CONTINUE
C
      KSEC0(1)= 4
      IF(KSEC0(3).GT.1) KSEC0(1)= 8
C
C*          1.8 SET SUPPLEMENTARY INFORMATION.
C               ------------------------------
 180  CONTINUE
C
      KSUP (9)= 3
C
      RETURN                                                            
C     -----------------------------------------------------------------
C
      END                                                               
      SUBROUTINE BUEXS1( KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KERR)
C                                                                       
C**** *BUEXS1*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Expands section 1 of Bufr message. Saves expanded information
C     in the array KSEC1.
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUEXS1( KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC0*   -  array containing section 0 information
C                            KSEC0( 1)-- length of section 0 (bytes)
C                            KSEC0( 2)-- total length of Bufr message (bytes)
C                            KSEC0( 3)-- Bufr Edition number
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C                            KSEC1(16) - KSEC1(JSEC1) -- local ADP centre
C                                        information(PACKED FORM)
C               *KERR*    -  returned error code
C                                                                       
C     *METHOD.                                                          
C      -------                                                          
C                                                                       
C           NONE.                                                       
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPCK          -  unpack bit pattern    
C          BUNEXS         -  set word and bit pointers at the begining of
C                            next section
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       16/01/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C
      DIMENSION KBUFF(KBUFL),KSUP(JSUP),KSEC0(JSEC0),KSEC1(JSEC1)
      DIMENSION ISEC1(JSEC1)
C                                                                       
C     ------------------------------------------------------------------
C*          1.  EXPAND SECTION 1.                                       
C               ------------------                                      
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
C     SAVE WORD AND BIT POINTERS OF THR BRGINING OF SECTION 1. 
C
      IWPTB=NWPT
      IBPTB=NBPT
C                                                                       
C*          1.1 UNPACK LENGTH OF SECTION 1.                             
C               ----------------------------                            
 110  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(1),NWPT,NBPT,24,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.1.1 SET THE POINTERS NWPTB AND NBPTB.                     
C                 ---------------------------------                     
C                 TO BEGINING OF THE NEXT SECTION.                      
C                 --------------------------------                      
 1110 CONTINUE
C
      CALL BUNEXS(KSEC1(1))                                             
C                                                                       
C
C*          1.2  UNPACK BUFR EDITION NUMBER/MASTER TABLE USED.
C                ---------------------------------------------
 120  CONTINUE
C
      IF(KSEC0(3).LE.1) THEN
         CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(2),NWPT,NBPT, 8,KERR)
         IF(KERR.NE.0) RETURN
      ELSE
         CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(14),NWPT,NBPT, 8,KERR)
         IF(KERR.NE.0) RETURN
C
         KSEC1(2)=KSEC0(3)
      END IF
C                                                                       
C*          1.3  UNPACK ORIGINATING CENTRE.                             
C                --------------------------                             
 130  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(3),NWPT,NBPT,16,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.4  UNPACK UPDATE SEQUENCE NUMBER.                         
C                ------------------------------                         
 140  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(4),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.5  UNPACK INTEGER VALUE OF THE OCTET CONTAINING           
C                --------------------------------------------           
C                 FLAG BITS(ZERO IF SECTION TWO IS NOT PRESENT).        
C                 ----------------------------------------------        
 150  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(5),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C                            
C*          1.6  UNPACK *BUFR* MESSAGE TYPE.                            
C                ---------------------------                            
 160  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(6),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.7  UNPACK *BUFR* MESSAGE SUB-TYPE.                        
C                -------------------------------                        
 170  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(7),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C
C*          1.8  UNPACK LOCAL TABLE VERSION NUMBER OR
C                ------------------------------------
C                VERSION NUMBER OF MASTER TABLE USED.
C                 -----------------------------------
 180  CONTINUE
C
      IF(KSEC0(3).LE.1) THEN
         CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(8),NWPT,NBPT,16,KERR)
         IF(KERR.NE.0) RETURN
      ELSE
         CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(15),NWPT,NBPT, 8,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1( 8),NWPT,NBPT, 8,KERR)
         IF(KERR.NE.0) RETURN
      END IF
C                                                                       
C*          1.9  UNPACK YEAR.                                           
C                ------------                                           
 190  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(9),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          2. UNPACK MONTH.                                          
C              -------------                                          
 200  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(10),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          2.1 UNPACK DAY.                                            
C               -----------                                            
 210  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(11),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          2.2 UNPACK HOUR.                                           
C               ------------                                           
 220  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(12),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          2.3 UNPACK MINUTE.                                         
C               --------------                                         
 230  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC1(13),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C
C*          2.4 UNPACK LOCAL ADP CENTRE INFORMATION IF ANY.
C               -------------------------------------------
 240  CONTINUE
C
      IOFF=KSEC1(1)-17
      IF(IOFF.LE.JSEC1) THEN
         CALL BUNPKS(NBPW,KBUFF(NWPT),ISEC1(1),NWPT,NBPT,
     1   8,0,IOFF,KERR)
         IF(KERR.NE.0) RETURN
C
C                PACK LOCAL ADP CENTRE INFORMATION INTO KSE1(16) ONWARD.
C
         IW=16
         IBT=0
         CALL BUPKS(NBPW,KSEC1(IW),ISEC1,IW,IBT,8,0,IOFF,KERR)
         IF(KERR.NE.0) RETURN 
      ELSE
         IOFF=0
         KERR=4
         CALL BUERR(KERR)
         GO TO 300
      END IF
C
C                                                                       
C*          3.  SET SUPPLEMENTARY INFORMATION.
C               ------------------------------
 300  CONTINUE
C
      KSUP(1)=15+IOFF/(NBPW/8)+1
C
C     ------------------------------------------------------------------
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUEXS2( KBUFL,KBUFF,KSUP,KSEC1,KSEC2,KERR )
C                                                                       
C**** *BUEXS2*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Expands section 2 of Bufr message. Expanded data are
C     stored in the array KSEC2.  
C
C                                                                      
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUEXS2(KBUFL,KBUFF,KSUP,KSEC1,KSEC2,KERR)*
C                                     
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C                            KSEC1(16) - KSEC1(JSEC1) -- local ADP centre
C                                        information(PACKED FORM)
C               *KSEC2*   -  array containing section 2 information
C                            KSEC2( 1)-- length of section 2 (bytes)
C                            KSEC2( 2) to KSEC2(JSEC2) local ADP centre 
C                                         information(PACKED FORM)
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          The length of section 2 and pointers to the begining 
C     of section 3 are set. RDB key is then unpacked and all information 
C     stored into array KSEC2.
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPCK          - unpack bit pattern         
C          BUNEXS         - set word and bit pointers at the begining
C                           of next Bufr section.
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       17/01/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C                                                                       
      DIMENSION KBUFF(KBUFL)
      DIMENSION KSUP(JSUP),KSEC1(JSEC1),KSEC2(JSEC2)
      DIMENSION ISEC2(JSEC2)
C
C     ------------------------------------------------------------------
C*          1.  EXPAND SECTION 2.                                       
C               -----------------                                       
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
      DO 101 I=1,JSEC2
      KSEC2(I)=0
 101  CONTINUE
C
      IF( KSEC1(5).EQ.0) THEN                                           
          KSUP(2) = 1                                 
          KSEC2(1)= 0                                 
          RETURN                                      
      END IF                                         
C                                                                       
C*          1.1 SET THE POINTERS NWPT AND NBPT AT THE BEGINING OF SECTION.
C              -----------------------------------------------------------
 110  CONTINUE
C
      NWPT = NWPTB                                                      
      NBPT = NBPTB                                                      
C                                                                       
C*          1.2  UNPACK LENGTH OF SECTION 2.                            
C                ---------------------------                            
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC2(1),NWPT,NBPT,24,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.2.1  SET POINTERS NWPTB AND NBPTB TO THE                  
C                  -----------------------------------                  
C                  BEGINING OF THE NEXT SECTION.                        
C                  -----------------------------                        
      CALL BUNEXS(KSEC2(1))                                             
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),IDUMMY,NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C
C*          1.3  UNPACK LOCAL ADP CENTRE INFORMATION.
C                ------------------------------------
C
      IOFF=KSEC2(1)-4
C
      IF(IOFF.GT.JSEC2) THEN
         KERR=5
         CALL BUERR(KERR)
         write(0,*)"ksec2(1)=",ksec2(1)
         RETURN
      END IF
C
      CALL BUNPKS(NBPW,KBUFF(NWPT),ISEC2(1),NWPT,NBPT,
     1            8,0,IOFF,KERR)
      IF(KERR.NE.0) RETURN
C
C                PACK LOCAL ADP CENTRE INFORMATION INTO KSE2(2) ONWARD.
C
      IW=2
      IBT=0
      CALL BUPKS(NBPW,KSEC2(IW),ISEC2,IW,IBT,8,0,IOFF,KERR)
      IF(KERR.NE.0) RETURN 
C
C                SET LENGHT OF KSEC2
C
      KSUP(2)=IW
C     ------------------------------------------------------------------
 200  CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUGBTS( KSEC1,KERR )
C                                                                       
C**** *BUGBTS*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Load Bufr table B, D and C according to Edition and version
C     of Bufr code.
C       
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUGBTS(KSEC1,KERR)*
C
C        OUTPUT:
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C                            KSEC1(16) - KSEC1(JSEC1) -- local ADP centre
C                                        information(BYTE by BYTE)
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       17/01/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMTAB'
      INCLUDE 'COMTABC'
      INCLUDE 'COMCT'
      INCLUDE 'COMCTC' 
      INCLUDE 'COMROOT'
C               
      CHARACTER*256 YTAB ,YTAC ,YTAD
      CHARACTER*7  YTABB,YTABC,YTABD                                     
      CHARACTER    CTABBEN*64,CTABBU*24,CTEXT*64                           
      CHARACTER*256 CROOT
C                                                                       
      DIMENSION KSEC1(JSEC1)
C
      SAVE OFIRST
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          1.   GET BUFR TABLES/LOCAL BUFR TABLES.                     
C                ----------------------------------                     
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0) RETURN                                             
C                                                                       
C*          2.   SET UP BUFR TABLE FILE NAME.                           
C                ----------------------------                           
 200  CONTINUE                                                          
C                          
      IF(KSEC1(2).GT.1) THEN
C
C             BUFR EDITION 2 NAMING CONVENTION
C
C             BXXYYZZ , CXXYYZZ , DXXYYZZ
C
C             B   - BUFR TABLE 'B'
C             C   - BUFR TABLE 'C'
C             D   - BUFR TABLE 'D'
C             XX  - BUFR MASTER TABLE ( 00 for WMO standard tables )
C             YY  - VERSION NUMBER OF MASTER TABLE USED( CURRENTLY 2 )
C             ZZ  - VERSION NUMBER OF LOCAL TABLE USED
C
C
         IXX=KSEC1(14) 
         IYY=KSEC1(15)
         IZZ=KSEC1(08)
C                                                                       
         IF(OFIRST) THEN
            IF(IXX.EQ.NXXP.AND.IYY.EQ.NYYP.AND.IZZ.EQ.NZZP) RETURN
         END IF
C           
         OFIRST=.TRUE.
C
         NXXP=IXX
         NYYP=IYY
         NZZP=IZZ
C
         WRITE(YTABB,'(A1,I2.2,I2.2,I2.2)') 'B',IXX,IYY,IZZ
         WRITE(YTABC,'(A1,I2.2,I2.2,I2.2)') 'C',IXX,IYY,IZZ
         WRITE(YTABD,'(A1,I2.2,I2.2,I2.2)') 'D',IXX,IYY,IZZ
C
         PRINT*,' BUFR Tables to be loaded ',YTABB,',',YTABC,',',YTABD
C
      ELSE
C
         IBEN =KSEC1(2)     ! bufr edition number
         ILTVN=KSEC1(8)     ! local table version number
C                                                                       
         IF(OFIRST) THEN
             IF(IBEN.EQ.NBENP  ) RETURN                                        
             IF(ILTVN.EQ.NLTVNP) RETURN                                        
         END IF
C           
         OFIRST=.TRUE.
C
         NLTVNP=ILTVN                                                      
         NBENP =IBEN                                                       
C                       
         IF(ILTVN.EQ.0) THEN                                                
            WRITE(YTABB,'(A1,I6.6)') 'B',IBEN      
            WRITE(YTABC,'(A1,I6.6)') 'C',IBEN      
            WRITE(YTABD,'(A1,I6.6)') 'D',IBEN      
         ELSE                                                           
            WRITE(YTABB,'(A2,I5.5)') 'LB',ILTVN     
            WRITE(YTABC,'(A2,I5.5)') 'LC',ILTVN     
            WRITE(YTABD,'(A2,I5.5)') 'LD',ILTVN     
         END IF                                       
         PRINT*,' BUFR Tables to be loaded ',YTABB,',',YTABC,',',YTABD
      END IF
C                                                                       
C     ----------------------------------------------------------------  
C*          3. OPEN AND READ FILES CONTAINING BUFR TABLES.              
C              -------------------------------------------              
 300  CONTINUE                                                          
C
      I=INDEX(CROOT(1:),' ')
      IF(I.NE.0) I=I-1
C                                                                       
C*          3.1 READ BUFR TABLE B.                                      
C               ------------------                                      
 310  CONTINUE                                                          
C
      YTAB=CROOT(1:I)//YTABB
      II=I+7
C      WRITE(*,'(A/)')YTAB
C
      OPEN(UNIT=38,IOSTAT=IOS,ERR=410,FILE=YTAB(1:II),
     1     FORM='UNFORMATTED',
     2     ACCESS='SEQUENTIAL',
     3     READONLY,
     4     SHARED, 
     5     STATUS='OLD')
C                                                                       
      READ(38,ERR=400,IOSTAT=IOS) NTABBTR,CTABBEN,CTABBU,NTABBS,        
     1                            NTABBRV,NTABBDW,NTABP                        
C                                                                       
      CLOSE(UNIT=38,IOSTAT=IOS,ERR=420)                                 
C                                                                       
C*          3.2 READ BUFR TABLE C.                                      
C               ------------------ 
 320  CONTINUE                     
C                    
      YTAC=CROOT(1:I)//YTABC
C      WRITE(*,'(A/)')YTAC
C
      OPEN(UNIT=39,IOSTAT=IOS,ERR=510,FILE=YTAC(1:II),
     1     FORM='UNFORMATTED',
     2     ACCESS='SEQUENTIAL',
     3     READONLY,
     4     SHARED, 
     5     STATUS='OLD')
C     
      READ(39,ERR=500,IOSTAT=IOS)NREF   ,NSTART,NLEN,NCODNUM,           
     1                           NSTARTC,NLENC ,CTEXT                   
C                                                                       
      CLOSE(UNIT=39,IOSTAT=IOS,ERR=520)                                 
C                                                                       
C                                                                       
C*          3.3 READ BUFR TABLE D.                                      
C               ------------------                                      
 330  CONTINUE                                                          
C
      YTAD=CROOT(1:I)//YTABD
C      WRITE(*,'(A/)')YTAD
C
      OPEN(UNIT=40,IOSTAT=IOS,ERR=610,FILE=YTAD(1:II),
     1     FORM='UNFORMATTED',
     2     ACCESS='SEQUENTIAL',
     3     READONLY,
     4     SHARED, 
     5     STATUS='OLD')
C                                                                       
      READ(40,ERR=600,IOSTAT=IOS) NTABDTR,NTABDL,NTABDST,NTABDSQ        
C                                                                       
      CLOSE(UNIT=40,IOSTAT=IOS,ERR=620)                                 
C                                                                       
C                                                                       
      RETURN                                                            
C     ----------------------------------------------------------------  
 400  CONTINUE                                                          
C                                                                       
      KERR=6
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C                                                                       
 410  CONTINUE                                                          
C                                                                       
      KERR=9
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C                                                                       
 420  CONTINUE                                                          
C                                                                       
      KERR=10
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C     ----------------------------------------------------------------  
 500  CONTINUE                                                          
C                                                                       
      KERR=7
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C                                                                       
 510  CONTINUE                                                          
C                                                                       
      KERR=9
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C                                                                       
 520  CONTINUE                                                          
C                                                                       
      KERR=11
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C     ----------------------------------------------------------------- 
 600  CONTINUE                                                          
C                                                                       
      KERR=8
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C             
 610  CONTINUE
C             
      KERR=9
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C                             
 620  CONTINUE                
C                             
      KERR=12
      CALL BUERR(KERR)
      OFIRST=.FALSE.
      RETURN
C     ----------------------------------------------------------------- 
C                                                                       
      END                                                               
      SUBROUTINE BUNPCK(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KERR)
C                 
C**** *BUNPCK*                                                       
C                                                                    
C                                                                    
C     PURPOSE.                                                       
C     --------                                                       
C          Purpose of this routine is to unpack bit string of      
C     KSIZE bits, started at word KWPT of array KSOURC after       
C     skipping KBPT bits. Result is put into KDEST. At the end
C     pointers KWPT and KBPT are adjusted.                   
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUNPCK(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KERR)*
C                                                                       
C        INPUT :
C              *KBPW*      - number of bits in computer word
C              *KSOURC*    - source (continuous bit string of
C                            arbitrary length)
C              *KWPT*      - word pointer
C              *KBPT*      - bit pointer
C              *KSIZE*     - number of bits to be extracted
C        OUTPUT:
C              *KDEST*     - destination
C              *KERR*      - return error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE.                                                      
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C                                                                       
C          GBYTE     - unpack bit pattern  
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      DIMENSION KSOURC(*) 
C                                                                       
C     ------------------------------------------------------------------
C*          1.   EXTRACT BIT PATTERN.                                   
C                --------------------                                   
 100  CONTINUE                                                          
C
      IF(KSIZE.GT.KBPW) THEN
         KERR=13
         CALL BUERR(KERR)
         RETURN
      END IF
C
      CALL GBYTE(KSOURC,KDEST,KBPT,KSIZE)
C                                                                       
C     ------------------------------------------------------------------
C*          1.1  UPDATE WORD AND BIT POINTERS.                          
C                -----------------------------                          
 110  CONTINUE                                                          
C                                                                       
      KBPT = KBPT + KSIZE                                                 
C                                                                       
      IF(KBPT.GE.KBPW) THEN                                             
         KBPT= KBPT -  KBPW                        
         KWPT= KWPT +1
      END IF                                           
C
      RETURN
      END                                                               
      SUBROUTINE BUNPKS(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KSKIPB,K,KERR)
C                 
C**** *BUNPKS*
C                                                                    
C                                                                    
C     PURPOSE.                                                       
C     --------                                                       
C          Purpose of this routine is to unpack bit string of      
C     KSIZE bits, started at word KWPT of array KSOURC after       
C     skipping KBPT bits. Result is put into KDEST. At the end
C     pointers KWPT and KBPT are adjusted.                   
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUNPKS(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KSKIPB,K,KERR)*
C                                                                       
C        INPUT :
C            *KBPW*      - number of bits per computer word
C            *KSOURC*    - source (continuous bit string of
C                          arbitrary length)
C            *KWPT*      - word pointer
C            *KBPT*      - bit pointer
C            *KSIZE*     - number of bits to be extracted
C            *KSKIPB*    - number of bits to skip between elements
C            *K*         - iteration 
C        OUTPUT:
C            *KDEST*     - destination
C            *KERR*      - return error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE.                                                      
C                                                                     
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C                                                                       
C          GBYTES     - unpack bit pattern  
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      DIMENSION KSOURC(*),KDEST(*)
C                                                                       
C     ------------------------------------------------------------------
C*          1.   EXTRACT BIT PATTERN.                                   
C                --------------------                                   
 100  CONTINUE
C
      IF(KSIZE.GT.KBPW) THEN
         KERR=13
         CALL BUERR(KERR)
         RETURN
      END IF
C
      CALL GBYTES(KSOURC,KDEST,KBPT,KSIZE,KSKIPB,K)
C                                                                       
C     ------------------------------------------------------------------
C*          1.1  UPDATE WORD AND BIT POINTERS.                          
C                -----------------------------                          
 110  CONTINUE                                                          
C                                                                       
      KBPT = KBPT + K*(KSIZE+KSKIPB)
C                                                                       
      IF(KBPT.GE.KBPW) THEN                                             
         IW  = KBPT/ KBPW                              
         KBPT= KBPT - IW * KBPW                        
         KWPT= KWPT +IW                                
      END IF                                           
C
      RETURN
      END                                                               
      SUBROUTINE BUNEXS( KLEN)                                          
C                                                                       
C**** *BUNEXS*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           Sets word (NWPT) and bit(NBPT) pointers at the begining
C     of next section of Bufr message.   
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C            *CALL* *BUNEXS( KLEN)*                                     
C                                                                       
C        INPUT :
C                 *KLEN*     - length of section in bytes
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          Length of each section is added up. Word and bit pointers
C     pointing to the begining of next section are calculated. 
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C           NONE.                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       16/01/91.                  
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          1.   SET UP POINTERS AT THE BEGINING OF THE NEXT SECTION.  
C                -----------------------------------------------------  
 100  CONTINUE                                                          
C                                                                       
      IBIT = (NWPT - 1)*NBPW +NBPT +KLEN * 8 - 24                       
C                                                                       
      NWPTB = IBIT/NBPW + 1                                             
      NBPTB = IBIT - (NWPTB - 1)*NBPW                                   
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUEXS4(KBUFL,KBUFF,KSUP,KSEC3,KSEC4,KELEM,CNAMES,
     1                  CUNITS,KVALS,VALUES,CVALS,KERR)
C
C**** *BUEXS4*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Expand preliminary items and data of section 4 of Bufr message.
C
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUEXS4(KBUFL,KBUFF,KSUP,KSEC3,KSEC4,KELEM,CNAMES,
C                         CUNITS,KVALS,VALUES,CVALS,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *KVALS*   -  dimension of VALUES array
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC3*   -  array containing section 3 information
C                            KSEC3( 1)-- length of section 3 (bytes)
C                            KSEC3( 2)-- reserved
C                            KSEC3( 3)-- number of subsets
C                            KSEC3( 4)-- flag (data type,data compression)
C               *KSEC4*   -  array containing section 4 information
C                            KSEC4( 1)-- length of section 4 (bytes)
C                            KSEC4( 2)-- reserved
C               *CNAMES*  -  character array containing element names
C               *CUNITS*  -  character array containig units
C               *VALUES*  -  real array (expanded data values)               
C               *CVALS*   -  character array containing text
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C           NONE.                                                       
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPCK          -  unpack bit pattern    
C          BUNPKS          -  unpack bit pattern in repeated way,
C                             pointer adjustment
C          BUUNPS          -  unpack bit pattern in repeated way,
C                             no pointer adjustment
C          BUUNP           -  unpack bit pattern, no pointer adjustment
C          BUNEXS          -  set word and bit pointers at the begining of
C                             next section
C          
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       17/01/91.                  
C                                                                       
C                                                                      
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMWT'
      INCLUDE 'COMWTC'
      INCLUDE 'COMBEF'
C      INCLUDE 'COMCT'
C      INCLUDE 'COMCTC'
      INCLUDE 'COMRQ'
C                                                                       
      CHARACTER CTEXT*64 
      CHARACTER CNAMR*64,CUNIR*24                  
      CHARACTER CWTEN*64,CWTU*24 
C
      DIMENSION KBUFF(KBUFL)
      DIMENSION ICH(255),ILIST(JSUBS*9),IVALUES(JSUBS*9)
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
      CHARACTER*80 CVALS(KVALS)
C
      DIMENSION VALUES(KVALS)
      DIMENSION KSUP(JSUP),KSEC3(JSEC3),KSEC4(JSEC4)
      DIMENSION IMASK(8)
      DATA IMASK/1,2,4,8,16,32,64,128/
C                                                                       
C     ------------------------------------------------------------------
C*          1.  EXPAND PRELIMINARY ITEMS FROM SECTION4.                 
C               ---------------------------------------                 
 100  CONTINUE                                                          
C                                                                       
C                JCV - POINTER TO CVALS ARRAY                           
C                JWT - POINTER TO WORKING TABLE ARRAY                   
C                JNS - POINTER TO VALUES ARRAY FOR SUB-SETS.            
C                                                                       
      IF(KERR.NE.0) RETURN
C
      JCV = 0                                                           
      OREPF =.FALSE.                                                    
C
      IF(KELEM*N.GT.KVALS) THEN
         KERR=14
         CALL BUERR(KERR)
         WRITE(0,'(1H ,A,I4)') ' BUEXS4: Number of elements ',M
         write(0,*)" Maximum number of elements(KELEM) =",KELEM
         WRITE(0,'(1H ,A,I4)') ' BUEXS4: Number of subsets  ',N
         MN=KELEM*N
         WRITE(0,'(1H ,A,I7)') ' BUEXS4: Suggested value for KVALS ',MN
         WRITE(0,'(1H ,A,I4)') ' BUEXS4: Suggested value for KELEM ',M
         write(0,*)" Current value for KVALS =",KVALS
         RETURN
      END IF
C                                                                       
C*          1.1  SET THE POINTERS NWPT AND NBPT                         
C                ------------------------------                         
C                TO THE BEGINING OF THE SECTION.                        
C                -------------------------------                        
 110  CONTINUE                                                          
C                                                                       
      NWPT = NWPTB                                                      
      NBPT = NBPTB                                                      
      NWPTB4 = NWPTB                                                      
      NBPTB4 = NBPTB                                                      
C                                                                       
C*          1.2  UNPACK LENGTH OF SECTION 4.                            
C                ---------------------------                            
 120  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC4(1),NWPT,NBPT,24,KERR)
      IF(KERR.NE.0) RETURN
C
C*          1.3  SET THE POINTERS NWPTB AND NBPTB.                     
C                ---------------------------------                     
C                TO BEGINING OF THE NEXT SECTION.                      
C                --------------------------------                      
 130  CONTINUE
C
      CALL BUNEXS(KSEC4(1))                                             
C
C
C*          1.4  EXPAND RESERVED BYTE.
C                ---------------------
 140  CONTINUE
C
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC4(2),NWPT,NBPT, 8,KERR)
      IF(KERR.NE.0) RETURN
C
C     -----------------------------------------------------------------
C*          2. EXPAND DATA.
C              ------------
 200  CONTINUE
C
C
C*          2.1  CHECK IF DATA ARE COMRESSED.
C                ----------------------------
 210  CONTINUE
C
      IB=0
      IF((KSEC3(4).AND.IMASK(7)).NE.0) IB=1
C
      IF(IB.EQ.0) THEN
C
C     ------------------------------------------------------------------
C                                                                       
C*          3.  UNCOMPRESSED DATA.
C               ------------------
 300  CONTINUE                                                          
C                                                                       
         DO 301 JNS=1,N
         DO 302 JWT=1,M
C
         IF(NWTR(JWT).EQ.0) GO TO 302
C
         JWTJNS=JWT+(JNS-1)*KELEM
C
         NWPT=NWORDP(JWTJNS)
         NBPT=NBITP (JWTJNS)
C
         IF(NWTR(JWT).EQ.31011.OR.NWTR(JWT).EQ.31012) OREPF=.TRUE.
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C*          3.1 CHARACTER DATA ?                                        
C               ----------------                                        
 310  CONTINUE                                                          
C                                                                       
         IF(NWTEN(JWT).EQ.658367) THEN
C                                                                       
C*          3.2  OBTAIN CHARACTER DATA FROM DATA SECTION.               
C                ----------------------------------------               
 320  CONTINUE                                                          
C                                                                       
            IY=NWTDW(JWT)/8                                        
C                                                                       
            CALL  BUUNPS(NBPW,KBUFF(NWPT),ICH,NWPT,NBPT,8,0,IY,KERR)
            IF(KERR.NE.0) RETURN
C                                                                       
C*          3.3  MOVE CHARACTER DATA TO "CVALS".                        
C                -------------------------------                        
 330  CONTINUE                                                          
C                                                                       
            IYLINE=IY/80                                      
            IYOFF =IY-IYLINE*80                               
            JCVINC=IYLINE                                     
C        
            JCV = JCV + 1                                     
            JCVW= JCV                                         
C                                                                       
            IF(IYLINE.EQ.0) THEN                              
               DO 331 J=1,IY                  
               CVALS(JCVW)(J:J)=CHAR(ICH(J))
 331           CONTINUE                       
            ELSE                              
               DO 332 J=1,IYLINE              
               DO 333 JJ=1,80                 
               CVALS(JCVW)(JJ:JJ)= CHAR(ICH(JJ))
 333           CONTINUE                       
C                                                                       
               JCVW=JCVW+1                    
C                                                                       
 332           CONTINUE                       
C                                                                       
               DO 334 J=1,IYOFF               
               CVALS(JCVW)(J:J)= CHAR(ICH(J))
 334           CONTINUE                       
C                                                                       
            END IF                            
C                                                                       
C*          3.4  COMPUTE POINTER VALUES TO BE STORED IN "VALUES".       
C                ------------------------------------------------       
 340  CONTINUE                                                          
C                                                                       
            VALUES(JWTJNS) = JCV*1000 + IY                        
            JCV            = JCV + JCVINC                         
C                                                                       
            GO TO 302
C                                                                       
         END IF                                               
C                                                                       
C*          3.5  OBTAIN VALUE FROM DATA SECTION.                        
C                -------------------------------                        
 350  CONTINUE                                                          
C                                                                       
         CALL GBYTE(KBUFF(NWPT),IVAL,NBPT,NWTDW(JWT))
C                                                                       
C        CHECK IF DATA IS MISSING                                          
C     
         IVALUES(1)=IVAL
         IF(IVAL.NE.NMASK(NWTDW(JWT)))  IVALUES(1)=IVAL+NWTRV(JWT)
C                                                                       
C                                                                       
C     ----------------------------------------------------------------- 
C*          3.6  UPDATE THE ARRAY "VALUES".                            
C                --------------------------                             
 360  CONTINUE                                                          
C                                                                       
C                                                                       
C        CHECK IF DATA IS MISSING                           
C                                                                       
         VALUES(JWTJNS)=RVIND
         IF(IVAL.NE.NMASK(NWTDW(JWT))) THEN
            IF(NWTS(JWT).GT.0) THEN                                      
               VALUES(JWTJNS)= IVALUES(1)/10.**NWTS(JWT)        
            ELSE                                            
               IIWTS=IABS(NWTS(JWT))                             
               VALUES(JWTJNS)= IVALUES(1)*10.**IIWTS       
            END IF                                          
         END IF
C                                                                       
 302     CONTINUE
C
         KSUP(7)=JCV      
C
 301     CONTINUE
C
      ELSE
C     ------------------------------------------------------------------
C
C*          4.  COMPRESSED DATA.
C               ----------------
 400  CONTINUE                                                          
C                                                                       
C*          4.1 OBTAIN N VALUES BY EXPANSION.                           
C               -----------------------------                           
 410  CONTINUE                                                          
C
         DO 411 JWT=1,M
C
         NWPT=NWORDP(JWT)
         NBPT=NBITP (JWT)
C
C        CHECK IF CHARACTER DATA
C
         IF(NWTEN(JWT).EQ.658367) THEN
            IICH=NWTDW(JWT)/8
            CALL BUNPKS(NBPW,KBUFF(NWPT),IR0,NWPT,NBPT,8,0,IICH,KERR)
            IF(KERR.NE.0) RETURN
            CALL BUNPCK(NBPW,KBUFF(NWPT),IDWINC,NWPT,NBPT,6,KERR)
            IF(KERR.NE.0) RETURN
C
            ITOTAL=IDWINC*KSEC3(3)
            DO 413 J=1,ITOTAL
            ILIST(J)=0
 413        CONTINUE
C
            CALL BUNPKS(NBPW,KBUFF(NWPT),ILIST,NWPT,NBPT,
     1                  8,0,ITOTAL,KERR)
            IF(KERR.NE.0) RETURN
C
C           MOVE CHARACTER DATA TO "CVALS" ARRAY.
C
            IY=IDWINC
            IYLINE=IY/80
            IYOFF=IY-IYLINE*80
C
            JCVINC=IYLINE
            JJC=0
C
            DO 414 I=1,KSEC3(3)
C
            JCV=JCV+1
            JCVW=JCV
C
            IF(IYLINE.EQ.0) THEN
               DO 415 J=1,IY
               JJC=JJC+1
               CVALS(JCVW)(J:J)=CHAR(ILIST(JJC))
 415           CONTINUE
C
            ELSE
C
               DO 416 J=1,IYLINE
               DO 417 JJJ=1,80
               JJC=JJC+1
               CVALS(JCVW)(JJJ:JJJ)=CHAR(ILIST(JJC))
 417           CONTINUE
C
               JCVW=JCVW+1
 416           CONTINUE
C
               DO 418 J=1,IYOFF
               JJC=JJC+1
               CVALS(JCVW)(J:J)=CHAR(ILIST(JJC))
 418           CONTINUE
C
            END IF
C
C           COMPUTE POINTERS TO VALUES ARRAY
C
            JWTI=JWT+(I-1)*KELEM
            VALUES(JWTI)=JCV*1000+IY
            JCV         =JCV+JCVINC
C
 414        CONTINUE
C
            GO TO 411
C
         ELSE
            CALL BUNPCK(NBPW,KBUFF(NWPT),IR0,NWPT,NBPT,NWTDW(JWT),KERR)
            IF(KERR.NE.0) RETURN
            CALL BUNPCK(NBPW,KBUFF(NWPT),IDWINC,NWPT,NBPT,6,KERR)
            IF(KERR.NE.0) RETURN
            IF(IDWINC.GT.JBPW) THEN
               KERR=15
               WRITE(0,'(A)')' BUEXS4 :'
               CALL BUERR(KERR)
               RETURN
            END IF
         END IF
C
         DO 423 J=1,N
         ILIST(J)=0
 423     CONTINUE
C
         IF(IDWINC.NE.0) THEN
            CALL BUNPKS(NBPW,KBUFF(NWPT),ILIST,NWPT,NBPT,
     1                  IDWINC,0,N,KERR)
            IF(KERR.NE.0) RETURN
         END IF
C
         IF(NWTEN(JWT).NE.658367) THEN
            IF(IR0.EQ.NMASK(NWTDW(JWT))) THEN
               DO 425 J=1,N                                                
               IVALUES(J)=NMASK(NWTDW(JWT))
 425           CONTINUE                    
            ELSE                                
               IWTPR0=NWTRV(JWT)+IR0
C
               IF(IDWINC.EQ.0) THEN
                  DO 426 J=1,N
                  IVALUES(J)=IWTPR0
 426              CONTINUE
C
               ELSE               
                  DO 424 J=1,N         
                  IF(ILIST(J).EQ.NMASK(IDWINC)) THEN
                     IVALUES(J)=NMASK(NWTDW(JWT))
                  ELSE
                     IVALUES(J)= IWTPR0 + ILIST(J)
                  END IF
 424              CONTINUE                                                    
C
               END IF
            END IF
         END IF
C                                                                       
         DO 427 J=1,N   
C
         JWTJ=JWT+(J-1)*KELEM
C                                    
         VALUES(JWTJ)=RVIND
         IF(IVALUES(J).NE.NMASK(NWTDW(JWT))) THEN
            IF(NWTS(JWT).GT.0) THEN
               VALUES(JWTJ)= IVALUES(J)/10.**NWTS(JWT) 
            ELSE 
               IIWTS=IABS(NWTS(JWT))                    
               VALUES(JWTJ)= IVALUES(J)*10.**IIWTS 
            END IF                                  
         END IF
 427     CONTINUE  
C                                                                       
 411     CONTINUE
C         
         KSUP(7)=JCV
C
      END IF
C
C     ------------------------------------------------------------------
C*             5.  REPEAT ENTRIES IN CNAMES,CUNITS AND VALUES IF NEEDED.
C                  -----------------------------------------------------
 500  CONTINUE                                                          
C                                                                       
      IF(.NOT.OREPF) GO TO 600                                           
      IST=1                                                             
C                                                                       
C*             5.1 SEARCH CNAMES FOR DELAYED REPETITION FACTOR.         
C                  --------------------------------------------         
 510  CONTINUE                                                          
C                                                                       
      DO 511 J=IST,M                                                    
      IF(NWTR(J).EQ.31001.OR.NWTR(J).EQ.31002) GO TO 520
 511  CONTINUE                                                          
C                                                                       
      GO TO 600                                                         
C                                                                       
C*             5.2 GET REPETITION FACTOR FROM ARRAY "VALUES".           
C                  -------------------------------------------          
 520  CONTINUE                                                          
C                                                                       
      IREPF=VALUES(J)                                                 
      CNAMR=CNAMES(J+1)                                                 
      CUNIR=CUNITS(J+1)                                                 
      VALUR=VALUES(J+1)                                               
C                                                                       
C*             5.3 PUSH DOWN ENTRIES IN CNAMES,CUNITS,VALUES.           
C                  ------------------------------------------           
 530  CONTINUE                                                          
C                                                                       
      IREPM2=IREPF-2                                                    
C                                                                       
      DO 531 JA=M,J+2,-1
C
      CNAMES(JA+IREPM2)=CNAMES(JA)                                      
      CUNITS(JA+IREPM2)=CUNITS(JA)                                      
C                                                                        
      DO 932 JB=1,N                                                     
C
      JAJB=JA+(JB-1)*KELEM
C
      VALUES(JAJB+IREPM2)=VALUES(JAJB)                                
 932  CONTINUE                                                          
c
 531  CONTINUE                                                          
C                                                                       
C*             5.4 REPETITION.                                          
C                  -----------                                          
 540  CONTINUE                                                          
C                                                                       
      DO 541 JA=1,IREPF                                                 
      CNAMES(J+JA)=CNAMR                                                
      CUNITS(J+JA)=CUNIR                                                
C                                                                       
      DO 542 JB=1,N                                                     
C
      JAJB=JA+(JB-1)*KELEM
C
      VALUES(J+JAJB)=VALUR                                             
 542  CONTINUE                                                          
C
 541  CONTINUE                                                          
C                                                                       
C*             5.5 UPDATE M AND POINTER TO CONTINUE SEARCH.             
C                  ----------------------------------------             
 550  CONTINUE                                                          
C                                                                       
      M=M+IREPF-2                                                       
      IST=J+IREPF                                                       
C                                                                       
      GO TO 510                                                         
C                                                                       
C     -----------------------------------------------------------------
C*            6.  SET SUPPLEMENTARY INFORMATION.
C                 ------------------------------
C                                                                       
 600  CONTINUE                                                          
C                                                                       
      KSUP(4)= 2                                                        
C
C     ------------------------------------------------------------------
      RETURN                                                            
      END                                                               
      SUBROUTINE BUUNP(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KERR)
C                 
C**** *BUUNP*                                                       
C                                                                    
C                                                                    
C     PURPOSE.                                                       
C     --------                                                       
C          Purpose of this routine is to unpack bit string of      
C     KSIZE bits, started at word KWPT of array KSOURC after       
C     skipping KBPT bits. Result is put into KDEST. 
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUUNP(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KERR)*
C                                                                       
C        INPUT :
C                 *KBPW*      - number of bits in computer word
C                 *KSOURC*    - source (continuous bit string of
C                               arbitrary length)
C                 *KWPT*      - word pointer
C                 *KBPT*      - bit pointer
C                 *KSIZE*     - number of bits to be extracted
C        OUTPUT:
C                 *KDEST*     - destination
C                 *KERR*      - return error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE.                                                      
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C                                                                       
C          GBYTE     - unpack bit pattern  
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      DIMENSION KSOURC(*) 
C                                                                       
C     ------------------------------------------------------------------
C*          1.   EXTRACT BIT PATTERN.                                   
C                --------------------                                   
 100  CONTINUE                                                          
C
      IF(KERR.NE.0) RETURN
C
      IF(KSIZE.GT.KBPW) THEN
         KERR=13
         WRITE(0,'(A)')' BUUNP :'
         CALL BUERR(KERR)
         RETURN
      END IF
C
      CALL GBYTE(KSOURC,KDEST,KBPT,KSIZE)
C                                                                       
      RETURN
      END                                                               
      SUBROUTINE BUUNPS(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KSKIPB,K,KERR)
C                 
C**** *BUUNPS*
C                                                                    
C                                                                    
C     PURPOSE.                                                       
C     --------                                                       
C          Purpose of this routine is to unpack bit string of      
C     KSIZE bits, started at word KWPT of array KSOURC after       
C     skipping KBPT bits. Result is put into KDEST.
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUUNPS(KBPW,KSOURC,KDEST,KWPT,KBPT,KSIZE,KSKIPB,K,KERR)*
C                                                                       
C        INPUT :
C                 *KBPW*      - number of bits per computer word
C                 *KSOURC*    - source (continuous bit string of
C                               arbitrary length)
C                 *KWPT*      - word pointer
C                 *KBPT*      - bit pointer
C                 *KSIZE*     - number of bits to be extracted
C                 *KSKIPB*    - number of bits to skip between elements
C                 *K*         - iteration 
C        OUTPUT:
C                 *KDEST*     - destination
C                 *KERR*      - return error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE.                                                      
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C                                                                       
C          GBYTES     - unpack bit pattern  
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      DIMENSION KSOURC(*),KDEST(*)
C                                                                       
C     ------------------------------------------------------------------
C*          1.   EXTRACT BIT PATTERN.                                   
C                --------------------                                   
 100  CONTINUE                                                          
C
      IF(KSIZE.GT.KBPW) THEN
         KERR=13
         WRITE(0,'(A)')' BUUNPS :'
         CALL BUERR(KERR)
      END IF
C
      CALL GBYTES(KSOURC,KDEST,KBPT,KSIZE,KSKIPB,K)
C                                                                       
      RETURN
      END                                                               
      SUBROUTINE BUEXS3(KBUFL,KBUFF,KSUP,KSEC3,KELEM,CNAMES,CUNITS,KERR)
C
C**** *BUEXS3*                                                           
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C
C          Expand section 3 of Bufr message.
C
C
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUEXS3( KBUFL,KBUFF,KSUP,KSEC3,KELEM,CNAMES,CUNITS,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC3*   -  array containing section 3 information
C                            KSEC3( 1)-- length of section 3 (bytes)
C                            KSEC3( 2)-- reserved
C                            KSEC3( 3)-- number of subsets
C                            KSEC3( 4)-- flag (data type,data compression)
C               *CNAMES*  -  character array containing element names
C               *CUNITS*  -  character array containig units
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C      -------                                                          
C                                                                       
C          Expands list of data descriptors packed in section 3
C     of Bufr message. Working tables for further data decoding are set,
C     list of packed Bufr data descriptors and list of Bufr data descriptors
C     expanded according to table D reference are returned respectively.
C
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNEXS        - set word and bit pointers at the begining of
C                          next section
C          BUNPCK        - unpacks bit pattern
C          BUSRP         - solves replication problem
C          BUSTDR        - solves table D reference
C          BUPRCO        - process operator
C          BUUPWT        - updates working table
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMBEF'
      INCLUDE 'COMWT'
C
      CHARACTER CWTEN*64,CWTU*24                                
C
      INCLUDE 'COMWTC'
      INCLUDE 'COMRQ'
      INCLUDE 'COMREQ'
      INCLUDE 'COMEL'
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
C                                                                       
      DIMENSION ISTACK(JELEM)                                             
      DIMENSION KBUFF(KBUFL)
      DIMENSION IMASK(8)
C
      DIMENSION KSUP(JSUP),KSEC3(JSEC3)
C                       
      DATA IMASK/1,2,4,8,16,32,64,128/
C     ------------------------------------------------------------------
C
C*          1.   EXPAND PRELIMINARY ITEMS OF SECTION 3.                 
C                --------------------------------------                 
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
C*          1.1   SET THE POINTERS NWPT AND NBPT TO THE                 
C                 -------------------------------------                 
C                 BEGINING OF THE SECTION 3.                            
C                 --------------------------                            
 110  CONTINUE                                                          
C                                                                       
      NWPT = NWPTB                                                      
      NBPT = NBPTB                                                      
C                                                                       
C*          1.2   UNPACK LENGTH OF SECTION 3.                           
C                 ---------------------------                           
 120  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC3(1),NWPT,NBPT,24,KERR)
      IF(KERR.NE.0) RETURN
      if (ksec3(1).le.0) then
           kerr=91
           return
      endif
C                                                                       
C*          1.2.1  SET THE POINTERS NWPTB AND NBPTB TO                  
C                  -----------------------------------                  
C                  THE BEGINNING OF THE NEXT SECTION.                   
C                  ----------------------------------                   
      CALL BUNEXS(KSEC3(1))                                             
C                                                                       
C*          1.3    UNPACK ZERO BYTE AND PUT IT IN KSEC3(2).             
C                  ----------------------------------------             
 130  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC3(2),NWPT,NBPT,8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.4    UNPACK NUMBER OF DATA SUB-SETS.                      
C                  -------------------------------                      
 140  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC3(3),NWPT,NBPT,16,KERR)
      IF(KERR.NE.0) RETURN
      IF(KSEC3(3).GT.JSUBS) THEN
         KERR=16
         WRITE(0,'(A)')' BUEXS3 :'
         CALL BUERR(KERR)
         RETURN 
      END IF 
C
      N = KSEC3(3)
C
C                                                                       
C*          1.5    UNPACK INTEGER VALUE OF THE OCTET                    
C                  ---------------------------------                    
C                  CONTAINING FLAG BITS.                                 
C                  --------------------                                 
 150  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),KSEC3(4),NWPT,NBPT,8,KERR)
      IF(KERR.NE.0) RETURN
C
C     -----------------------------------------------------------------
C
C*          2.   EXPAND DATA DESCRIPTORS.                               
C                ------------------------                               
 200  CONTINUE                                                          
C                                                                       
C                                                                       
C*          2.1  CALCULATE EXPECTED NUMBER OF DATA DESCRIPTORS.         
C                ----------------------------------------------         
C                AND INITIALIZE NUMBER OF DATA VALUES PER SUB-SET.      
C                -------------------------------------------------      
 210  CONTINUE                                                          
C                                                                       
      J      = 0                                                        
      NWT    = 0                                                        
      JMAX   = ( KSEC3(1) - 7)/2                                        
C                                                                       
C*          2.2  UNPACK AND PUT DATA DESCRIPTORS IN STACK.              
C                -----------------------------------------              
 220  CONTINUE                                                          
C                                                                       
      DO 221 JJ=1,JMAX                                                  
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),IF,NWPT,NBPT,2,KERR)
      IF(KERR.NE.0) RETURN
      CALL BUNPCK(NBPW,KBUFF(NWPT),IX,NWPT,NBPT,6,KERR)
      IF(KERR.NE.0) RETURN
      CALL BUNPCK(NBPW,KBUFF(NWPT),IY,NWPT,NBPT,8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
      ISTACK(JJ)=IF*100000+IX*1000+IY                                   
      NTDLST (JJ)=ISTACK(JJ)      
C                    
 221  CONTINUE                                                          
C                                                                       
C*          2.2.1 CHECK IF IT IS SAME DATA DESCRIPTOR DESCRIPTION.     
C                 ------------------------------------------------     
C                 TO MAKE MORE EFFICIENT DATA DESCRIPTOR DESCRIPTION    
C                 EXPANSION, IN CASE THAT DELAYED REPLICATION FACTOR    
C                 IS NOT PRESENT AND DATA DESCRIPTORS ARE THE SAME,     
C                 PREVIOUS WORKING TABLE SHOULD BE USED. IT IS POSIBLE  
C                 AT THIS PLACE IN THE FUTURE TO MAKE MORE SOPHISTICATED
C                 CONTROL.                                              
C                                                                       
C                                                                       
      DO 222 JC=1,JMAX
C                                                                       
      IF(ISTACK(JC).NE.NSTACK(JC)) THEN                                 
C
C
         ODREPF=.FALSE.  
C                                                                       
C        SWAP CONTENT OF THE STACKS.  
C                               
         DO 223 JJC=1,JMAX
         NSTACK(JJC)=ISTACK(JJC)
 223     CONTINUE                     
C
         NTDLEN = JMAX
         M=0
         NOLD=N
         NFCM=0
         MREL=0
C
         GO TO 230
C
      END IF             
C                                                                       
 222  CONTINUE                                                          
C                                                                       
C*    CHECK IF THE SAME NUMBER OF DESCRIPTORS
C     AS IN A PREVIOUS MESSAGE
C
      IF(JMAX.NE.NTDLEN) THEN
         M=0
         NOLD=N
         NFCM=0
         MREL=0
         NTDLEN=JMAX
         GO TO 230
      END IF
C
C*    RETURN IF DELAYED REPLICATION FACTOR IS NOT PRESENT.              
C
      OB=.FALSE.
      IF((KSEC3(4).AND.IMASK(7)).NE.0) OB=.TRUE.
      IF(.NOT.ODREPF) THEN
         IF(OB) GO TO 270
         IF(N.GT.NOLD) THEN
            NOLD=N
            GO TO 270
         END IF
         GO TO 270        
      END IF
C
      M=0
      NOLD=N
      NFCM=0
      MREL=0
      NTDLEN=JMAX
C                                                                       
C     ------------------------------------------------------------------
C*          2.3  GET NEXT DESCRIPTOR FROM THE STACK.                    
C                -----------------------------------                    
 230  CONTINUE                                                          
C                                                                       
      J   = J + 1                                                       
      IF(J.GT.JMAX) GO TO 270
C                                                                       
      IDD = ISTACK(J)                                                   
      IF(IDD.EQ.0)  GO TO 230
C                                                                       
      IF = IDD/100000                                                   
C                                                                       
C     ------------------------------------------------------------------
C*          2.4  CHECK IF IT IS REPLICATION DESCRIPTOR.                 
C                --------------------------------------                 
 240  CONTINUE                                                          
C                                                                       
      IF( IF.EQ.1) THEN                                                 
C                                                                       
C*          2.4.1     SOLVE REPLICATION PROBLEM.                        
C                     --------------------------                        
C
         IF(IDD-(IDD/1000)*1000 .EQ.0 ) ODREPF=.TRUE.      
C
         CALL BUSRP(KBUFL,KBUFF,J,JMAX,IDD,ISTACK,
     1              KELEM,CNAMES,CUNITS,KERR)
         IF(KERR.NE.0) RETURN                              
         GO TO 230                                         
      END IF                                               
C                                                                       
C     ------------------------------------------------------------------
C*          2.5  CHECK IF IT IS NOT ELEMENT DESCRIPTOR.                 
C                --------------------------------------                 
 250  CONTINUE                                                          
C                                                                       
      IF( IF.NE.0) THEN                                                 
C                                                                       
C*          2.5.1 CHECK IF IT IS SEQUENCE DESCRIPTOR.                   
C                 -----------------------------------                   
         IF( IF.EQ.3) THEN                                      
C                                                                       
C*          2.5.2 REPLACE BY LIST OF DESCRIPTORS FROM TABLE *D.         
C                 ---------------------------------------------         
            CALL BUSTDR(J,JMAX,IDD,ISTACK,KERR)        
            IF(KERR.NE.0) RETURN                       
            GO TO 230                                  
         ELSE                                           
C
C*          2.5.3 PROCESS OPERATOR.                                     
C                 -----------------                                     
            CALL BUPRCO(KBUFL,KBUFF,J,IDD,ISTACK,
     1                  KELEM,CNAMES,CUNITS,KERR)
            IF(KERR.NE.0) RETURN                        
            GO TO 230                                   
         END IF                                         
      END IF                                               
C                                                                       
C     ------------------------------------------------------------------
C*          2.6  ELEMENT DESCRIPTOR, SO UPDATE WORKING TABLE.           
C                --------------------------------------------           
 260  CONTINUE                                                          
C                                                                       
      CALL BUUPWT(IDD,KELEM,CNAMES,CUNITS,KERR)
      IF(KERR.NE.0) RETURN                                              
C                                                                       
       GO TO 230                                                        
C     ------------------------------------------------------------------
C*          2.7  CHECK IF IT IS CORRESPONDING DATA.                     
C                ----------------------------------                     
 270  CONTINUE                                                          
C
C     CHECK FOR WORKING SPACE.
C   
      IF(JWORK/N.LT.KELEM) THEN
         KERR=17
         WRITE(0,'(1H )')
         WRITE(0,'(1H ,A)')   ' BUEXS3:'
         CALL BUERR(KERR)
         MN=KELEM*N
         write(0,'(1H ,A,i5,A,i5)')' n= ',n,' jwork= ', jwork
         WRITE(0,'(1H ,A,I6)')' Suggested value for JWORK ',MN
         RETURN
      END IF
C
      IF((KSEC3(4).AND.IMASK(7)).NE.0) THEN
C
C        COMPRESSED DATA
C
         CALL BURQC(KBUFL,KBUFF,KELEM,CNAMES,CUNITS,KSUP ,KSEC3,KERR)
         IF(KERR.NE.0) RETURN
      ELSE
C
C        UNCOMPRESSED DATA
C
         CALL BURQUC(KBUFL,KBUFF,KELEM,CNAMES,CUNITS,KSUP ,KSEC3,KERR)
         IF(KERR.NE.0) RETURN
C
      END IF
C                                                                       
C     ------------------------------------------------------------------
C
C*          3. COLLECT  SUPPLEMENTARY ITEMS.
C              -----------------------------
 300  CONTINUE 
C                                                         
      NTDEXL =M
      DO 301 I=1,NTDEXL
      NTDEXP(I)=NWTR(I)  
 301  CONTINUE
C
      KSUP(3)= 4                                                         
      KSUP(5)= M                                                         
      KSUP(6)= KSEC3(3)                                                  
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUSRP(KBUFL,KBUFF,KJ,KJ1,KDD,KSTACK,
     1                 KELEM,CNAMES,CUNITS,KERR)
C                                                                       
C**** *BUSRP*                                                           
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Resolve data descriptor replication problem.
C
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUSRP(KBUFL,KBUFF,KJ,KJ1,KDD,KSTACK,
C                        KELEM,CNAMES,CUNITS,KERR)*
C                                                                       
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *KDD*      - data descriptor
C        OUTPUT:
C               *KJ*       - pointer to kstack array
C               *KJ1*      - pointer to last element in kstack
C               *KSTACK*   - list of data descriptors
C               *CNAMES*   - list of element names
C               *CUNITS*   - list of units
C               *KERR*     - return code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C          NONE.                                                        
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUSTDR            - resolve table D reference
C          BUUPWT              - update working tables
C          GBYTE             - unpack bit pattern
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      CHARACTER CWTEN*64,CWTU*24
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMWT'
      INCLUDE 'COMWTC'
      INCLUDE 'COMBEF'
C
      DIMENSION KBUFF(KBUFL),ILIST(JELEM)
      DIMENSION KSTACK(*)
C                                          
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          1.   STORE K, NUMBER OF DESCRIPTORS TO BE REPLICATED.       
C                ------------------------------------------------       
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
      IF  = KDD / 100000                                                
      IDIF= KDD - IF * 100000                                           
      IX  = IDIF / 1000                                                 
      IY  = IDIF - IX * 1000                                            
      K   = IX                                                          
C                                                                       
C*          1.1   DELAYED REPLICATION ?                                 
C                 ---------------------                                 
 110  CONTINUE                                                          
C                                                                       
      IF( IY .NE. 0 ) THEN                                              
C                                                                       
C*          1.2   STORE NUMBER OF DESCRIPTORS, K, AND REPLICATION       
C                 -----------------------------------------------       
C                 FACTOR JR.                                            
C                 ----------                                            
 120  CONTINUE                                                          
C                                                                       
                         K  = IX                                        
                         JR = IY                                        
                         GO TO 500                                      
                      END IF                                            
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          2.   GET NEXT DESCRIPTOR.                                   
C                --------------------                                   
CENIVA
ceniva      PRINT *,'BUSRP CALLED WITH DELAYED REPLICATION FACTOR'

 200  CONTINUE                                                          
C                                                                       
      KJ =KJ + 1                                                        
      KDD= KSTACK(KJ)                                                   
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          2.1  REPLICATION FACTOR ?                                   
C                --------------------                                   
 210  CONTINUE                                                          
C                                                                       
      IF(KDD.NE.31001.AND.KDD.NE.31002 )THEN

         IF=KDD/100000

C                                                                       
C*          2.1.1  SEQUENCE DESCRIPTOR ?                                
C                  ---------------------                                
         IF( IF.EQ.3) THEN
C                                                                       
C*          2.1.1.1  SOLVE TABLE D REFERENCE.                           
C                    ------------------------                           
            CALL BUSTDR(KJ,KJ1,KDD,KSTACK,KERR)    
            IF(KERR.NE.0) THEN
               DO 252 IQ=1,JELEM
                  NSTACK(IQ)=0.
 252           CONTINUE
               RETURN 
            END IF                  
            GO TO 200 
         END IF 
C
         IF (IF.EQ.2) THEN
            CALL BUPRCO(KBUFL,KBUFF,KJU,KDD,KSTACK,KELEM,CNAMES,
     1                  CUNITS,KERR)
            IF (KERR.NE.0) THEN
                CALL BUERR(KERR)
                RETURN
            END IF
            GO TO 200
         END IF
C
         KERR=36
         CALL BUERR(KERR)
         WRITE(0,'(A,I8)')'Bad descriptor is ',KDD
         RETURN      
      ENDIF
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          3.  UPDATE WORKING TABLE.                                   
C               ---------------------                                   
 300  CONTINUE                                                          
C                                                                       
      CALL BUUPWT (KDD,KELEM,CNAMES,CUNITS,KERR)
      IF(KERR.NE.0) RETURN                                              
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          4.  LOCATE AND STORE JR, THE REPLICATION FACTOR FROM DATA.  
C               ------------------------------------------------------  
 400  CONTINUE                                                          
C                                                                       
      IBIT=0                                                            
C                                                                       
      DO 401 J=1,NWT-1                                                  
C                                                                       
      IBIT=IBIT+NWTDW(J)                                                
C                                                                       
 401  CONTINUE                                                          
C                                                                       
      IBIT = IBIT + 32 + NBPTB                                          
C                                                                       
      IWORD= IBIT/NBPW                                                  
      ISKIP= IBIT - IWORD*NBPW                                          
      IWORD= IWORD + NWPTB                                              
C                                                                       
      CALL GBYTE(KBUFF(IWORD),JR,ISKIP,NWTDW(NWT))                      
      IF(JR.EQ.0) THEN
         KERR=18
         WRITE(0,'(A)')'BUSRP :'
         CALL BUERR(KERR)
         WRITE(0,'(A)') (CWTEN(IH),IH=1,5)
         RETURN
      END IF        
C
      JRTK=JR*K+KJ1-K
      IF(JRTK.GT.JELEM) THEN
         KERR=19
         WRITE(0,'(A)')'BUSRP :'
         CALL BUERR(KERR)
         RETURN
      END IF    
C                                                                       
C     ------------------------------------------------------------------
C*          5.  GET NEXT K DESCRIPTORS.                                 
C               -----------------------                                 
 500  CONTINUE                                                          
C                                                                       
      DO 501 J=1,K                                                      
C                                                                       
      ILIST(J)=KSTACK(KJ+J)                                             
C                                                                       
 501  CONTINUE                                                          
C                                                                       
C     ------------------------------------------------------------------
C*          6.  ADD JR TIMES K DESCRIPTORS IN PLACE OF K                
C               ----------------------------------------                
C               DESCRIPTORS OBTAINED.                                   
C               ---------------------                                   
 600  CONTINUE                                                          
C                                                                       
C     ------------------------------------------------------------------
C*          6.1  PUSH DOWN DESCRIPTORS IN KSTACK FOR (JR-1)*K PLACES    
C                ---------------------------------------------------    
C                STARTING AT KJ1 AND ENDING AT KJ+K.                    
C                -----------------------------------                    
 610  CONTINUE                                                          
C                                                                       
      JRKM1=(JR-1)*K                                                    
C                                                                       
      DO 611 J=KJ1,KJ+K,-1                                              
C                                                                       
      KSTACK(J+JRKM1)=KSTACK(J)                                         
C                                                                       
 611  CONTINUE                                                          
C                                                                       
C*          6.2  INSERT LIST IN THE STACK.                              
C                -------------------------                              
 620  CONTINUE                                                          
C                                                                       
      DO 622 J=1,JR                                                     
C                                                                       
      KJJM1K=KJ+(J-1)*K                                                 
C                                                                       
      DO 623 J1=1,K                                                     
C                                                                       
      KSTACK(KJJM1K+J1)=ILIST(J1)                                       
C                                                                       
 623  CONTINUE                                                          
 622  CONTINUE                                                          
C                                                                       
C     ------------------------------------------------------------------
C*          6.3  ADJUST DESCRIPTOR COUNT FOR LIST LENGTH.               
C                ----------------------------------------               
 630  CONTINUE                                                          
C                                                                       
      KJ1 = KJ1  + (JR-1)*K                                             
C                                                                       
C     ------------------------------------------------------------------
C*          6.4  ADJUST NUMBER OF DATA DESCRIPTORS NOT PRESENT.
C                ----------------------------------------------               
 640  CONTINUE                                                          
C                                                                       
      IF(N221.NE.0)  N221= KJ1  - KJ + 1
C                                                                       
C     ------------------------------------------------------------------
 700  CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUSTDR(KJ,KJ1,KDD,KSTACK,KERR)                         
C                                                                       
C**** *BUSTDR*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Solve Bufr table D reference.
C                                                                       
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUSTDR(KJ,KJ1,KDD,KSTACK,KERR)*                      
C                                                                       
C        INPUT :
C                 *KDD*      - data descriptor
C        OUTPUT:
C                 *KJ*       - pointer to kstack array
C                 *KJ1*      - pointer to last element in kstack
C                 *KSTACK*   - list of data descriptors
C                 *KERR*     - return error code
C                                                                       
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C           NONE.                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C           NONE.                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMTAB'
      INCLUDE 'COMTABC'
C
      CHARACTER CTABBEN*64,CTABBU*24
C
      DIMENSION ILIST(JELEM),KSTACK(*)                                    
C                                                                       
C     ------------------------------------------------------------------
C
C*          1.   OBTAIN LIST OF DESCRIPTORS FROM BUFR TABLE D.        
C                ---------------------------------------------
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
      DO 110 J=1,JTAB
C                                                                       
      IF(KDD.EQ.NTABDTR(J)) THEN                                        
         I=J                                      
         GO TO 120                                
      END IF                                      
C                                                                       
 110  CONTINUE                                                          
C                                                                       
      KERR=20
      WRITE(0,'(A)')' BUESTDR :'
      CALL BUERR(KERR)
      write(0,*)"KDD=",kdd
      RETURN
C                                                                       
 120  CONTINUE                                                          
C                                                                       
      J1=NTABDST(I)                                                     
      J2=NTABDL (I)                                                     
      J3=0                                                              
C                                                                       
      DO 121 J=J1,J1+J2-1                                               
C                                                                       
      J3 = J3 +1                                                        
      ILIST(J3) = NTABDSQ(J)                                            
C                                                                       
 121  CONTINUE                                                          
C                                                                       
C     ------------------------------------------------------------------
C*          2.  PUSH DOWN DATA DESCRIPTION DESCRIPTORS                  
C               --------------------------------------                  
C               TO MAKE ROOM FOR LIST.                                  
C               ----------------------                                  
 200  CONTINUE                                                          
C                                                                       
      J2M1=J2-1                                                         
C                                                                       
      DO 210 J=KJ1,KJ+1,-1                                              
C                                                                       
      KSTACK(J+J2M1) = KSTACK(J)                                        
C                                                                       
 210  CONTINUE                                                          
C                                                                       
C     ------------------------------------------------------------------
C*          3.  INSERT LIST IN PLACE OF SEQUENCE DESCRIPTORS.           
C               ---------------------------------------------           
 300  CONTINUE                                                          
C                                                                       
      KJM1=KJ-1                                                         
C                                                                       
      DO 310 J=1,J3                                                     
C                                                                       
      KSTACK(KJM1+J)= ILIST(J)                                          
C                                                                       
 310  CONTINUE                                                          
C                                                                       
C     ------------------------------------------------------------------
C*          4.  ADJUST DESCRIPTOR COUNT FOR LIST LENGTH.                
C               ----------------------------------------                
 400  CONTINUE                                                          
C                                                                       
      KJ  = KJ  - 1                                                     
      KJ1 = KJ1 +J3 -1                                                  
C     ------------------------------------------------------------------
C*          4.1  ADJUST NUMBER OF DATA DESCRIPTORS NOT PRESENT.
C                ----------------------------------------------               
 610  CONTINUE                                                          
C                                                                       
      IF(N221.NE.0)  N221= KJ1  - KJ + 1
C     ----------------------------------------------------------------- 
 500  CONTINUE                                                          
C                                                                       
      RETURN                                                            
C                                                                       
 9901 FORMAT(1H ,' BUSTDR : table D reference not found, error=',I2)      
C                                                                    
      END                                                               
      
      SUBROUTINE BUPRCO(KBUFL,KBUFF,KJ,KDD,KSTACK,
     1                  KELEM,CNAMES,CUNITS,KERR)
C                                                                       
C**** *BUPRCO*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C                                                                       
C          Process Bufr operator.
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUPRCO(KBUFL,KBUFF,KJ,KDD,KSTACK,
C                         KELEM,CNAMES,CUNITS,KERR)*
C                                                                       
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KJ*     - Pointer t array kstack
C               *KDD*    - data descriptor
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C        OUTPUT:
C               *KSTACK* - list of descriptors
C               *CNAMES* - list of element names
C               *CUNITS* - list of units
C               *KERR*   - return error code
C                                                                       
C     *METHOD.                                                          
C      -------                                                          
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUUATB           - update augmented table b
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMWT'
      INCLUDE 'COMWTC'
      INCLUDE 'COMBEF'  
C
      CHARACTER CWTEN*64,CWTU*24                                        
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
C
      DIMENSION KBUFF(KBUFL)
      DIMENSION KSTACK(*)
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          1.   DETERMINE *F *X AND *Y.                                
C                -----------------------                                
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
      IF  = KDD / 100000                                                
      IDIF= KDD - IF * 100000                                           
      IX  = IDIF / 1000                                                 
      IY  = IDIF - IX * 1000                                            
C                                                                       
      IF( IF.NE.2 ) THEN                                                
         KERR=21
         WRITE(0,'(A)')' BUPRCO : '
         CALL BUERR(KERR)
         GO TO 400                                        
      END IF                                              
C                                                                       
C*          1.1   CHANGE DATA WIDTH ?                                   
C                 -------------------                                   
 110  CONTINUE                                                          
C                                                                       
      IF(IX.EQ.1) THEN                                                  
         NDWINC= IY-128                                     
         IF(IY.EQ.0) NDWINC=0                               
         GO TO 400                                          
      END IF                                                
C                                                                       
C*          1.2   CHANGE SCALE ?                                        
C                 --------------                                        
 120  CONTINUE                                                          
C                                                                       
      IF(IX.EQ.2) THEN                                                  
C                                                                       
C*          1.2.1  UPDATE SCALE MULTIPLIER.                             
C                  ------------------------                             
C                                                                       
         NSCAM=IY-128                                       
         IF(IY.EQ.0) NSCAM=0                                
         GO TO 400                                          
      END IF                                                
C                                                                       
C*          1.3  CHANGE REFERENCE VALUE ?                              
C                ------------------------
 130  CONTINUE                                                          
C                                                                       
      IF(IX.EQ.3) THEN                                                  
C                                                                       
C*          1.3.1  UPDATE AUGMENTED TABLE B.                           
C                  -------------------------                           
         CALL BUUATB(KBUFL,KBUFF,KJ,IY,KSTACK,KERR)                      
         GO TO 400                                          
      END IF                                                
C                                                                       
C*          1.4   ADD ASSOCIATED FIELD ?                                
C                 ----------------------                                
 140  CONTINUE                                                          
C                                                                       
      IF(IX.EQ.4) THEN                                                  
C                                                                       
C*          1.4.1   UPDATE ASSOCIATED FIELD WIDTH.                      
C                   ------------------------------                      
         NAFDW= IY                                          
         IF(IY.EQ.0) NAFDW=0                                
         GO TO 400                                          
      END IF                                                
C                                                                       
C*          1.5   SIGNIFY CHARACTER ?                                   
C                 -------------------                                   
 150  CONTINUE                                                          
C                                                                       
      IF(IX.EQ.5) THEN                                                  
C                                                                       
C*          1.5.1  ADD SPECIAL ENTRY TO WORKING TABLE.                  
C                  -----------------------------------                  
         NWT = NWT + 1                                      
         CWTEN(NWT)='CHARACTERS'                            
         CWTU (NWT)=' '               
         NWTR (NWT)= 0
         NWTDW(NWT)= IY * 8                                 
         NWTEN(NWT)=658367
         GO TO 400                                          
      END IF                                                
C                                    
C*          1.6   DATA NOT PRESENT.
C                 -----------------
 160  CONTINUE                       
C                                    
      IF(IX.EQ.63) THEN
         N263=IY
         GO TO 400                             
      END IF                                   
C
C                                    
C*          1.7   QUALITY INFORMATION FOLLOWS.
C                 ----------------------------
 170  CONTINUE                       
C                                    
      IF(IX.EQ.62) THEN
C                                          
         IF(IY.EQ.0) GO TO 400
         N262=IY
C              
C*          1.7.1  ADD SPECIAL ENTRY TO WORKING TABLE.    
C                  -----------------------------------    
         NWT = NWT + 1                        
         CWTEN(NWT)='QUALITY INFORMATION'
         CWTU (NWT)='                       ' 
         NWTDW(NWT)= 0
         NWTR (NWT)= 0
         NWTEN(NWT)= 0
         IF(MREL.EQ.0) MREL=NWT-1
         M=M+1
         IF(M.GT.KELEM) THEN
            KERR=25
            WRITE(0,'(1H ,A)') 'BUPRCO:'
            CALL BUERR(KERR)
            RETURN
         END IF
         CNAMES(M)='QUALITY INFORMATION'
         CUNITS(M)=' '
         GO TO 400                            
      END IF                                  
C
C                                    
C*          1.8   SUBSTITUTED VALUES.
C                 -------------------
 180  CONTINUE                       
C                                    
      IF(IX.EQ.61) THEN
         IF(IY.EQ.0) GO TO 400
         N261=IY
C              
C*          1.8.1  ADD SPECIAL ENTRY TO WORKING TABLE.    
C                  -----------------------------------    
         NWT = NWT + 1                        
         CWTEN(NWT)='SUBSTITUTED VALUES'
         CWTU (NWT)=' ' 
         NWTDW(NWT)= 0
         NWTR (NWT)= 0
         NWTEN(NWT)= 0
         MSUB=NWT-1
         M=M+1
         IF(M.GT.KELEM) THEN
            KERR=25
            WRITE(0,'(1H ,A)') 'BUPRCO:'
            CALL BUERR(KERR)
            RETURN
         END IF
         CNAMES(M)='SUBSTITUTED VALUES'
         CUNITS(M)=' '
         GO TO 400                            
      END IF                                  
C
C
C*          2.  PROCESSING NEW OPERATORS.
C               -------------------------
 200  CONTINUE
C
C                                    
C*          2.1   DATA NOT PRESENT.
C                 -----------------
 210  CONTINUE
C
      IF(IX.EQ.21) THEN
         N221=IY
         GO TO 400                             
      END IF                                   
C
C                                    
C*          2.2   QUALITY INFORMATION FOLLOWS.
C                 ----------------------------
 220  CONTINUE
C
      IF(IX.EQ.22) THEN
C
C*          1.7.2  ADD SPECIAL ENTRY TO WORKING TABLE.    
C                  -----------------------------------    
         NWT = NWT + 1                        
         CWTEN(NWT)='QUALITY INFORMATION'
         CWTU (NWT)=' ' 
         NWTDW(NWT)= 0
         NWTR (NWT)= 0
         NWTEN(NWT)= 0
         IF(MREL.EQ.0) MREL=NWT-1
         M=M+1
         IF(M.GT.KELEM) THEN
            KERR=25
            WRITE(0,'(1H ,A)') 'BUPRCO:'
            CALL BUERR(KERR)
            RETURN
         END IF
         CNAMES(M)='QUALITY INFORMATION'
         CUNITS(M)=' '
         GO TO 400                            
      END IF                                  
C
C                                    
C*          2.3   SUBSTITUTED VALUES FOLLOWS.
C                 ---------------------------
 230  CONTINUE
C
      IF(IX.EQ.23) THEN
         IF(IY.EQ.0) THEN
            NWT=NWT+1
            CWTEN(NWT)='SUBSTITUTED VALUES'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)= 223000
            NWTEN(NWT)= 0
            IF(MREL.EQ.0) MREL=NWT-1
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M)='SUBSTITUTED VALUES'
            CUNITS(M)=' '
            GO TO 400
         ELSE
            NWT=NWT+1
            CWTEN(NWT)='SUBSTITUTED VALUE MARKER'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)=KDD
            NWTEN(NWT)= 0
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M)='SUBSTITUTED VALUE MARKER'
            CUNITS(M)=' '
            GO TO 400
         END IF
      END IF
C
C                                    
C*          2.4   FIRST ORDER STATISTICS FOLLOWS.
C                 -------------------------------
 240  CONTINUE
C
      IF(IX.EQ.24) THEN
         IF(IY.EQ.0) THEN
            NWT=NWT+1
            CWTEN(NWT)='FIRST ORDER STATISTICS FOLLOW'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)= 224000
            NWTEN(NWT)=0
            IF(MREL.EQ.0) MREL=NWT-1
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M)='FIRST ORDER STATISTICS FOLLOW'
            CUNITS(M)=' '
            GO TO 400
         ELSE
            NWT=NWT+1
            CWTEN(NWT)='FIRST ORDER STATISTICS VALUE MARKER'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)=KDD
            NWTEN(NWT)= 0
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M)='FIRST ORDER STATISTICS VALUE MARKER'
            CUNITS(M)=' '
            GO TO 400
         END IF
      END IF
C                                    
C*          2.5   DIFFERENCE STATISTICAL VALUES FOLLOW.
C                 -------------------------------------
 250  CONTINUE
C
      IF(IX.EQ.25) THEN
         IF(IY.EQ.0) THEN
            NWT=NWT+1
            CWTEN(NWT)='DIFFERENCE STATISTICAL VALUES FOLLOW'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)= 225000
            NWTEN(NWT)= 0
            IF(MREL.EQ.0) MREL=NWT-1
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M)='DIFFERENCE STATISTICAL VALUES FOLLOW'
            CUNITS(M)=' '
            GO TO 400
         ELSE
            NWT=NWT+1
            CWTEN(NWT)='DIFFERENCE STATISTICS VALUE MARKER'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)=KDD
            NWTEN(NWT)= 0
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M) ='DIFFERENCE STATISTICS VALUE MARKER'
            CUNITS(M) =' '
            GO TO 400
         END IF
      END IF
C                                                                       
C                                    
C*          2.6   REPLACED/RETAINED VALUES FOLLOWS.
C                 ---------------------------------
 260  CONTINUE
C
      IF(IX.EQ.32) THEN
         IF(IY.EQ.0) THEN
            NWT=NWT+1
            CWTEN(NWT)='REPLACE/RETAINED VALUES FOLLOW'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)= 232000
            NWTEN(NWT)= 0
            IF(MREL.EQ.0) MREL=NWT-1
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M)='REPLACE/RETAINED VALUES FOLLOW'
            CUNITS(M)=' '
            GO TO 400
         ELSE
            NWT=NWT+1
            CWTEN(NWT)='REPLACE/RETAINED VALUE MARKER'
            CWTU (NWT)=' '
            NWTDW(NWT)=0
            NWTR (NWT)=KDD
            NWTEN(NWT)= 0
            M=M+1
            IF(M.GT.KELEM) THEN
               KERR=25
               WRITE(0,'(1H ,A)') 'BUPRCO:'
               CALL BUERR(KERR)
               RETURN
            END IF
            CNAMES(M) ='REPLACE/RETAINED VALUE MARKER'
            CUNITS(M) =' '
            GO TO 400
         END IF
      END IF
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
 300  CONTINUE                                                          
C                                                                       
      KERR=22
      WRITE(0,'(1H ,A)') 'BUPRCO:'
      CALL BUERR(KERR)
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
 400  CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUUATB(KBUFL,KBUFF,KJ,KY,KSTACK,KERR)
C        
C**** *BUUATB*                                                           
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Update augmented Bufr table B.
C
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUUATB(KBUFL,KBUFF,KJ,KY,KSTACK,KERR)*
C                                                                       
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KJ*     - pointer to kstack array
C               *KY*     - operand of the data descriptor operator
C        OUTPUT:
C               *KSTACK* - list of elements
C               *KERR*   - return error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          GBYTE        - pack bit pattern
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       04/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      CHARACTER CATBEN*64,CWTEN*64,CTABBEN*64                           
      CHARACTER CATBU*24,CWTU*24,CTABBU*24                              
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMATB'
      INCLUDE 'COMATBC'
      INCLUDE 'COMTAB'
      INCLUDE 'COMTABC'
      INCLUDE 'COMWT'
      INCLUDE 'COMWTC'
C                                                                       
      DIMENSION KBUFF(KBUFL)
      DIMENSION KSTACK(*)
C     ------------------------------------------------------------------
C
C*          1.   UPDATE AUGMENTED TABLE B .                             
C                --------------------------                             
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
C*          1.1 *Y = 0 ?                                                
C                -----                                                  
 110  CONTINUE                                                          
C                                                                       
      IF( KY.EQ.0) THEN                                                 
C                                                                       
C*          1.1.1 CLEAR AUGMENTED TABLE B.                              
C                 ------------------------                              
         NJA= 0                                            
C                                                                       
         DO 111 J=1,JTAB
C                                                                       
         NATBTR(J)= 0                                      
         NATBS (J)= 0                                      
         NATBRV(J)= 0                                      
         NATBDW(J)= 0                                      
         CATBEN(J)=' '                             
         CATBU (J)=' '              
C                                                                       
 111     CONTINUE                                          
C                                                                       
         GO TO 300                                         
      END IF                                               
C                                                                       
C*          1.2  GET NEXT DESCRIPTOR FROM STACK.                        
C                -------------------------------                        
 120  CONTINUE                                                          
C                                                                       
      KJ=KJ + 1                                                         
      KDD = KSTACK(KJ)                                                  
C                                                                       
C*          1.3  ELEMENT DESCRIPTOR  ?                                  
C                ---------------------                                  
 130  CONTINUE                                                          
C                                                                       
      IF  = KDD /100000                                                 
      IDIF= KDD -IF*100000                                              
      IX  = IDIF/1000                                                   
      IY  = IDIF-IX*1000                                                
C                                                                       
      IF(IF.EQ.0) THEN                                                  
C                                                                       
C*          1.3.1 ADD SPECIAL ENTRY TO WORKING TABLE.                   
C                 -----------------------------------                   
         NWT = NWT + 1                                      
         CWTEN(NWT)='REFERENCE VALUE'                       
         CWTU (NWT)='                        '              
         NWTDW(NWT)= KY                                     
C                                                                       
C*          1.3.2 ADD ENTRY TO AUGMENTED TABLE B .                      
C                 --------------------------------                      
         DO 131 J=1,JTAB
C                                                                       
         IF(NTABBTR(J).EQ.KDD) THEN                         
            I=J                       
            GO TO 133                 
         END IF                       
C                                                                       
 131     CONTINUE                                           
C                                                                       
         KERR=23
         WRITE(0,'(1H ,A)') 'BUUATB :'
         CALL BUERR(KERR)
C
         GO TO 300                                          
C                                                                       
 133  CONTINUE                                                          
C                                                                       
         NJA=NJA + 1                                        
C                                                                       
         NATBTR(NJA)=NTABBTR(I)                             
         NATBS (NJA)=NTABBS (I)                             
         NATBRV(NJA)=NTABBRV(I)                             
         NATBDW(NJA)=NTABBDW(I)                             
         CATBEN(NJA)=CTABBEN(I)                             
         CATBU (NJA)=CTABBU (I)                             
C                                                                       
C*          1.3.3 COMPLITE ENTRY WITH NEW REFERENCE VALUE               
C                 ----------------------------------------              
C                 FROM DATA SECTION.                                    
C                 ------------------                                    
C                                                                       
         IBIT=0                                             
C                                                                       
         DO 134 J=1,NWT-1                                   
C        
         IBIT =IBIT + NWTDW(J)                              
C                                                                       
 134     CONTINUE                                           
C                                                                       
         IBIT = IBIT +32 + NBPTB                            
         IWORD= IBIT/NBPW                                   
         ISKIP= IBIT - IWORD*NBPW                           
         IWORD= IWORD + NWPTB                               
C                                                                       
         CALL GBYTE(KBUFF(IWORD),NATBRV(NJA),ISKIP,KY)      
C                                                                       
         GO TO 120                                          
      END IF                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*           1.4   CHANGE REFERENCE VALUE ?                             
C                  ------------------------                             
 140  CONTINUE                                                          
C                                                                       
      IF( IF.EQ.2.AND.IX.EQ.3) THEN
         IF(IY.EQ.255) GO TO 300               
      END IF                                   
C     ------------------------------------------------------------------
 200  CONTINUE                                                          
C                                                                       
      KERR=23
      WRITE(0,'(1H ,A)') 'BUUATB :'
      CALL BUERR(KERR)
C                                                                       
C     ------------------------------------------------------------------
 300  CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUUPWT(KDD,KELEM,CNAMES,CUNITS,KERR)
C                                                                       
C**** *BUUPWT*                                                            
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Updates working tables setting element name,unit,scale,
C     reference value and data width.
C                                                                       
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUUPWT(KDD,KELEM,CNAMES,CUNITS,KERR)*
C                                                                       
C        INPUT :
C               *KDD*     -  data descriptor
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C        OUTPUT:
C               *CNAMES*  -  list of element names
C               *CUNITS*  -  list of units
C               *KERR*    -  return error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWT'
      INCLUDE 'COMWTC'
      INCLUDE 'COMTAB'
      INCLUDE 'COMTABC'
      INCLUDE 'COMATB'
      INCLUDE 'COMATBC'
      INCLUDE 'COMBEF'
C
      CHARACTER CATBEN*64,CWTEN*64,CTABBEN*64,YWTEN*64 
      CHARACTER CATBU*24,CWTU*24,CTABBU*24,YWTU*24
      CHARACTER YCH6*6
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
C                                                                       
C     ------------------------------------------------------------------
C*          1.   UPDATE WORKING TABLE.                                  
C                ---------------------                                  
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C
      ICLASS=KDD/1000
C                                                                       
C*          1.1  ASSOCIATED FIELD ?                                     
C                ------------------                                     
 110  CONTINUE                                                          
C                                                                       
      IF(NAFDW.EQ.0) GO TO 140                                          
C                                                                       
C*          1.2  UNITS ELEMENT DESCRIPTOR ?                             
C                --------------------------                             
 120  CONTINUE                                                          
C                                                                       
      DO 121 J=NTABP(ICLASS),NTABP(ICLASS+1)
C                                                                       
      IF(NTABBTR(J).EQ.KDD) THEN                                        
         I=J                                      
         GO TO 122                                
      END IF                                      
C                                                                       
 121  CONTINUE                                                          
C                                                                       
      KERR=24
      WRITE(0,'(1H ,A)') 'BUUPWT :'
      CALL BUERR(KERR)
      write(0,*)"KDD=",kdd
      RETURN                                                            
C                                                                       
 122  CONTINUE                                                          
C                                                                       
      IF(CTABBU(I)(1:3).EQ.'NUM' ) GO TO 140                           
      IF(CTABBU(I)(1:4).EQ.'CODE') GO TO 140                           
      IF(CTABBU(I)(1:4).EQ.'FLAG') GO TO 140                           
C                                                                       
C*          1.3   ADD SPECIAL ENTRY TO WORKING TABLE.                   
C                 -----------------------------------                   
 130  CONTINUE                                                          
C                                                                       
      NWT=NWT+1                                                         
      CWTEN(NWT)='ASSOCIATED FIELD'                                     
      CWTU (NWT)=' '                       
      NWTDW(NWT)=NAFDW
      NWTR (NWT)=999999
      NWTEN(NWT)= 0
      NWTS (NWT)= 0
C
C*                UPDATE CNAMES AND CUNITS
C
      M=M+1
      IF(M.GT.KELEM) THEN
         KERR=25
         WRITE(0,'(1H ,A)') 'BUUPWT:'
         CALL BUERR(KERR)
         RETURN
      END IF
      CNAMES(M)=CWTEN(NWT)
      CUNITS(M)=CWTU (NWT) 
C                                                                       
C     ------------------------------------------------------------------
C*          1.4   SEARCH AUGMENTED TABLE *B ENTRIES .                   
C                 -----------------------------------                   
 140  CONTINUE                                                          
C                                                                       
      DO 141 J=1,NJA
C                                                                       
      IF(NATBTR(J).EQ.KDD) THEN                                         
         II=J                                      
         GO TO 143                                 
      END IF                                       
 141  CONTINUE                                                          
C                                                                       
C*          1.5  GET TABLE *B ENTRY .                                   
C                ---------------------                                  
 150  CONTINUE                                                          
C                                                                       
      DO 151 J=NTABP(ICLASS),NTABP(ICLASS+1)
C                                                                       
      IF(NTABBTR(J).EQ.KDD) THEN                                        
         I=J                                      
         GO TO 155                                
      END IF                                      
C                                                                       
 151  CONTINUE                                                          
C                                                                       
      KERR=24
      WRITE(0,'(1H ,A)') 'BUUPWT :'
      CALL BUERR(KERR)
      write(0,*)"KDD=",kdd
      RETURN
C                                                                       
 155  CONTINUE                                                          
C
      YWTEN = CTABBEN(I)                                                
      YWTU  = CTABBU (I)                                                
C                                                                       
      IWTS  = NTABBS (I)                                                
      IWTRV = NTABBRV(I)                                                
      IWTDW = NTABBDW(I)                                                
C                                                                       
      GO TO 160                                                         
C                                                                       
 143  CONTINUE                                                          
C                                                                       
C*          1.4.1  GET AUGMENTED TABLE *B ENTRY.                        
C                  -----------------------------                        
      YWTEN = CATBEN(II)                                                
      YWTU  = CATBU (II)                                                
C                                                                       
      IWTS  = NATBS (II)                                                
      IWTRV = NATBRV(II)                                                
      IWTDW = NATBDW(II)                                                
C                                                                       
C     ----------------------------------------------------------------- 
C*          1.6   MODIFY ENTRY FOR OPERATOR IN FORCE.                   
C                 -----------------------------------                   
 160  CONTINUE                                                          
C                                                                       
      IWTDW=IWTDW + NDWINC                                              
      IWTS =IWTS  + NSCAM                                               
C                                                                       
C     ------------------------------------------------------------------
C*          1.7   ADD ENTRY TO WORKING TABLE.                           
C                 ---------------------------                           
 170  CONTINUE                                                          
C                                                                       
      NWT=NWT+1                                                         
      NWTR (NWT) = KDD                                                  
      NWTS (NWT) = IWTS                                                 
      NWTRV(NWT) = IWTRV                                                
      NWTDW(NWT) = IWTDW                                                
C
C     CHECK IF DATA ARE PRESENT IN DATA SECTION.
C
      IF(N221.NE.0) THEN 
         IX=KDD/1000
         IF(IX.GT.9.AND.IX.NE.31) NWTDW(NWT)=0
         N221=N221-1
      END IF
C
 175  CONTINUE
C
C     ------------------------------------------------------------------
C*          1.8  UPDATE M, CNAMES, CUNITS.                              
C                -------------------------                              
 180  CONTINUE                                                          
C                                                                       
      CWTEN(NWT) = YWTEN                                                
      CWTU (NWT) = YWTU                       
      NWTEN(NWT) = 0                          
      IF(YWTU(1:3).EQ.'CCI') NWTEN(NWT)=658367
C
       M = M + 1                                                        
       IF(M.GT.KELEM) THEN
          KERR=25
          WRITE(0,'(1H ,A)') 'BUUPWT:'
          CALL BUERR(KERR)
          RETURN
       END IF
C                                                                       
       CNAMES(M) = CWTEN(NWT)                                           
       CUNITS(M) = CWTU (NWT)                                           
C                                                                       
       IF(CWTU(NWT)(1:4).EQ.'CODE') THEN                                    
          WRITE(YCH6,'(I6)') NWTR(NWT)                               
          CUNITS(M) = CWTU(NWT)(1:11)//YCH6                          
       END IF                                                        
C                                                                       
       IF(CWTU(NWT)(1:4).EQ.'FLAG') THEN
          WRITE(YCH6,'(I6)') NWTR(NWT)                               
          CUNITS(M) = CWTU(NWT)(1:11)//YCH6                          
       END IF                                                        
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
 200  CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUEXS5( KBUFL,KBUFF,KERR)
C                                                                       
C**** *BUEXS5*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Expands section 5 of Bufr message.
C       
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUEXS5( KBUFL,KBUFF,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C        OUTPUT:
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     --------                                                          
C                                                                       
C          NONE.                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPKS         - unpack bit pattern in repeated way
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/09/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C                                                                       
      DIMENSION KBUFF(KBUFL)
      DIMENSION IBUFR(4)                                                
C
      CHARACTER*4 YBUFR
C                                                                       
C     ------------------------------------------------------------------
C*          1.   EXPAND SECTION 5.                                      
C                -----------------                                      
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C
      NWPT = NWPTB                                                      
      NBPT = NBPTB                                                      
C                                                                       
C                                                                       
C*          1.2  UNPACK LAST FOUR OCTETS CONTAINING '7777'.            
C                ------------------------------------------            
C                                                                       
      CALL BUNPKS(NBPW,KBUFF(NWPT),IBUFR,NWPT,NBPT,8,0,4,KERR)
C                                                                       
C*          1.3  CHECK IF THE LAST FOUR OCTETS ARE '7777'.
C                --------------------------------------
      YBUFR=CHAR(IBUFR(1))//CHAR(IBUFR(2))//
     1      CHAR(IBUFR(3))//CHAR(IBUFR(4))
      IF(YBUFR.NE.'7777') THEN
         KERR = 2
         WRITE(0,'(1H ,A)') 'BUEXS5 :'
         CALL BUERR(KERR)
         RETURN                                 
      END IF                                    
C
C     -----------------------------------------------------------------
      RETURN
      END                                                               
      SUBROUTINE BUPRT(K,KSUB1,KSUB2,KELEM,CNAMES,CUNITS,
     1                 CVALS,KVALS,VALUES,KSUP,KSEC1,ilens4,KERR)
C                                                                       
C**** *BUPRT*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           Print expanded Bufr messag. 
C        
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C           *CALL* *BUPRT(K,KSUB1,KSUB2,KELEM,CNAMES,CUNITS,
C                         CVALS,KVALS,VALUES,KSUP,KSEC1,ilens4,KERR)*
C
C        INPUT :
C               *K*       -  switch to print with/witout content of code tables
C                            0  - no  code table content
C                            1  - yes code table content
C               *KSUB1*   -  starting subset
C               *KSUB2*   -  ending subset
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *CNAMES*  -  character array containing element names
C               *CUNITS*  -  character array containig units
C               *CVALS*   -  character array containing bufr code table
C                            entries 
C               *KVALS*   -  dimension of VALUES array
C               *VALUES*  -  real array (expanded data values)               
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in VALUES array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           VALUES array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C                            KSEC1(16) - KSEC1(JSEC1) -- local ADP centre
C                                        information(BYTE by BYTE)
c               *ilens4*   - length of section 4 (bytes)
C        OUTPUT:
C               *KERR*    -  returned error code
C                           
C                           
C           
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C            NONE.                                                      
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       04/02/91.                  
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C                                                                       
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
      CHARACTER*80 CVALS(KVALS)
      CHARACTER YCHAR*30,YLONG*320
C                                                                       
      DIMENSION KSUP(JSUP),KSEC1(JSEC1)
      DIMENSION VALUES(KVALS)
C
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          1.   PRINT BUFR MESSAGE.
C                -------------------
 100  CONTINUE                                                          
C
      IF(KERR.NE.0) RETURN
C
      ISUB1=KSUB1
      ISUB2=KSUB2
      IF(ISUB1.LE.0.OR.ISUB2.LE.0) THEN
         WRITE(0,'(A)')    ' Warning - NEGATIVE KSUB1 OR KSUB2.'
         WRITE(0,'(A,I5)') ' Warning - number of subsets is ',KSUP(6)
         RETURN
      END IF
      IF(ISUB1.GT.KSUP(6)) THEN
         WRITE(0,'(A,I5)') ' Warning - number of subsets is ',KSUP(6)
         RETURN
      END IF
      IF(ISUB2.GT.KSUP(6)) THEN
         ISUB2=KSUP(6)
         WRITE(0,'(A,I5)') ' Warning - KSUB2 replaced by ',KSUP(6)
      END IF
C
      JQCP1= 0
C
      IF(K.EQ.0) THEN
         JQPR=0
         JQUA=0
         JQC=0
         DO 171 J171=1,KSUP(5)
         IF(CNAMES(J171)(1:8).EQ.'DATA PRE') THEN
           JQPR=J171
           GO TO 172
         END IF
 171     CONTINUE
 172     DO 173 J173=1,KSUP(5)
         IF(CNAMES(J173)(1:9).EQ.'QUALITY I') JQUA=J173
 173     CONTINUE
         DO 174 J174=1,KSUP(5)
         IF(CNAMES(J174)(1:3).EQ.'% C') THEN 
           JQC =J174
           GO TO 175
         END IF
 174     CONTINUE
C
 175     CONTINUE
C
         WRITE(6,90000)
         WRITE(6,90500)ilens4
         WRITE(6,90600)
C
         NTYPE=KSEC1(7)
         IF(JQUA.EQ.0) THEN
            JQUA=KSUP(5)
C
            DO 103 JB=ISUB1,ISUB2
C                                                          
            WRITE(*,'(/A,I3/)')'====> Data Subset # ',JB
C
            DO 104 JA=1,JQUA
C                                          
            JAJB=JA+(JB-1)*KELEM
C
            IF(VALUES(JAJB).EQ.RVIND) THEN
               WRITE(*,9918) CNAMES(JA),CUNITS(JA)
            ELSE
               IF(CUNITS(JA)(1:4).EQ.'CCIT') THEN
                  I=NINT(VALUES(JAJB)/1000)                     
                  NCHAR=VALUES(JAJB)-I*1000               
                  NW=NCHAR/80                              
                  NWOFF=NCHAR-NW*80                        
                  IF(NWOFF.NE.0) NW=NW+1                   
C                          
                  YLONG=' '
                  YLONG(1:80)=CVALS(I)                     
C                                                                       
                  II=I                                     
                  DO 125 JC=1,NW-1                         
                  II=II+1                                  
                  KF=JC*80+1                               
                  KL=(JC+1)*80                             
                  YLONG(KF:KL)=CVALS(II)                   
 125              CONTINUE                                 
C                                                                       
                  NLINE=NCHAR/30                           
                  IDIF =NCHAR-NLINE*30                     
                  IF(IDIF.NE.0) NLINE=NLINE+1              
                  YCHAR=' '
                  YCHAR=YLONG(1:30)
C          PRINT *,'CCIT : I=',I,' NCHAR=',NCHAR
C          PRINT *,'CVALS(I)=',CVALS(I)                        
C                                                                       
                  WRITE(*,9919)CNAMES(JA),VALUES(JAJB),
     1                         CUNITS(JA),YCHAR
C                                                                       
                  IF(NLINE.GT.1) THEN                           
                     DO 130 JJ=1,NLINE-1                
C                                                                       
                     K=JJ*30+1                          
                     K1=(JJ+1)*30                       
                     YCHAR=' '
                     YCHAR=YLONG(K:K1)                  
C                                                                       
                     WRITE(*,9920) YCHAR                
 130                 CONTINUE                           
C              
                   END IF
               ELSE
                  WRITE(*,9917) CNAMES(JA),VALUES(JAJB), 
     1            CUNITS(JA)
               END IF
            END IF
C
 104        CONTINUE                                          
 103        CONTINUE                                          
C
         ELSE         
            JQPRM1=JQPR-1
            JQC=JQC-1
C                                                          
            DO 101 JB=ISUB1,ISUB2
C                                                          
            JQCP1=0
C
            WRITE(*,'(/A,I3/)')'====> Data Subset # ',JB                                    
C                                                          
            DO 102 JA=1,JQUA-1
C                                                
            JAJB=JA+(JB-1)*KELEM
            JQPJB=JQPRM1+JA+(JB-1)*KELEM
C          
            IF(VALUES(JQPJB).EQ.0.0) THEN
               JQCP1=JQCP1+1
               JQCPP1=JQC+JQCP1+(JB-1)*KELEM
               IF(VALUES(JAJB).EQ.RVIND) THEN
                  WRITE(*,9918) CNAMES(JA),CUNITS(JA)
               ELSE
                  WRITE(*,9916) CNAMES(JA),VALUES(JAJB), 
     1            CUNITS(JA),
     1            CNAMES(JQC+JQCP1),VALUES(JQCPP1),
     1            CUNITS(JQC+JQCP1)
               END IF
            ELSE
               IF(VALUES(JAJB).EQ.RVIND) THEN
                  WRITE(*,9918) CNAMES(JA),CUNITS(JA)
                  IF(NTYPE.EQ.5.OR.NTYPE.EQ.3) JQCP1=JQCP1+1
               ELSE                                  
                  WRITE(*,9917) CNAMES(JA),VALUES(JAJB), 
     1            CUNITS(JA)
                  IF(NTYPE.EQ.5.OR.NTYPE.EQ.3) JQCP1=JQCP1+1
               END IF                                  
            END IF 
C                                                          
C                                                          
 102        CONTINUE                                          
 101        CONTINUE                                          
C                                                          
         END IF
      END IF
C                                                                       
      IF(K.EQ.1) THEN                                                    
C
C---------------------------------------------------------------------
          WRITE(*,'(1H )')
C
          WRITE(*,'(1H ,A)') 'WARNING : Printing content of code'//
     1   ' tables not yet implemented.'
          RETURN
C---------------------------------------------------------------------
C
C          WRITE(*,'(1H ,A)')    'EXPANDED BUFR MESSAGE  '
C          WRITE(*,'(1H ,//)')
C          WRITE(*,'(1H ,A,I6)') 'RDB DATA TYPE             ',KSEC1(6)  
C          WRITE(*,'(1H ,A,I6)') 'RDB DATA SUBTYPE          ',KSEC1(7)  
C          WRITE(*,'(1H ,A,I6)') 'TOTAL BUFR LENGTH (BYTES) ',KSUP(8)
C
C          DO 150 JB=1,KSUP(6)                                  
C                                                                       
C          WRITE(*,'(1H )')                                     
C                                                                       
C          DO 160 JA=1,KSUP(5)                                  
C                                                   
C          JAJB=JA+(JB-1)*KELEM
C
C          IF(VALUES(JAJB).EQ.RVIND) THEN
C             WRITE(*,9903) CNAMES(JA)(1:32),CUNITS(JA)            
C             WRITE(*,9903) CNAMES(JA)(33:64)
C          ELSE                                              
C             IF(CUNITS(JA)(1:10).EQ.'CODE TABLE'.OR.        
C     1         CUNITS(JA)(1:9) .EQ.'CCITTIA5'     ) THEN        
C                I=NINT(VALUES(JAJB)/1000)                     
C                NCHAR=VALUES(JAJB)-I*1000               
C                NW=NCHAR/80                              
C                NWOFF=NCHAR-NW*80                        
C                IF(NWOFF.NE.0) NW=NW+1                   
Cc                                                                       
C                YLONG(1:80)=CVALS(I)                     
C                                                                       
C                II=I                                     
C                DO 165 JC=1,NW-1                         
C                II=II+1                                  
C                KF=JC*80+1                               
C                KL=(JC+1)*80                             
C                YLONG(KF:KL)=CVALS(II)                   
C  165           CONTINUE                                 
C                                                                       
C                NLINE=NCHAR/30                           
C                IDIF =NCHAR-NLINE*30                     
C                IF(IDIF.NE.0) NLINE=NLINE+1              
C                YCHAR=YLONG(1:30)                        
C                                                                       
C                WRITE(*,9904)CNAMES(JA)(1:32),VALUES(JAJB),   
C     1         CUNITS(JA),YCHAR            
C                WRITE(*,9904)CNAMES(JA)(33:64)
C                                                                       
C                IF(NLINE.GT.1) THEN                           
C                   DO 170 JJ=1,NLINE-1                
C                                                                       
C                   K=JJ*30+1                          
C                   K1=(JJ+1)*30                       
C                   YCHAR=YLONG(K:K1)                  
C                                                                       
C                   WRITE(*,9905) YCHAR                
C  170              CONTINUE                           
C              
C                END IF                                
C             ELSE                                        
C                WRITE(*,9906) CNAMES(JA)(1:32),VALUES(JAJB),  
C     1                       CUNITS(JA)                 
C                WRITE(*,9906) CNAMES(JA)(33:64)
C             END IF                                      
C          END IF                                            
C                                                                        
C  160     CONTINUE                                             
C  150     CONTINUE                                             
C                                                                       
        END IF                                                  
C                                                                       
C                                                                       
C       RETURN                                                            
C                                                                       
C     ------------------------------------------------------------------

 9903 FORMAT(1H ,A,'     MISSING',2X,A)                                 
 9904 FORMAT(1H ,A,F20.4,2X,A,2X,A)                                     
 9905 FORMAT(1H ,100X,A)                                                
 9906 FORMAT(1H ,A,F14.4,2X,A)                                          
 9916 FORMAT(1H ,A15,1X,F20.4,1X,A20,1X,A15,1X,F3.0,1X,A15)
 9917 FORMAT(1H ,A15,1X,F20.4,1X,A24)
 9918 FORMAT(1H ,A15,1X,'             MISSING',1X,A24)
 9919 FORMAT(1H ,A15,1X,F20.4,1X,A24,1X,A)
 9920 FORMAT(1H ,62X,A)
C
90000 FORMAT(/,10X,'BUFR SECTION 4',/)
90500 FORMAT('Length of section 4 (bytes)  ',I5)
90600 FORMAT(/,'====> EXPANDED BUFR MESSAGE <====',/)
C
      END                                                               
      SUBROUTINE BUPRS2(KSUP,KEY)
C                                                                       
C**** *BUPRS2*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           Print section 2 of bufr message (expanded RDB key).
C        
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C           *CALL* *BUPRS2(KSUP,KEY)*
C
C        INPUT :
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KEY*     -  array containing section 2 information
C                            KEY( 1)-- length of section 2 (bytes)
C                            KEY( 2)-- RDB type
C                            KEY( 3)-- RDB subtype
C                            KEY( 4)-- year
C                            KEY( 5)-- month
C                            KEY( 6)-- day 
C                            KEY( 7)-- hour
C                            KEY( 8)-- minute
C                            KEY( 9)-- second
C                            KEY(10)-- longitude1
C                            KEY(11)-- latitude1
C                            KEY(12)-- longitude2
C                            KEY(13)-- latitude2
C                            KEY(14)-- number of subsets
C                            KEY(15)-- ident (numeric)
C                            KEY(16)-- ident ( CCITTIA5) one character 
C                            KEY(17)-- ident ( CCITTIA5) one character 
C                            KEY(18)-- ident ( CCITTIA5) one character
C                            KEY(19)-- ident ( CCITTIA5) one character
C                            KEY(20)-- ident ( CCITTIA5) one character
C                            KEY(21)-- ident ( CCITTIA5) one character
C                            KEY(22)-- ident ( CCITTIA5) one character
C                            KEY(23)-- ident ( CCITTIA5) one character
C                            KEY(24)-- ident ( CCITTIA5) one character
C                            KEY(25)-- total Bufr message length
C                            KEY(26)-- day    (RDB insertion)
C                            KEY(27)-- hour   (RDB insertion)
C                            KEY(28)-- minute (RDB insertion)
C                            KEY(29)-- second (RDB insertion)
C                            KEY(30)-- day    (MDB insertion)
C                            KEY(31)-- hour   (MDB insertion)
C                            KEY(32)-- minute (MDB insertion)
C                            KEY(33)-- second (MDB insertion)
C                            KEY(34)-- correction number
C                            KEY(35)-- part
C                            KEY(36)-- 0
C                            KEY(37)-- correction number
C                            KEY(38)-- part
C                            KEY(39)-- 0
C                            KEY(40)-- correction number    
C                            KEY(41)-- part
C                            KEY(42)-- 0
C                            KEY(43)-- correction number
C                            KEY(44)-- part
C                            KEY(45)-- 0
C                            KEY(46)-- the lowest Q/C % confidence
C
C           
C           
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C            NONE.                                                      
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       04/02/91.                  
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
C                                                                       
      DIMENSION KSUP(JSUP),KEY(JKEY)
C
      CHARACTER*9 CIDENT
      CHARACTER*13 YFM
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*          1.   PRINT SECTION 2.
C                ----------------
 100  CONTINUE                                                          
C
      YFM='(1H ,A,TR0,A)'
C
      IF(KSUP(2).LE.1) THEN
         WRITE(0,'(A)')'Prtkey : RDB key not defined in section 2.'
         RETURN
      END IF
C
      WRITE(*,'(1H )')
C
      WRITE(*,'(1H ,A)')       '        BUFR SECTION 2    '
      WRITE(*,'(1H )')
      WRITE(*,'(1H ,A,I9)')    'Length of section 2       ', KEY(1)
      WRITE(*,'(1H )')
      WRITE(*,'(1H ,A)')       '      Report Data Base Key  '
      WRITE(*,'(1H )')
C
      IKTYPE=0
      IF(KEY(2).EQ.2) IKTYPE=2
      IF(KEY(2).EQ.3) IKTYPE=2
      IF(KEY(2).EQ.12)IKTYPE=2
      IF(IKTYPE.EQ.0.AND.KSUP(6).GT.1) IKTYPE=2
C
      IF(IKTYPE.EQ.2) THEN
C      IF(KEY(2).EQ.2.OR.KEY(2).EQ.3.OR.KEY(2).EQ.12) THEN
C
         WRITE(*,'(1H ,A,I9)') 'RDB data type             ', KEY(2)
         WRITE(*,'(1H ,A,I9)') 'RDB data subtype          ', KEY(3)
         WRITE(*,'(1H ,A,I9)') 'Year                      ', KEY(4)
         WRITE(*,'(1H ,A,I9)') 'Month                     ', KEY(5)
         WRITE(*,'(1H ,A,I9)') 'Day                       ', KEY(6)
         WRITE(*,'(1H ,A,I9)') 'Hour                      ', KEY(7)
         WRITE(*,'(1H ,A,I9)') 'Minute                    ', KEY(8)
         WRITE(*,'(1H ,A,I9)') 'Second                    ', KEY(9)
         RLAT1=(KEY(11)-9000000)/100000.
         RLON1=(KEY(10)-18000000)/100000.
         WRITE(*,'(1H ,A,F9.2)')'Latitude  1               ', RLAT1
         WRITE(*,'(1H ,A,F9.2)')'Longitude 1               ', RLON1
         RLAT2=(KEY(13)-9000000)/100000.
         RLON2=(KEY(12)-18000000)/100000.
         WRITE(*,'(1H ,A,F9.2)')'Latitude  2               ', RLAT2
         WRITE(*,'(1H ,A,F9.2)')'Longitude 2               ', RLON2
         WRITE(*,'(1H ,A,I9)') 'Number of observations    ', KEY(14)
         WRITE(*,'(1H ,A,I9)') 'Identifier                ', KEY(15)
         WRITE(*,'(1H ,A,I9)') 'Total Bufr message length ', KEY(25)
         WRITE(*,'(1H ,A,I9)') 'Day    (RDB insertion)    ', KEY(26)
         WRITE(*,'(1H ,A,I9)') 'Hour   (RDB insertion)    ', KEY(27)
         WRITE(*,'(1H ,A,I9)') 'Minute( (RDB insertion)   ', KEY(28)
         WRITE(*,'(1H ,A,I9)') 'Second (RDB insertion)    ', KEY(29)
         WRITE(*,'(1H ,A,I9)') 'Day    (MDB arrival)      ', KEY(30)
         WRITE(*,'(1H ,A,I9)') 'Hour   (MDB arrival)      ', KEY(31)
         WRITE(*,'(1H ,A,I9)') 'Minute (MDB arrival)      ', KEY(32)
         WRITE(*,'(1H ,A,I9)') 'Second (MDB arrival       ', KEY(33)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(34)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(35)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(37)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(38)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(40)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(41)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(43)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(44)
         WRITE(*,'(1H ,A,I9)') 'Quality control % conf    ', KEY(46)
      ELSE
         WRITE(*,'(1H ,A,I9)') 'RDB data type             ', KEY(2)
         WRITE(*,'(1H ,A,I9)') 'RDB data subtype          ', KEY(3)
         WRITE(*,'(1H ,A,I9)') 'Year                      ', KEY(4)
         WRITE(*,'(1H ,A,I9)') 'Month                     ', KEY(5)
         WRITE(*,'(1H ,A,I9)') 'Day                       ', KEY(6)
         WRITE(*,'(1H ,A,I9)') 'Hour                      ', KEY(7)
         WRITE(*,'(1H ,A,I9)') 'Minute                    ', KEY(8)
         WRITE(*,'(1H ,A,I9)') 'Second                    ', KEY(9)
         RLAT1=(KEY(11)-9000000)/100000.
         RLON1=(KEY(10)-18000000)/100000.
         WRITE(*,'(1H ,A,F9.2)')'Latitude  1               ', RLAT1
         WRITE(*,'(1H ,A,F9.2)')'Longitude 1               ', RLON1
         IDD=0
         CIDENT=' '
         DO 201 ID=16,24
         IDD=IDD+1
         CIDENT(IDD:IDD)=CHAR(KEY(ID))
 201     CONTINUE
         IDD=INDEX(CIDENT,' ')
         IDD=10-IDD
         WRITE(YFM(10:10),'(I1)',ERR=202) IDD
         GO TO 203
 202     YFM(10:10)='9' 
 203     WRITE(*,FMT=YFM)        'Identifer                 ', CIDENT 
         WRITE(*,'(1H ,A,I9)') 'Total Bufr message length ', KEY(25)
         WRITE(*,'(1H ,A,I9)') 'Day    (RDB insertion)    ', KEY(26)
         WRITE(*,'(1H ,A,I9)') 'Hour   (RDB insertion)    ', KEY(27)
         WRITE(*,'(1H ,A,I9)') 'Minute (RDB insertion)    ', KEY(28)
         WRITE(*,'(1H ,A,I9)') 'Second (RDB insertion)    ', KEY(29)
         WRITE(*,'(1H ,A,I9)') 'Day    (MDB arrival)      ', KEY(30)
         WRITE(*,'(1H ,A,I9)') 'Hour   (MDB arrival)      ', KEY(31)
         WRITE(*,'(1H ,A,I9)') 'Minute (MDB arrival)      ', KEY(32)
         WRITE(*,'(1H ,A,I9)') 'Second (MDB arrival       ', KEY(33)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(34)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(35)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(37)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(38)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(40)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(41)
         WRITE(*,'(1H ,A,I9)') 'Correction number         ', KEY(43)
         WRITE(*,'(1H ,A,I9)') 'Part of message           ', KEY(44)
         WRITE(*,'(1H ,A,I9)') 'Quality control % conf    ', KEY(46)
      END IF
C
      RETURN
      END                                                               
      SUBROUTINE BUPRS1(KSEC1)
C                                                                       
C**** *BUPRS1*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           Print section 1 of bufr message.
C        
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C           *CALL* *BUPRS1(KSEC1)*
C
C        INPUT :
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C                            KSEC1(16) - KSEC1(JSEC1) -- local ADP centre
C                                        information(BYTE by BYTE)
C           
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C            NONE.                                                      
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       04/02/91.                  
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
C                                                                       
      DIMENSION KSEC1(JSEC1)
C
C     ------------------------------------------------------------------
C                                                                       
C*          1.   PRINT SECTION 1.
C                ----------------
 100  CONTINUE                                                          
C
      WRITE(*,'(1H )')
C
      if (ksec1(5) .eq. 128) then
          iflags2=1
      else
          iflags2=0
      endif
      WRITE(*,'(1H ,A)')    '        BUFR SECTION 1    '
      WRITE(*,'(1H )')
      WRITE(*,'(1H ,A,I5)') 'Length of section 1 (bytes)    ', KSEC1( 1)
      WRITE(*,'(1H ,A,I5)') 'Bufr Edition number            ', KSEC1( 2)
      WRITE(*,'(1H ,A,I5)') 'Originating centre             ', KSEC1( 3)
      WRITE(*,'(1H ,A,I5)') 'Update sequence number         ', KSEC1( 4)
      WRITE(*,'(1H ,A,I5)') 'Flag (presence of section 2)   ', iflags2
      WRITE(*,'(1H ,A,I5)') 'Bufr message type              ', KSEC1( 6)
      WRITE(*,'(1H ,A,I5)') 'Bufr message subtype           ', KSEC1( 7)
      WRITE(*,'(1H ,A,I5)') 'Version number of local table  ', KSEC1( 8)
      WRITE(*,'(1H ,A,I5)') 'Year                           ', KSEC1( 9)
      WRITE(*,'(1H ,A,I5)') 'Month                          ', KSEC1(10)
      WRITE(*,'(1H ,A,I5)') 'Day                            ', KSEC1(11)
      WRITE(*,'(1H ,A,I5)') 'Hour                           ', KSEC1(12)
      WRITE(*,'(1H ,A,I5)') 'Minute                         ', KSEC1(13)
      WRITE(*,'(1H ,A,I5)') 'Version number of Master table ', KSEC1(15)
      WRITE(*,'(1H ,A,I5)') 'Bufr Master table              ', KSEC1(14)
C
      RETURN
      END                                                               
      SUBROUTINE BUPRS0(KSEC0,KSUP)
C                                                                       
C**** *BUPRS0*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           Print section 0 of Bufr message.
C        
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C           *CALL* *BUPRS0(KSEC0,KSUP)*
C
C        INPUT :
C               *KSEC0*   -  array containing section 0 information
C                            KSEC0( 1)-- length of section 0 (bytes)
C                            KSEC0( 2)-- total length of Bufr message (bytes)
C                            KSEC0( 3)-- Bufr Edition number
C               *KSUP*    -  array with supplementary information
C                            KSUP(8)  -- total length of Bufr message (bytes)
C
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C            NONE.                                                      
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       04/02/91.                  
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
C                                                                       
      DIMENSION KSEC0(JSEC0),KSUP(JSUP)
C
C     ------------------------------------------------------------------
C                                                                       
C*          1.   PRINT SECTION 0.
C                ----------------
 100  CONTINUE                                                          
C
      WRITE(*,'(1H )')
C
      WRITE(*,'(1H ,A)')    '         BUFR SECTION 0    '
      WRITE(*,'(1H )')
      WRITE(*,'(1H ,A,I5)') 'Length of section 0 (bytes)         ',
     1                       KSEC0(1)
      WRITE(*,'(1H ,A,I5)') 'Total length of Bufr message (bytes)',
     1                       KSEC0(2)
      IF (KSEC0(3) .LE. 1) WRITE(*,'(1H ,A,I5)')
     1                      '      (not obtained from section 0):',
     2                       KSUP(8)
      WRITE(*,'(1H ,A,I5)') 'Bufr Edition number                 ',
     1                       KSEC0(3)
C
      RETURN
      END                                                               
      SUBROUTINE BUPRS3(KSEC3,KTDLEN,KTDLST,KTDEXL,KTDEXP,KELEM,CNAMES)
C                                                                       
C**** *BUPRS3*
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           Print section 3 of Bufr message.
C        
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C           *CALL* *BUPRS3(KSEC3,KTDLEN,KTDLST,KTDEXL,KTDEXP,
C                          KELEM,CNAMES)*
C
C        INPUT :
C               *KSEC3*   -  array containing section 3 information
C                            KSEC3( 1)-- length of section 3 (bytes)
C                            KSEC3( 2)-- reserved
C                            KSEC3( 3)-- number of subsets
C                            KSEC3( 4)-- flag (data type,data compression)
C               *KTDLEN*  -  number of data descriptors in section 3
C               *KTDLST*  -  array containing data descriptors in section 3
C               *KTDEXL*  -  number of entries in list of expanded data
C                            descriptors
C               *KTDEXP*  -  array containig expanded data descriptors
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *CNAMES*  -  character array containing element names
C
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C            NONE.                                                      
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       04/02/91.                  
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
C                                                                       
      DIMENSION KSEC3(JSEC3)
      DIMENSION KTDLST(KTDLEN),KTDEXP(KTDEXL)
C
      CHARACTER*64 CNAMES(KELEM)
C
C     ------------------------------------------------------------------
C                                                                       
C*          1.   PRINT SECTION 3.
C                ----------------
 100  CONTINUE                                                          
C
      WRITE(*,'(1H )')
C
      WRITE(*,'(1H ,A)')    '         BUFR SECTION 3    '
      WRITE(*,'(1H )')
      WRITE(*,'(1H ,A,I5)') 'Length of section 3 (bytes)         ',
     1                       KSEC3(1)
      WRITE(*,'(1H ,A,I5)') 'Number of data subsets              ',
     1                       KSEC3(3)
      if (btest(ksec3(4),7)) then
          iflagobs=1
      else
          iflagobs=0
      endif
      write(*,'(1h ,a,i5)') 'Flag (observed data)                ',
     1                       iflagobs
      
      if (btest(ksec3(4),6)) then
          iflagcomp=1
      else
          iflagcomp=0
      endif
      WRITE(*,'(1H ,A,I5)') 'Flag (compressed data)              ',
     1                       iflagcomp
C
      WRITE(*,'(1H ,//)')
      WRITE(*,'(1H ,A)')    '       Data descriptors (unexpanded)'
C
      WRITE(*,'(1H )')
      DO 110 I=1,KTDLEN
       WRITE(*,'(1H ,I4,2X,I6.6)') I,KTDLST(I)
 110  CONTINUE
C
      WRITE(*,'(1H ,/)')
      WRITE(*,'(1H ,A)')    '       Data descriptors (expanded)'
      WRITE(*,'(1H )')
      DO 120 I=1,KTDEXL
       WRITE(*,'(1H ,I4,2X,I6.6,2X,A)') I,KTDEXP(I),CNAMES(I)
 120  CONTINUE
      
      RETURN
      END                                                               
      SUBROUTINE BURQUC(KBUFL,KBUFF,KELEM,CNAMES,CUNITS,KSUP,KSEC3,KERR)
C
C**** *BURQUC*
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C
C          Create parameters needed for partial expansion of
C     Bufr message according to requested input lists.
C
C
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BURQUC(KBUFL,KBUFF,KELEM,CNAMES,CUNITS,KSUP,KSEC3,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *CNAMES*  -  character array containing element names
C               *CUNITS*  -  character array containig units
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC3*   -  array containing section 3 information
C                            KSEC3( 1)-- length of section 3 (bytes)
C                            KSEC3( 2)-- reserved
C                            KSEC3( 3)-- number of subsets
C                            KSEC3( 4)-- flag (data type,data compression)
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     -------                                                          
C          Word and bit pointers are calculated for every element 
C     in the expanded list of elements. If partial expansion requested,
C     indeces to required elements are determined, as well as corresponding
C     quality control, statistics etc. information.
C     
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPCK          - unpacks bit pattern
C          BUNPKS         - unpacks bit pattern in repeated way
C          BUPMRK          - proces marker operator
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMBEF'
      INCLUDE 'COMWT'
C
      CHARACTER CWTEN*64,CWTU*24                                
C
      INCLUDE 'COMWTC'
      INCLUDE 'COMRQ'
      INCLUDE 'COMREQ'
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
C
      DIMENSION KBUFF(KBUFL)
      DIMENSION KSUP(JSUP),KSEC3(JSEC3)
C
      DIMENSION IRQEI(JELEM),IQCI(JWORK),IQCDPI(JELEM/3)
      DIMENSION IBVAL(JWORK),IBV(JELEM/3)
      DIMENSION IC7(JELEM/3),IC8(JELEM/3),IC7R(JELEM/3),IC8R(JELEM/3)
      DIMENSION INWTR(JELEM),INWTS(JELEM),INWTRV(JELEM),INWTDW(JELEM)
      DIMENSION INWTEN(JELEM)
      CHARACTER*64 CCWTEN(JELEM)
      CHARACTER*24 CCWTU(JELEM)
 
C
      DIMENSION IMASK(8)
C
      SAVE INWTR,INWTS,INWTRV,INWTDW,INWTEN,CCWTEN,CCWTU
C
      DATA IMASK /1,2,4,8,16,32,64,128/
C
C
C     ------------------------------------------------------------------
C
C*          1.  DEFINE WORD/BIT POINTERS TO EVERY ELEMENT.
C               ------------------------------------------                 
 100  CONTINUE                                            
C

      IF(KERR.NE.0) RETURN
C
      IBP32= 32 + NBPTB                          
C                                                                       
      IB2=(NREQ.AND.IMASK(2))
      IB3=(NREQ.AND.IMASK(3))
      IB4=(NREQ.AND.IMASK(4))
      IB5=(NREQ.AND.IMASK(5))
      IB6=(NREQ.AND.IMASK(6))
C
      IF(NFCM.EQ.0) THEN
         MM=M
         DO 101 I=1,MM
         INWTR (I)=NWTR (I)
         INWTS (I)=NWTS (I)
         INWTRV(I)=NWTRV(I)
         INWTDW(I)=NWTDW(I)
         INWTEN(I)=NWTEN(I)
         CCWTEN(I)=CWTEN(I)
         CCWTU (I)=CWTU (I)
 101     CONTINUE
         NFCM=1
      ELSE
         M=MM
         DO 102 I=1,M
         NWTR (I)=INWTR (I)
         NWTS (I)=INWTS (I)
         NWTRV(I)=INWTRV(I)
         NWTDW(I)=INWTDW(I)
         NWTEN(I)=INWTEN(I)
         CWTEN(I)=CCWTEN(I)
         CWTU (I)=CCWTU (I)
 102     CONTINUE
      END IF
C
C*          1.1 RESOLVE MARKER OPERATOR.
C
 110  CONTINUE
C
      CALL BUPMRK(KBUFL,KBUFF,KSEC3,KELEM,CNAMES,CUNITS,KERR)
      IF(KERR.NE.0) RETURN
C
C*          1.2 NON-COMPRESSED DATA.                                    
C               --------------------                                    
 120  CONTINUE                                                          
C                                                                       
      IBIT0= 0
C
      IF(N.GT.1) THEN
         DO 121 J=1,M                                
         IBIT0 = IBIT0 + NWTDW(J)                      
 121     CONTINUE                                    
      END IF
C
      DO 122 I=1,N
      IBIT=0
      IBIT=IBIT0*(I-1)                           
      IBIT = IBIT + IBP32
      IWORD= IBIT/NBPW 
C                                 
      I1= 1+(I-1)*KELEM
C
      NBITP (I1)= IBIT - IWORD*NBPW                          
      NWORDP(I1)= NWPTB + IWORD                              
C
      DO 123 J=2,M
      IBIT = IBIT + NWTDW(J-1)            
      IWORD= IBIT/NBPW                                  
C
      JI= J+(I-1)*KELEM
C
      NBITP (JI)= IBIT - IWORD*NBPW                          
      NWORDP(JI)= NWPTB + IWORD                              
 123  CONTINUE                          
 122  CONTINUE
C                                                                       
C     -----------------------------------------------------------------
C*          2. CREATE POINTERS FOR REQUESTED ELEMENTS.
C              ---------------------------------------
 200  CONTINUE
C
C
C*          2.1 CHECK IF SUBSET OF ELEMENTS REQUESTED.
C
 210  CONTINUE
C
      IF(NREQ.EQ.0) RETURN
      IF(MREL.EQ.0) MREL=M
C
C
C*          2.2 CREATE INDICES TO CLASS 7/8
C               ---------------------------
 220  CONTINUE
C
      J=0
      JJ=0
      DO 221 I=1,MREL
      ICLASS=NWTR(I)/1000
      IF(ICLASS.EQ.7) THEN
         J=J+1
         IC7(J)=I
      END IF
C
      IF(ICLASS.EQ.8) THEN
         JJ=JJ+1
         IC8(JJ)=I
      END IF
 221  CONTINUE
C
      IC7L=J
      IC8L=JJ
      J=0
      JJ=0
      DO 223 I=1,NRQL
      ICLASS=NRQ(I)/1000
      IF(ICLASS.EQ.7.AND.RQVAL(I).NE.0) THEN
         J=J+1
         IC7R(J)=I
      END IF
C
      IF(ICLASS.EQ.8.AND.RQVAL(I).NE.0) THEN
         JJ=JJ+1
         IC8R(JJ)=I
      END IF
 223  CONTINUE
C
      IC7RL=J
      IC8RL=JJ
C
C*          2.3 CREATE INDECES TO THE REQUESTED ELEMENTS.
C               -----------------------------------------
 230  CONTINUE
C
      if(ic7rl.eq.0.and.ic8rl.eq.0) then
         il=mrel
         irl=nrql
      end if
      if(ic7rl.ne.0.and.ic8rl.eq.0) then
         il=ic7(1)
         irl=ic7r(1)-1
      end if
      if(ic8rl.ne.0.and.ic7rl.eq.0) then
         il=ic8(1)
         irl=ic8r(1)-1
      end if
      if(ic7rl.ne.0.and.ic8rl.ne.0) then
         il=ic7(1)
         if(ic7(1).gt.ic8(1)) il=ic8(1)
         irl=ic7r(1)
         if(ic7r(1).gt.ic8r(1)) irl=ic8r(1)
         irl=irl-1
      end if
c
      do 299 i=1,mrel
      irqei(i)=0
 299  continue
c
      DO 231 I=1,il   
      IF(IB2.EQ.0.OR.NRQL.EQ.0) THEN
         IRQEI(I)=I
      ELSE
         DO 232 K=1,irl
         IF(NWTR(I).EQ.NRQ(K)) IRQEI(I)=I
 232     CONTINUE
      END IF
 231  CONTINUE
C
C
      IF(IC7RL.NE.0) THEN
C
C        CLASS 7 SIGNIFYING
C
         IST=1
         DO 233 IJ=1,IC7RL
         III=IC7R(IJ)
         DO 234 I=IST,IC7L
         II=IC7(I)
         IBITP=NBITP(II)
         IWORD=NWORDP(II)
         CALL GBYTE(KBUFF(IWORD),IVAL,IBITP,NWTDW(II))
         ISCALE=NWTS(II)
         IREF  =NWTRV(II)
         IVAL  =IVAL+IREF
         VAL   =RVIND
         IF(IVAL.NE.NMASK(NWTDW(II))) THEN
            IF(ISCALE.GT.0) THEN
               VAL=IVAL/10.**ISCALE
            ELSE
               IISCALE=IABS(ISCALE)
               VAL=IVAL*10.**IISCALE
            END IF
         END IF
C
         INEXT=IC7(I+1)-1
         IF(IC7(I+1).EQ.0) INEXT=MREL
         IF(VAL.eq.RQVAL(III)) THEN
            DO 236 KI=II,INEXT
            IF(IB2.EQ.0) THEN
               IRQEI(KI)=KI
            ELSE
               IE=IC7R(IJ+1)-1
               IF(IJ.EQ.IC7RL) IE=NRQL
               if(ic8rl.ne.0) then
                  do 238 ja=1,ic8rl
                  if(ic8r(ja).gt.iii.and.ic8r(ja).lt.ie) then
                     ie=ic8r(ja)-2
                     go to 298
                  end if
 238              continue
               end if
 298           continue
               DO 237 IK=III,IE
               IF(NWTR(KI).EQ.NRQ(IK)) IRQEI(KI)=KI
 237           CONTINUE
            END IF
 236        CONTINUE
C            
            GO TO 233
         END IF
 234     CONTINUE
 233     CONTINUE
C
      END IF
C
      IF(IC8RL.NE.0) THEN
C
C        CLASS 8 SIGNIFYING
C
         IST=1
         DO 241 IJ=1,IC8RL
         III=IC8R(IJ)
         DO 242 I=IST,IC8L
         II=IC8(I)
         IBITP=NBITP(II)
         IWORD=NWORDP(II)
         CALL GBYTE(KBUFF(IWORD),IVAL,IBITP,NWTDW(II))
         ISCALE=NWTS(II)
         IREF  =NWTRV(II)
         IVAL  =IVAL+IREF
         VAL   =RVIND
         IF(IVAL.NE.NMASK(NWTDW(II))) THEN
            IF(ISCALE.GT.0) THEN
               VAL=IVAL/10.**ISCALE
            ELSE
               IISCALE=IABS(ISCALE)
               VAL=IVAL*10.**IISCALE
            END IF
         END IF
C
         INEXT=IC8(I+1)-2
         IF(IC8(I+1).EQ.0) INEXT=MREL
C
         OT=.FALSE.
         IF(VAL.NE.RVIND) THEN
            IVAL=NINT(VAL)
            IRQV=NINT(RQVAL(III))
C
            IF(NWTR(II).EQ.008001) THEN
               OT=.TRUE.
               IF((IVAL.AND.IRQV).NE.IRQV) OT=.FALSE.
            ELSE
               OT=.TRUE.
               IF(VAL.NE.RQVAL(III)) OT=.FALSE.
            END IF
         END IF
C
         IF(OT) THEN
            icl=nwtr(ii-1)/1000
            if(ICL.eq.7) irqei(ii-1)=ii-1
            DO 244 KI=II,INEXT
            IF(IB2.EQ.0) THEN
               IRQEI(KI)=KI
            ELSE
               IE=IC8R(IJ+1)-1
               IF(IJ.EQ.IC8RL) IE=NRQL
               if(ic7rl.ne.0) then
                  do 248 ja=1,ic7rl
                  if(ic7r(ja).gt.iii.and.ic7r(ja).lt.ie) then
                     ie=ic7r(ja)-1
                     go to 297
                  end if
 248              continue
               end if
 297           continue
               DO 245 IK=III,IE
               IF(NWTR(KI).EQ.NRQ(IK)) IRQEI(KI)=KI
 245           CONTINUE
            END IF
 244        CONTINUE
C            
         END IF
 242     CONTINUE
 241     CONTINUE
C
      END IF
C
C    
C
C*          3. FIND POINTERS TO QUALITY CONTROL,START OF DATA
C               ----------------------------------------------
C               PRESENT INDICATORS AND %CONFIDENCE.
C               -----------------------------------
C
 300  CONTINUE
C
      IF(IB3.NE.0) THEN
         JQPR=0
         JQUA=0
         JQCA=0
         JQC =0
C
         IDPRF=0
         DO 301 I=1,M
         IF(NWTR(I).EQ.31192) THEN
            IDPRF=IDPRF+1
            IF(IDPRF.EQ.1) JQPR=I
         END IF
 301     CONTINUE
C
 302     CONTINUE
C
         DO 303 I=1,M
         IF(NWTR(I).EQ.0) THEN
            JQUA=I
            GO TO 304
         END IF
 303     CONTINUE
C
 304     CONTINUE
C
         DO 305 I=1,M
         IF(NWTR(I).EQ.63192) THEN
            JQC=I
            GO TO 306
         END IF
 305     CONTINUE
C
 306     CONTINUE
C
         DO 307 I=1,M
         IF(NWTR(I).EQ.63191) THEN
            JQCA=I
            GO TO 310
         END IF
 307     CONTINUE
C
C
C*          3.1  SET INDICES TO GET Q/C.
C                GET BIT MAP FROM DATA SECTION.
C
 310  CONTINUE
C
         IF(JQUA.EQ.0) THEN
            WRITE(*,'(1H )')
            WRITE(*,'(1H ,A)') 'Q/C not present in this Bufr message.'
            WRITE(*,'(1H )')
         ELSE
C
            DO 311 K=1,N
C
            JQPRK=JQPR+(K-1)*KELEM
C
            IWPT=NWORDP(JQPRK)
            IBPT=NBITP (JQPRK)
            CALL GBYTES(KBUFF(IWPT),IBV,IBPT,1,0,IDPRF)
            DO 312 KA=1,IDPRF
            KAK=KA+(K-1)*KELEM
            IBVAL(KAK)=IBV(KA)
 312        CONTINUE
 311        CONTINUE
C
            IDIF=MREL-IDPRF
C
            J=0
            IF(IDIF.NE.0) THEN
               DO 313 I=1,IDIF
               J=J+1
               IQCDPI(J)=0
 313           CONTINUE
            END IF
C
            JQPRM1=JQPR-1
            DO 314 I=1,IDPRF
            J=J+1
            IQCDPI(J)=I+JQPRM1
 314        CONTINUE
C
            J=0
            IF(IDIF.NE.0) THEN
               DO 315 I=1,IDIF
               J=J+1
               DO 316 JJ=1,N
               JJJ=J+(JJ-1)*KELEM
               IQCI(JJJ)=0
 316           CONTINUE
 315           CONTINUE
            END IF
C
            K=JQC-1
            DO 317 I=1,IDPRF
            OK=.TRUE.
            J=J+1
            DO 318 JJ=1,N
            JJJ=J+(JJ-1)*KELEM
            IJJ=I+(JJ-1)*KELEM
            IQCI(JJJ)=0
            IF(IBVAL(IJJ).EQ.0) THEN 
               IF(OK) K=K+1
               IQCI(JJJ)=K
               OK=.FALSE.
            END IF
 318        CONTINUE
 317        CONTINUE
C
         END IF
      END IF
C
C
C*          3.2   FIND POINTERS TO STATISTICS,START OF DATA
C                 ----------------------------------------------
C                 PRESENT INDICATORS AND START OF STATISTICS INFORMATION.
C                 -------------------------------------------------------
 320  CONTINUE
C
      IF(IB4.NE.0) THEN
C
      END IF
C
C*          3.3   FIND POINTERS TO DIFFERENCE STATISTICS,START OF DATA,
C                 -----------------------------------------------------
C                 PRESENT INDICATORS AND STATRT OF STATISTICS INFORMATION.
C                 --------------------------------------------------------
 330  CONTINUE
C
      IF(IB5.NE.0) THEN
      END IF
C
C
C*          3.4   FIND POINTERS TO SUBSITUTED VALUES,START OF DATA
C                 ----------------------------------------------
C                 PRESENT INDICATORS AND STATRT OF DATA.
C                 --------------------------------------
 340  CONTINUE
C
      IF(IB6.NE.0) THEN
      END IF
C
C
C*          4.  CREATE REQUESTED ELEMENT LIST.
C               ------------------------------
 400  CONTINUE
C
      IF(IB2.EQ.0) NRQL=0
      J=0
      DO 401 I=1,MREL
      IF(IRQEI(I).NE.0) THEN
         II=IRQEI(I)
         J=J+1
         DO 402 K=1,N
C
         JK=J+(K-1)*KELEM
         IIK=II+(K-1)*KELEM
C
         NBITP(JK)=NBITP(IIK)
         NWORDP(JK)=NWORDP(IIK)
 402     CONTINUE 
         NWTR(J)  =NWTR(II)
         NWTS(J)  =NWTS(II)
         NWTRV(J) =NWTRV(II)
         NWTDW(J) =NWTDW(II)
         NWTEN(J) =NWTEN(II)
         CWTEN(J) =CWTEN(II)
         CWTU (J) =CWTU(II)
         CNAMES(J)=CWTEN(II)
         CUNITS(J)=CWTU(II)
      END IF
 401  CONTINUE
C
      IF(IB3.NE.0.AND.JQUA.NE.0) THEN
C
C*          4.1 ADD DATA PRESENT INDICATOR AND Q/C.
C               -----------------------------------
 410     CONTINUE
C
         J=J+1
         DO 411 K=1,N
C
         JK=J+(K-1)*KELEM
         JQUAK=JQUA+(K-1)*KELEM
C
         NWORDP(JK)=NWORDP(JQUAK)
         NBITP (JK)=NBITP (JQUAK)
 411     CONTINUE
         NWTR(J)  =NWTR(JQUA)
         NWTS(J)  =NWTS(JQUA)
         NWTRV(J) =NWTRV(JQUA)
         NWTDW(J) =NWTDW(JQUA)
         NWTEN(J) =NWTEN(JQUA)
         CWTEN(J) =CNAMES(JQUA)
         CWTU (J) =CUNITS(JQUA)
         CNAMES(J)=CWTEN(JQUA)
         CUNITS(J)=CWTU(JQUA)
C
         DO 412 I=1,MREL
         IF(IRQEI(I).NE.0) THEN
            IF(IQCDPI(I).NE.0) THEN
               J=J+1
               DO 413 K=1,N
C
               JK=J+(K-1)*KELEM
               IQCDK=IQCDPI(I)+(K-1)*KELEM
C
               NWORDP(JK)=NWORDP(IQCDK)
               NBITP (JK)=NBITP (IQCDK)
 413           CONTINUE
               NWTR(J)  =NWTR(IQCDPI(I))
               NWTS(J)  =NWTS(IQCDPI(I))
               NWTRV(J) =NWTRV(IQCDPI(I))
               NWTDW(J) =NWTDW(IQCDPI(I))
               NWTEN(J) =NWTEN(IQCDPI(I))
               CWTEN(J) =CNAMES(IQCDPI(I))
               CWTU (J) =CUNITS(IQCDPI(I))
               CNAMES(J)=CWTEN(IQCDPI(I))
               CUNITS(J)=CWTU(IQCDPI(I))
            END IF
         END IF
 412     CONTINUE
C
C        Q/C APPLICATION
C
         J=J+1
         DO 414 K=1,N

         JK=J+(K-1)*KELEM
         JQCAK=JQCA+(K-1)*KELEM
C
         NWORDP(JK)=NWORDP(JQCAK)
         NBITP (JK)=NBITP (JQCAK)
 414     CONTINUE
         NWTR(J)  =NWTR(JQCA)
         NWTS(J)  =NWTS(JQCA)
         NWTRV(J) =NWTRV(JQCA)
         NWTDW(J) =NWTDW(JQCA)
         NWTEN(J) =NWTEN(JQCA)
         CWTEN(J) =CNAMES(JQCA)
         CWTU (J) =CUNITS(JQCA)
         CNAMES(J)=CWTEN(JQCA)
         CUNITS(J)=CWTU(JQCA)
C
C        % CONFIDENCE
C
         DO 415 I=1,MREL
         IF(IRQEI(I).NE.0) THEN
            OK=.TRUE.
            DO 416 JJ=1,N
            IJJ=I+(JJ-1)*KELEM
            IF(IQCI(IJJ).NE.0) THEN
               IF(OK) J=J+1
               OK=.FALSE.
               NWTR(J)  =NWTR  (IQCI(IJJ))
               NWTS(J)  =NWTS  (IQCI(IJJ))
               NWTRV(J) =NWTRV (IQCI(IJJ))
               NWTDW(J) =NWTDW (IQCI(IJJ))
               NWTEN(J) =NWTEN (IQCI(IJJ))
               CWTEN(J) =CNAMES(IQCI(IJJ))
               CWTU (J) =CUNITS(IQCI(IJJ))
               CNAMES(J)=CWTEN (IQCI(IJJ))
               CUNITS(J)=CWTU  (IQCI(IJJ))
C
               JJJ=J+(JJ-1)*KELEM
               JQCIJJ=IQCI(IJJ)+(JJ-1)*KELEM
C
               NWORDP(JJJ)=NWORDP(JQCIJJ)
               NBITP (JJJ)=NBITP (JQCIJJ)
            END IF
 416        CONTINUE
C
         END IF
 415     CONTINUE
      END IF
C
C*          4.2  STATISTICS
C
 420  CONTINUE
C
      IF(IB4.NE.0) THEN
C
      END IF  
C
C*          4.3 DIFFERENCE STATISTICS.
C               ----------------------
 430  CONTINUE
C
      IF(IB5.NE.0) THEN
C
      END IF  
C
C*          4.4 SUBSTITUTED VALUES.
C               -------------------
 440  CONTINUE
C
      IF(IB6.NE.0) THEN
C
      END IF  
C
C*          5. MAKE ONE TO ONE CORRESPONDENCE BETWEEN ELEMENTS AND
C              Q/C,STATISTICS,DIFFERENCE STATISTICS OR SUBSTITUTED VALUES
C              LEAVING BEHIND DATA PRESENT INDICATORS.
 500  CONTINUE
C
      IF(IB3.NE.0) THEN
      END IF
C
      IF(IB4.NE.0) THEN
      END IF
C
      IF(IB5.NE.0) THEN
      END IF
C
      IF(IB6.NE.0) THEN
      END IF
C
C*          9.  UPDATE TOTAL NUMBER OF ELEMENTS.
C               --------------------------------
 900  CONTINUE
C
      M=J
C
      RETURN
C
      END                         
      SUBROUTINE BURQC(KBUFL,KBUFF,KELEM,CNAMES,CUNITS,KSUP,KSEC3,KERR)
C
C**** *BURQC*
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C
C          Create parameters needed for partial expansion of
C     Bufr message with compressed data according to requested 
C     input lists.
C
C
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BURQC(KBUFL,KBUFF,KELEM,CNAMES,CUNITS,KSUP,KSEC3,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *CNAMES*  -  character array containing element names
C               *CUNITS*  -  character array containig units
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KSEC3*   -  array containing section 3 information
C                            KSEC3( 1)-- length of section 3 (bytes)
C                            KSEC3( 2)-- reserved
C                            KSEC3( 3)-- number of subsets
C                            KSEC3( 4)-- flag (data type,data compression)
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C      -------                                                          
C                                                                       
C          Word and bit pointers are calculated for every element 
C     in the expanded list of elements. If partial expansion requested,
C     indeces to required elements are determined, as well as corresponding
C     quality control, statistics etc. information.
C     
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPCK          - unpacks bit pattern
C          BUNPKS         - unpacks bit pattern in repeated way
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMBEF'
      INCLUDE 'COMWT'
C
      CHARACTER CWTEN*64,CWTU*24                                
C
      INCLUDE 'COMWTC'
      INCLUDE 'COMRQ'
      INCLUDE 'COMREQ'
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
C
      DIMENSION KSUP(JSUP),KSEC3(JSEC3)
      DIMENSION KBUFF(KBUFL)
C
      DIMENSION IRQEI(JELEM),IQCI(JWORK),IQCDPI(JELEM/3)
      DIMENSION IBVAL(JWORK),IBV(JELEM/3)
      DIMENSION IC7(JELEM/3),IC8(JELEM/3),IC7R(JELEM/3),IC8R(JELEM/3)
      DIMENSION INWTR(JELEM),INWTS(JELEM),INWTRV(JELEM),INWTDW(JELEM)
      DIMENSION INWTEN(JELEM)
      CHARACTER*64 CCWTEN(JELEM)
      CHARACTER*24 CCWTU(JELEM)
 
C
      DIMENSION IMASK(8)
C
      SAVE INWTR,INWTS,INWTRV,INWTDW,INWTEN,CCWTEN,CCWTU
C
      DATA IMASK /1,2,4,8,16,32,64,128/
C
C
C     ------------------------------------------------------------------
C
C*          1.  DEFINE WORD/BIT POINTERS TO EVERY ELEMENT.
C               ------------------------------------------                 
 100  CONTINUE                                            
C
      IBP32= 32 + NBPTB                          
C                                                                       
      IB2=(NREQ.AND.IMASK(2))
      IB3=(NREQ.AND.IMASK(3))
      IB4=(NREQ.AND.IMASK(4))
      IB5=(NREQ.AND.IMASK(5))
      IB6=(NREQ.AND.IMASK(6))
C      
      IF(NFCM.EQ.0) THEN
         MM=M
         DO 101 I=1,MM
         INWTR (I)=NWTR (I)
         INWTS (I)=NWTS (I)
         INWTRV(I)=NWTRV(I)
         INWTDW(I)=NWTDW(I)
         INWTEN(I)=NWTEN(I)
         CCWTEN(I)=CWTEN(I)
         CCWTU (I)=CWTU (I)
 101     CONTINUE
         NFCM=1
      ELSE
         M=MM
         DO 102 I=1,M
         NWTR (I)=INWTR (I)
         NWTS (I)=INWTS (I)
         NWTRV(I)=INWTRV(I)
         NWTDW(I)=INWTDW(I)
         NWTEN(I)=INWTEN(I)
         CWTEN(I)=CCWTEN(I)
         CWTU (I)=CCWTU (I)
 102     CONTINUE
      END IF
C                                                                       
C*          1.1 COMPRESSED DATA.                                        
C               ----------------                                        
 110  CONTINUE
C                                                          
      IBIT=IBP32
      IWORD=IBIT/NBPW
C         
      NWORDP(1)=NWPTB+IWORD
      NBITP (1)=IBIT-IWORD*NBPW
C
      DO 111 I=2,M
      IWRD=NWORDP(I-1)
      IBTP=NBITP (I-1)
C
      IF(CWTU(I-1).EQ.'CCITTIA5') THEN
         IICH=NWTDW(I-1)/8
         CALL BUNPKS(NBPW,KBUFF(IWRD),IR0,IWRD,IBTP,8,0,IICH,KERR)
         IF(KERR.NE.0) RETURN
         IF(IWRD.GT.KBUFL) THEN
            KERR=26
            WRITE(0,'(1H ,A)') 'BURQC :'
            CALL BUERR(KERR)
            RETURN
         END IF
      ELSE
         CALL BUNPCK(NBPW,KBUFF(IWRD),IR0,IWRD,IBTP,NWTDW(I-1),KERR)
         IF(KERR.NE.0) RETURN
         IF(IWRD.GT.KBUFL) THEN
            KERR=26
            WRITE(0,'(1H ,A)') 'BURQC :'
            CALL BUERR(KERR)
            RETURN
         END IF
      END IF
C
      CALL BUNPCK(NBPW,KBUFF(IWRD),IDWINC,IWRD,IBTP,6,KERR)
      IF(KERR.NE.0) RETURN
      IF(IDWINC.GT.JBPW) THEN
            print*,'idwinc=',idwinc
            KERR=15
            WRITE(0,'(1H ,A)') 'BURQC :'
            CALL BUERR(KERR)
            RETURN
      END IF
      IF(IWRD.GT.KBUFL) THEN
            KERR=26
            WRITE(0,'(1H ,A)') 'BURQC :'
            CALL BUERR(KERR)
            RETURN
      END IF
C      
      IF(CWTU(I-1).EQ.'CCITTIA5') THEN
         NWTIWS(I-1)=NWTDW(I-1)+6+N*IDWINC*8
      ELSE
         NWTIWS(I-1)=NWTDW(I-1)+6+N*IDWINC
      END IF
C
      IBIT = NBITP(I-1) + NWTIWS(I-1)
      IWORD= IBIT/NBPW                  
C                                                                       
      NBITP (I)= IBIT - IWORD*NBPW          
      NWORDP(I)= NWORDP(I-1) + IWORD              
C
      IF(NWORDP(I).GT.KBUFL) THEN
            KERR=26
            WRITE(0,'(1H ,A)') 'BURQC :'
            CALL BUERR(KERR)
            RETURN
      END IF
 111  CONTINUE
C                                                                       
C     -----------------------------------------------------------------
C*          2. CREATE POINTERS FOR REQUESTED ELEMENTS.
C              ---------------------------------------
 200  CONTINUE
C
C
C*          2.1 CHECK IF SUBSET OF ELEMENTS REQUESTED.
C
 210  CONTINUE
C
      IF(NREQ.EQ.0) RETURN
      IF(MREL.EQ.0) MREL=M
C
C
C*          2.2 CREATE INDECES TO CLASS 7/8
C               ---------------------------
 220  CONTINUE
C
      J=0
      JJ=0
      DO 221 I=1,MREL
      ICLASS=NWTR(I)/1000
      IF(ICLASS.EQ.7) THEN
         J=J+1
         IC7(J)=I
      END IF
C
      IF(ICLASS.EQ.8) THEN
         JJ=JJ+1
         IC8(JJ)=I
      END IF
 221  CONTINUE
C
      IC7L=J
      IC8L=JJ
      J=0
      JJ=0
      DO 223 I=1,NRQL
      ICLASS=NRQ(I)/1000
      IF(ICLASS.EQ.7) THEN
         J=J+1
         IC7R(J)=I
      END IF
C
      IF(ICLASS.EQ.8) THEN
         JJ=JJ+1
         IC8R(JJ)=I
      END IF
 223  CONTINUE
C
      IC7RL=J
      IC8RL=JJ
C
C*          2.3 CREATE INDECES TO THE REQUESTED ELEMENTS.
C               -----------------------------------------
 230  CONTINUE
C
      if(ic7rl.eq.0.and.ic8rl.eq.0) then
         il=mrel
         irl=nrql
      end if
      if(ic7rl.ne.0.and.ic8rl.eq.0) then
         il=ic7(1)
         irl=ic7r(1)-1
      end if
      if(ic8rl.ne.0.and.ic7rl.eq.0) then
         il=ic8(1)
         irl=ic8r(1)-1
      end if
      if(ic7rl.ne.0.and.ic8rl.ne.0) then
         il=ic7(1)
         if(ic7(1).gt.ic8(1)) il=ic8(1)
         irl=ic7r(1)
         if(ic7r(1).gt.ic8r(1)) irl=ic8r(1)
         irl=irl-1
      end if
c
      do 299 i=1,mrel
      irqei(i)=0
 299  continue
c
      DO 231 I=1,il   
      IF(IB2.EQ.0.OR.NRQL.EQ.0) THEN
         IRQEI(I)=I
      ELSE
         DO 232 K=1,irl
         IF(NWTR(I).EQ.NRQ(K)) IRQEI(I)=I
 232     CONTINUE
      END IF
 231  CONTINUE
C
C
      IF(IC7RL.NE.0) THEN
C
C        CLASS 7 SIGNIFYING
C
         IST=1
         DO 233 IJ=1,IC7RL
         III=IC7R(IJ)
         DO 234 I=IST,IC7L
         II=IC7(I)
         IBITP=NBITP(II)
         IWORD=NWORDP(II)
         CALL GBYTE(KBUFF(IWORD),IVAL,IBITP,NWTDW(II))
         ISCALE=NWTS(II)
         IREF  =NWTRV(II)
         IVAL  =IVAL+IREF
         VAL   =RVIND
         IF(IVAL.NE.NMASK(NWTDW(II))) THEN
            IF(ISCALE.GT.0) THEN
               VAL=IVAL/10.**ISCALE
            ELSE
               IISCALE=IABS(ISCALE)
               VAL=IVAL*10.**IISCALE
            END IF
         END IF
C
         INEXT=IC7(I+1)-1
         IF(IC7(I+1).EQ.0) INEXT=MREL
         IF(VAL.eq.RQVAL(III)) THEN
            DO 236 KI=II,INEXT
            IF(IB2.EQ.0) THEN
               IRQEI(KI)=KI
            ELSE
               IE=IC7R(IJ+1)-1
               IF(IJ.EQ.IC7RL) IE=NRQL
               if(ic8rl.ne.0) then
                  do 238 ja=1,ic8rl
                  if(ic8r(ja).gt.iii.and.ic8r(ja).lt.ie) then
                     ie=ic8r(ja)-2
                     go to 298
                  end if
 238              continue
               end if
 298           continue
               DO 237 IK=III,IE
               IF(NWTR(KI).EQ.NRQ(IK)) IRQEI(KI)=KI
 237           CONTINUE
            END IF
 236        CONTINUE
C            
            GO TO 233
         END IF
 234     CONTINUE
 233     CONTINUE
C
      END IF
C
      IF(IC8RL.NE.0) THEN
C
C        CLASS 8 SIGNIFYING
C
         IST=1
         DO 241 IJ=1,IC8RL
         III=IC8R(IJ)
         DO 242 I=IST,IC8L
         II=IC8(I)
         IBITP=NBITP(II)
         IWORD=NWORDP(II)
         CALL GBYTE(KBUFF(IWORD),IVAL,IBITP,NWTDW(II))
         ISCALE=NWTS(II)
         IREF  =NWTRV(II)
         IVAL  =IVAL+IREF
         VAL   =RVIND
         IF(IVAL.NE.NMASK(NWTDW(II))) THEN
            IF(ISCALE.GT.0) THEN
               VAL=IVAL/10.**ISCALE
            ELSE
               IISCALE=IABS(ISCALE)
               VAL=IVAL*10.**IISCALE
            END IF
         END IF
C
         INEXT=IC8(I+1)-2
         IF(IC8(I+1).EQ.0) INEXT=MREL
C
         OT=.FALSE.
         IF(VAL.NE.RVIND) THEN
            IVAL=NINT(VAL)
            IRQV=NINT(RQVAL(III))
C
            IF(NWTR(II).EQ.008001) THEN
               OT=.TRUE.
               IF((IVAL.AND.IRQV).NE.IRQV) OT=.FALSE.
            ELSE
               OT=.TRUE.
               IF(VAL.NE.RQVAL(III)) OT=.FALSE.
            END IF
         END IF
C
         IF(OT) THEN
            icl=nwtr(ii-1)/1000
            if(ICL.eq.7) irqei(ii-1)=ii-1
            DO 244 KI=II,INEXT
            IF(IB2.EQ.0) THEN
               IRQEI(KI)=KI
            ELSE
               IE=IC8R(IJ+1)-1
               IF(IJ.EQ.IC8RL) IE=NRQL
               if(ic7rl.ne.0) then
                  do 248 ja=1,ic7rl
                  if(ic7r(ja).gt.iii.and.ic7r(ja).lt.ie) then
                     ie=ic7r(ja)-1
                     go to 297
                  end if
 248              continue
               end if
 297           continue
               DO 245 IK=III,IE
               IF(NWTR(KI).EQ.NRQ(IK)) IRQEI(KI)=KI
 245           CONTINUE
            END IF
 244        CONTINUE
C            
         END IF
 242     CONTINUE
 241     CONTINUE
C
      END IF
C
C
C
C
C*          3. FIND POINTERS TO QUALITY CONTROL,START OF DATA
C               ----------------------------------------------
C               PRESENT INDICATORS AND %CONFIDENCE.
C               -----------------------------------
C
 300  CONTINUE
C
      IF(IB3.NE.0) THEN
         JQPR=0
         JQUA=0
         JQCA=0
         JQC =0
C
         IDPRF=0
         DO 301 I=1,M
         IF(NWTR(I).EQ.31192) THEN
            IDPRF=IDPRF+1
            IF(IDPRF.EQ.1) JQPR=I
         END IF
 301     CONTINUE
C
 302     CONTINUE
C
         DO 303 I=1,M
         IF(NWTR(I).EQ.0) THEN
            JQUA=I
            GO TO 304
         END IF
 303     CONTINUE
C
 304     CONTINUE
C
         DO 305 I=1,M
         IF(NWTR(I).EQ.63192) THEN
            JQC=I
            GO TO 306
         END IF
 305     CONTINUE
C
 306     CONTINUE
C
         DO 307 I=1,M
         IF(NWTR(I).EQ.63191) THEN
            JQCA=I
            GO TO 310
         END IF
 307     CONTINUE
C
C
C*          3.1  SET INDICES TO GET Q/C.
C                GET BIT MAP FROM DATA SECTION.
C
 310  CONTINUE
C
         IF(JQUA.EQ.0) THEN
            WRITE(*,'(1H )')
            WRITE(*,'(1H ,A)') 'Q/C not present in this Bufr message.'
            WRITE(*,'(1H )')
         ELSE
C
            DO 311 K=1,N
C
            JQPRK=JQPR+(K-1)*KELEM
C
            IWPT=NWORDP(JQPRK)
            IBPT=NBITP (JQPRK)
            CALL GBYTES(KBUFF(IWPT),IBV,IBPT,1,0,IDPRF)
            DO 312 KA=1,IDPRF
            KAK=KA+(K-1)*KELEM
            IBVAL(KAK)=IBV(KA)
 312        CONTINUE
 311        CONTINUE
C
            IDIF=MREL-IDPRF
C
            J=0
            IF(IDIF.NE.0) THEN
               DO 313 I=1,IDIF
               J=J+1
               IQCDPI(J)=0
 313           CONTINUE
            END IF
C
            JQPRM1=JQPR-1
            DO 314 I=1,IDPRF
            J=J+1
            IQCDPI(J)=I+JQPRM1
 314        CONTINUE
C
            J=0
            IF(IDIF.NE.0) THEN
               DO 315 I=1,IDIF
               J=J+1
               DO 316 JJ=1,N
               JJJ=J+(JJ-1)*KELEM
               IQCI(JJJ)=0
 316           CONTINUE
 315           CONTINUE
            END IF
C
            K=JQC-1
            DO 317 I=1,IDPRF
            OK=.TRUE.
            J=J+1
            DO 318 JJ=1,N
            JJJ=J+(JJ-1)*KELEM
            IJJ=I+(JJ-1)*KELEM
            IQCI(JJJ)=0
            IF(IBVAL(IJJ).EQ.0) THEN 
               IF(OK) K=K+1
               IQCI(JJJ)=K
               OK=.FALSE.
            END IF
 318        CONTINUE
 317        CONTINUE
C
         END IF
      END IF
C
C
C*          3.2   FIND POINTERS TO STATISTICS,START OF DATA
C                 ----------------------------------------------
C                 PRESENT INDICATORS AND STATRT OF STATISTICS INFORMATION.
C                 --------------------------------------------------------
 320  CONTINUE
C
      IF(IB4.NE.0) THEN
C
      END IF
C
C*          3.3   FIND POINTERS TO DIFFERENCE STATISTICS,START OF DATA,
C                 -----------------------------------------------------
C                 PRESENT INDICATORS AND STATRT OF STATISTICS INFORMATION.
C                 --------------------------------------------------------
 330  CONTINUE
C
      IF(IB5.NE.0) THEN
      END IF
C
C
C*          3.4   FIND POINTERS TO SUBSITUTED VALUES,START OF DATA
C                 ----------------------------------------------
C                 PRESENT INDICATORS AND STATRT OF DATA.
C                 --------------------------------------
 340  CONTINUE
C
      IF(IB6.NE.0) THEN
      END IF
C
C
C*          4.  CREATE REQUESTED ELEMENT LIST.
C               ------------------------------
 400  CONTINUE
C
      IF(IB2.EQ.0) NRQL=0
      J=0
      DO 401 I=1,MREL
      IF(IRQEI(I).NE.0) THEN
         II=IRQEI(I)
         J=J+1
         NBITP(J)=NBITP(II)
         NWORDP(J)=NWORDP(II)
         NWTR(J)  =NWTR(II)
         NWTS(J)  =NWTS(II)
         NWTRV(J) =NWTRV(II)
         NWTDW(J) =NWTDW(II)
         NWTEN(J) =NWTEN(II)
         CWTEN(J) =CWTEN(II)
         CWTU (J) =CWTU(II)
         CNAMES(J)=CWTEN(II)
         CUNITS(J)=CWTU(II)
      END IF
 401  CONTINUE
C
      IF(IB3.NE.0.AND.JQUA.NE.0) THEN
C
C*          4.1 ADD DATA PRESENT INDICATOR AND Q/C.
C               -----------------------------------
 410     CONTINUE
C
         J=J+1
         NWORDP(J)=NWORDP(JQUA)
         NBITP (J)=NBITP (JQUA)
         NWTR(J)  =NWTR(JQUA)
         NWTS(J)  =NWTS(JQUA)
         NWTRV(J) =NWTRV(JQUA)
         NWTDW(J) =NWTDW(JQUA)
         NWTEN(J) =NWTEN(JQUA)
         CWTEN(J) =CNAMES(JQUA)
         CWTU (J) =CUNITS(JQUA)
         CNAMES(J)=CWTEN(JQUA)
         CUNITS(J)=CWTU(JQUA)
C
         DO 412 I=1,MREL
         IF(IRQEI(I).NE.0) THEN
            IF(IQCDPI(I).NE.0) THEN
               J=J+1
               NWORDP(J)=NWORDP(IQCDPI(I))
               NBITP (J)=NBITP (IQCDPI(I))
               NWTR(J)  =NWTR(IQCDPI(I))
               NWTS(J)  =NWTS(IQCDPI(I))
               NWTRV(J) =NWTRV(IQCDPI(I))
               NWTDW(J) =NWTDW(IQCDPI(I))
               NWTEN(J) =NWTEN(IQCDPI(I))
               CWTEN(J) =CNAMES(IQCDPI(I))
               CWTU (J) =CUNITS(IQCDPI(I))
               CNAMES(J)=CWTEN(IQCDPI(I))
               CUNITS(J)=CWTU(IQCDPI(I))
            END IF
         END IF
 412     CONTINUE
C
C        Q/C APPLICATION
C
         J=J+1
         NWORDP(J)=NWORDP(JQCA)
         NBITP (J)=NBITP (JQCA)
         NWTR(J)  =NWTR(JQCA)
         NWTS(J)  =NWTS(JQCA)
         NWTRV(J) =NWTRV(JQCA)
         NWTDW(J) =NWTDW(JQCA)
         NWTEN(J) =NWTEN(JQCA)
         CWTEN(J) =CNAMES(JQCA)
         CWTU (J) =CUNITS(JQCA)
         CNAMES(J)=CWTEN(JQCA)
         CUNITS(J)=CWTU(JQCA)
C
C        % CONFIDENCE
C
         DO 415 I=1,MREL
         IF(IRQEI(I).NE.0) THEN
            OK=.TRUE.
            DO 416 JJ=1,N
            IJJ=I+(JJ-1)*KELEM
            IF(IQCI(IJJ).NE.0) THEN
               IF(OK) J=J+1
               OK=.FALSE.
               NWTR(J)  =NWTR  (IQCI(IJJ))
               NWTS(J)  =NWTS  (IQCI(IJJ))
               NWTRV(J) =NWTRV (IQCI(IJJ))
               NWTDW(J) =NWTDW (IQCI(IJJ))
               NWTEN(J) =NWTEN (IQCI(IJJ))
               CWTEN(J) =CNAMES(IQCI(IJJ))
               CWTU (J) =CUNITS(IQCI(IJJ))
               CNAMES(J)=CWTEN (IQCI(IJJ))
               CUNITS(J)=CWTU  (IQCI(IJJ))
            END IF
 416        CONTINUE
C
            IK=I+(K-1)*KELEM
            NWORDP(J)=NWORDP(IQCI(IK))
            NBITP (J)=NBITP (IQCI(IK))
         END IF
 415     CONTINUE
      END IF
C
C*          4.2  STATISTICS
C
 420  CONTINUE
C
      IF(IB4.NE.0) THEN
C
      END IF  
C
C*          4.3 DIFFERENCE STATISTICS.
C               ----------------------
 430  CONTINUE
C
      IF(IB5.NE.0) THEN
C
      END IF  
C
C*          4.4 SUBSTITUTED VALUES.
C               -------------------
 440  CONTINUE
C
      IF(IB6.NE.0) THEN
C
      END IF  
C
C*          5. MAKE ONE TO ONE CORRESPONDENCE BETWEEN ELEMENTS AND
C              Q/C,STATISTICS,DIFFERENCE STATISTICS OR SUBSTITUTED VALUES
C              LEAVING BEHIND DATA PRESENT INDICATORS.
 500  CONTINUE
C
      IF(IB3.NE.0) THEN
      END IF
C
      IF(IB4.NE.0) THEN
      END IF
C
      IF(IB5.NE.0) THEN
      END IF
C
      IF(IB6.NE.0) THEN
      END IF
C
C*          9.  UPDATE TOTAL NUMBER OF ELEMENTS.
C               --------------------------------
 900  CONTINUE
C
      M=J
C
      RETURN
C
      END                         
      SUBROUTINE BUSRQ(KREQ,KRQL,KRQ,RQV,KERR)
C                                                                       
C**** *BUSRQ*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Set list of Bufr table B element references 
C     for partial expansion.
C
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUSRQ(KREQ,KRQL,KRQ,RQV,KERR)*
C
C        INPUT :
C               *KREQ*    -  flag 
C                            bit number     meaning
C                                1        - 0 no bit map delivered to user
C                                           1    bit map delivered to user
C                                2        - 0 no partial expansion
C                                           1    partial expansion
C                                3        - 0 no Q/C required
C                                         - 1    Q/C required
C                                4        - 0 no statistics required
C                                         - 1    statistics
C                                5        - 0 no diffrence statistics
C                                         - 1    difference statistics
C                                6        - 0 no substituted values
C                                         - 1    substituted values
C               *KRQL*    -  number of requested elements
C               *KRQ*     -  list of requested table B reference 
C               *RQV*     -  list of values signifying requested element
C                            (say pressure  at 50000 Pa)
C        OUTPUT:
C               *KERR*    -  returned error code
C                           
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          None.
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          NONE.
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF' 
      INCLUDE 'COMREQ' 
      INCLUDE 'COMDEFC'
C
      CHARACTER*4 CUSER,CECMWF
      DIMENSION KRQ  (*),RQV(*)
C
C     ------------------------------------------------------------------
C*          1. Move requested elements into common block /comreq/.
C              ---------------------------------------------------
 100  CONTINUE
C
      IF(KERR.NE.0) RETURN
C
      CUSER='USER'
C
      NRQL=KRQL
      NREQ=KREQ
C
      DO 101 I=1,KRQL
      NRQ(I)=KRQ(I)
      RQVAL(I)=RQV(I)
 101  CONTINUE
C
      RETURN
      END
      SUBROUTINE BUSEL(KTDLEN,KTDLST,KTDEXL,KTDEXP,KERR)
C                                                                       
C**** *BUSEL*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Returns list of Data Descriptors as in Section 3  of Bufr
C     message and total/requested list of elements.
C
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUSEL(KTDLEN,KTDLST,KTDEXL,KTDEXP,KERR)*
C
C        OUTPUT:
C               *KTDLEN*  -  number of data descriptors in section 3
C               *KTDLST*  -  array containing data descriptors in section 3
C               *KTDEXL*  -  number of entries in list of expanded data
C                            descriptors
C               *KTDEXP*  -  array containig expanded data descriptors
C               *KERR*    -  returned error code
C                           
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          NONE.
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          NONE.
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMEL'
C
      DIMENSION  KTDLST(*),KTDEXP(*)
C
C
C     -----------------------------------------------------------------

C*          1.  PUT LIST OF ELEMENTS FROM COMMON BLOCK 
C               --------------------------------------
C               TO REQUESTED ARRAYS.
C               --------------------
 100  CONTINUE
C
      IF(KERR.NE.0) RETURN
C
      KTDLEN=NTDLEN
      KTDEXL=NTDEXL
C
      DO 101 I=1,NTDLEN
      KTDLST(I)=NTDLST(I)
 101  CONTINUE
C
      DO 102 I=1,NTDEXL
      KTDEXP(I)=NTDEXP(I)
 102  CONTINUE
C
      RETURN
      END
      SUBROUTINE BUPMRK(KBUFL,KBUFF,KSEC3,KELEM,CNAMES,CUNITS,KERR)
C
C**** *BUPMRK*
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C
C          Process marker operator, relacing it with corresponding
C     table B element descriptor.
C
C
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUPMRK(KBUFL,KBUFF,KSEC3,KELEM,CNAMES,CUNITS,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C               *KSEC3*   -  array containing section 3 information
C                            KSEC3( 1)-- length of section 3 (bytes)
C                            KSEC3( 2)-- reserved
C                            KSEC3( 3)-- number of subsets
C                            KSEC3( 4)-- flag (data type,data compression)
C               *KELEM*   -  dimension of CNAMES, CUNITS array
C               *CNAMES*  -  character array containing element names
C               *CUNITS*  -  character array containig units
C        OUTPUT:
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     -------                                                          
C          NONE.                                                             
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPCK          - unpacks bit pattern
C          BUNPKS         - unpacks bit pattern in repeated way
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF' 
      INCLUDE 'COMWORK' 
      INCLUDE 'COMBEF' 
      INCLUDE 'COMWT' 
C
      CHARACTER CWTEN*64,CWTU*24                                
C
      INCLUDE 'COMWTC' 
      INCLUDE 'COMRQ' 
C
      CHARACTER*64 CNAMES(KELEM)
      CHARACTER*24 CUNITS(KELEM)
      DIMENSION KSEC3(JSEC3)
C
      DIMENSION KBUFF(KBUFL),IBV(JELEM/3)
      DIMENSION IMASK(8)
      DATA IMASK/1,2,4,8,16,32,64,128/
C
C     ------------------------------------------------------------------
C*          1.  FINED OPERATOR 223000 TO 232000.
C               --------------------------------
 100  CONTINUE                                            
C
      IF(KERR.NE.0) RETURN
C
C           1.1 CHECK IF DATA ARE COMRESSED.
C               ----------------------------
 110  CONTINUE
C
      IB=0
      IF((KSEC3(4).AND.IMASK(7)).NE.0) IB=1
C
C*          1.2 FIND POSITION OF OPREATORS.
C               ---------------------------
 120  CONTINUE
C
      DO 121 J=1,M
C
      IF(NWTR(J).EQ.223000) THEN
         NP223000=J
         NWTR(J)=0
         GO TO 200
      END IF
      IF(NWTR(J).EQ.224000) THEN
         NP224000=J
         NWTR(J)=0
         GO TO 300
      END IF
      IF(NWTR(J).EQ.225000) THEN
         NP225000=J
         NWTR(J)=0
         GO TO 400
      END IF
      IF(NWTR(J).EQ.232000) THEN
         NP232000=J
         NWTR(J)=0
         GO TO 500
      END IF
C
      GO TO 121
C
C     -----------------------------------------------------------------
C*          2.  PROCESS SUBSTITUTED VALUES OPERATOR.
C               ------------------------------------
C
 200  CONTINUE
C
C*          2.1 FIND FIRST ACCURANCE OD 223255.
C               -------------------------------
 210  CONTINUE
C
      DO 211 I=NP223000,M
      IF(NWTR(I).EQ.223255) THEN
         NP223255=I
         GO TO 220
      END IF
 211  CONTINUE
C      
C*          2.2 COUNT NUMBER OF DATA PRESENT INDICATORS.
C               ----------------------------------------
 220  CONTINUE
C
      OF223=.TRUE.
      DO 221 I=NP223000,NP223255
      IF(NWTR(I).EQ.031031) THEN
         IF(OF223) THEN
            I223=I
            OF223=.FALSE.
         END IF
         NR223=NR223+1
      END IF
 221  CONTINUE
C
C*          2.3 CALCULATE WORD AND BIT POINTER TO FIRST
C               ---------------------------------------
C               DATA PRESENT INDICATOR.
C               -----------------------
 230  CONTINUE
C
      IBP32=32+NBPTB
C
      IF(IB.EQ.0) THEN
         IBIT0=0
         IBIT=IBIT0+IBP32
         DO 231 I=1,I223-1
         IBIT=IBIT+NWTDW(I)
 231     CONTINUE
C
         IWPT=IBIT/NBPW
         IBPT=IBIT-IWPT*NBPW
      ELSE
         IBIT=IBP32
         IWORD=IBIT/NBPW
C         
         NWORDP(1)=NWPTB+IWORD
         NBITP (1)=IBIT-IWORD*NBPW
C
         DO 232 I=2,I223-1
         IWRD=NWORDP(I-1)
         IBTP=NBITP (I-1)
         CALL BUNPCK(NBPW,KBUFF(IWRD),IR0,IWRD,IBTP,NWTDW(I-1),KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KBUFF(IWRD),IDWINC,IWRD,IBTP,6,KERR)
         IF(KERR.NE.0) RETURN
         IF(IDWINC.GT.JBPW) THEN
            KERR=15
            WRITE(0,'(1H ,A)') 'BUPMRK :'
            CALL BUERR(KERR)
            RETURN
         END IF
         NWTIWS(I-1)=NWTDW(I-1)+6+N*IDWINC
C
         IBIT = NBITP(I-1) + NWTIWS(I-1)
         IWORD= IBIT/NBPW                  
C                                                                       
         NBITP (I)= IBIT - IWORD*NBPW          
         NWORDP(I)= NWORDP(I-1) + IWORD              
 232     CONTINUE
C                                                                       
         IBPT=NBITP (I)
         IWPT=NWORDP(I)
      END IF
C
C*          2.4 GET BIT MAP FROM DATA SECTION.
C               ------------------------------
 240  CONTINUE
C
      CALL GBYTES(KBUFF(IWPT),IBV,IBPT,1,0,NR223)
C
C*          2.5 DEFINE POINTER REFERING BACK TO DATA.
C               -------------------------------------
 250  CONTINUE
C
      ISUBST=MREL-NR223+1
C
C*          2.6 REPLACE MARKERS WITH CORRESPONDING DATA WIDTHS.
C               -----------------------------------------------
C
 260  CONTINUE
C
      DO 261 I=1,NR223
      IF(IBV(I).EQ.0) THEN
         NWTDW (NP223255)=NWTDW (ISUBST)
         NWTR  (NP223255)=NWTR  (ISUBST)
         CUNITS(NP223255)=CUNITS(ISUBST)
         ISUBST=ISUBST+1
         NP223255=NP223255+1
      ELSE
         ISUBST=ISUBST+1
      END IF
 261  CONTINUE
C
      GO TO 121
C
C     ----------------------------------------------------------------- 
C*          3.  PROCESS FIRST ORDER STATISTICS OPERATOR.
C               ----------------------------------------
 300  CONTINUE
C
C*          3.1 FIND FIRST ACCURANCE OD 224255.
C               -------------------------------
 310  CONTINUE
C
      DO 311 I=NP224000,M
      IF(NWTR(I).EQ.224255) THEN
         NP224255=I
         GO TO 320
      END IF
 311  CONTINUE
C      
C*          3.2 COUNT NUMBER OF DATA PRESENT INDICATORS.
C               ----------------------------------------
 320  CONTINUE
C
      OF224=.TRUE.
      DO 321 I=NP224000,NP224255
      IF(NWTR(I).EQ.031031) THEN
         IF(OF224) THEN
            I224=I
            OF224=.FALSE.
         END IF
         NR224=NR224+1
      END IF
 321  CONTINUE
C
C*          3.3 CALCULATE WORD AND BIT POINTER TO FIRST
C               ---------------------------------------
C               DATA PRESENT INDICATOR.
C               -----------------------
 330  CONTINUE
C
      IBP32=32+NBPTB
C
      IF(IB.EQ.0) THEN
         IBIT0=0
         IBIT=IBIT0+IBP32
         DO 331 I=1,I224-1
         IBIT=IBIT+NWTDW(I)
 331     CONTINUE
C
         IWPT=IBIT/NBPW
         IBPT=IBIT-IWPT*NBPW
      ELSE
         IBIT=IBP32
         IWORD=IBIT/NBPW
C         
         NWORDP(1)=NWPTB+IWORD
         NBITP (1)=IBIT-IWORD*NBPW
C
         DO 332 I=2,I224-1
         IWRD=NWORDP(I-1)
         IBTP=NBITP (I-1)
         CALL BUNPCK(NBPW,KBUFF(IWRD),IR0,IWRD,IBTP,NWTDW(I-1),KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KBUFF(IWRD),IDWINC,IWRD,IBTP,6,KERR)
         IF(KERR.NE.0) RETURN
         IF(IDWINC.GT.JBPW) THEN
            KERR=15
            WRITE(0,'(1H ,A)') 'BUPMRK :'
            CALL BUERR(KERR)
            RETURN
         END IF
         NWTIWS(I-1)=NWTDW(I-1)+6+N*IDWINC
C
         IBIT = NBITP(I-1) + NWTIWS(I-1)
         IWORD= IBIT/NBPW                  
C                                                                       
         NBITP (I)= IBIT - IWORD*NBPW          
         NWORDP(I)= NWORDP(I-1) + IWORD              
 332     CONTINUE
C                                                                       
         IBPT=NBITP (I)
         IWPT=NWORDP(I)
      END IF
C
C
C*          3.4 GET BIT MAP FROM DATA SECTION.
C               ------------------------------
 340  CONTINUE
C
      CALL GBYTES(KBUFF(IWPT),IBV,IBPT,1,0,NR224)
C
C*          3.5 DEFINE POINTER REFERING BACK TO DATA.
C               -------------------------------------
 350  CONTINUE
C
      ISUBST=MREL-NR224+1
C
C*          3.6 REPLACE MARKERS WITH CORRESPONDING DATA WIDTHS.
C               -----------------------------------------------
C
 360  CONTINUE
C
      DO 361 I=1,NR224
      IF(IBV(I).EQ.0) THEN
         NWTDW (NP224255)=NWTDW (ISUBST)
         NWTR  (NP224255)=NWTR  (ISUBST)
         CUNITS(NP224255)=CUNITS(ISUBST)
         ISUBST=ISUBST+1
         NP224255=NP224255+1
      ELSE
         ISUBST=ISUBST+1
      END IF
 361  CONTINUE
C
      GO TO 121      
C
C
C     ----------------------------------------------------------------- 
C*          4.  PROCESS DIFFERENCE STATISTICS OPERATOR.
C               ---------------------------------------
 400  CONTINUE
C
C
C*          4.1 FIND FIRST ACCURANCE OD 223255.
C               -------------------------------
 410  CONTINUE
C
      DO 411 I=NP225000,M
      IF(NWTR(I).EQ.225255) THEN
         NP225255=I
         GO TO 420
      END IF
 411  CONTINUE
C      
C*          4.2 COUNT NUMBER OF DATA PRESENT INDICATORS.
C               ----------------------------------------
 420  CONTINUE
C
      OF225=.TRUE.
      DO 421 I=NP225000,NP225255
      IF(NWTR(I).EQ.031031) THEN
         IF(OF225) THEN
            I225=I
            OF225=.FALSE.
         END IF
         NR225=NR225+1
      END IF
 421  CONTINUE
C
C*          4.3 CALCULATE WORD AND BIT POINTER TO FIRST
C               ---------------------------------------
C               DATA PRESENT INDICATOR.
C               -----------------------
 430  CONTINUE
C
      IBP32=0. +NBPTB
C
      IF(IB.EQ.0) THEN
         IBIT0=0
         IBIT=IBIT0+IBP32
         DO 431 I=1,I225-1
         IBIT=IBIT+NWTDW(I)
 431     CONTINUE
C
         IWPT=IBIT/NBPW
         IBPT=IBIT-IWPT*NBPW
      ELSE
         IBIT=IBP32
         IWORD=IBIT/NBPW
C         
         NWORDP(1)=NWPTB+IWORD
         NBITP (1)=IBIT-IWORD*NBPW
C
         DO 432 I=2,I225-1
         IWRD=NWORDP(I-1)
         IBTP=NBITP (I-1)
         CALL BUNPCK(NBPW,KBUFF(IWRD),IR0,IWRD,IBTP,NWTDW(I-1),KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KBUFF(IWRD),IDWINC,IWRD,IBTP,6,KERR)
         IF(KERR.NE.0) RETURN
         IF(IDWINC.GT.JBPW) THEN
            KERR=15
            WRITE(0,'(1H ,A)') 'BUPMRK :'
            CALL BUERR(KERR)
            RETURN
         END IF
         NWTIWS(I-1)=NWTDW(I-1)+6+N*IDWINC
C
         IBIT = NBITP(I-1) + NWTIWS(I-1)
         IWORD= IBIT/NBPW                  
C                                                                       
         NBITP (I)= IBIT - IWORD*NBPW          
         NWORDP(I)= NWORDP(I-1) + IWORD              
 432     CONTINUE
C                                                                       
         IBPT=NBITP (I)
         IWPT=NWORDP(I)
      END IF
C
C
C*          4.4 GET BIT MAP FROM DATA SECTION.
C               ------------------------------
 440  CONTINUE
C
      CALL GBYTES(KBUFF(IWPT),IBV,IBPT,1,0,NR225)
C
C*          4.5 DEFINE POINTER REFERING BACK TO DATA.
C               -------------------------------------
 450  CONTINUE
C
      ISUBST=MREL-NR225+1
C
C*          4.6 REPLACE MARKERS WITH CORRESPONDING DATA WIDTHS.
C               -----------------------------------------------
C
 460  CONTINUE
C
      DO 461 I=1,NR225
      IF(IBV(I).EQ.0) THEN
         NWTDW (NP225255)=NWTDW (ISUBST)
         NWTR  (NP225255)=NWTR  (ISUBST)
         CUNITS(NP225255)=CUNITS(ISUBST)
         ISUBST=ISUBST+1
         NP225255=NP225255+1
      ELSE
         ISUBST=ISUBST+1
      END IF
 461  CONTINUE
C
      GO TO 121      
C
C     -----------------------------------------------------------------
C*          5.  PROCESS REPLACE/RETAINED OPERATOR.
C               ----------------------------------
 500  CONTINUE
C
C
C*          5.1 FIND FIRST ACCURANCE OD 223255.
C               -------------------------------
 510  CONTINUE
C
      DO 511 I=NP232000,M
      IF(NWTR(I).EQ.232255) THEN
         NP232255=I
         GO TO 520
      END IF
 511  CONTINUE
C      
C*          5.2 COUNT NUMBER OF DATA PRESENT INDICATORS.
C               ----------------------------------------
 520  CONTINUE
C
      OF232=.TRUE.
      DO 521 I=NP232000,NP232255
      IF(NWTR(I).EQ.031031) THEN
         IF(OF232) THEN
            I232=I
            OF232=.FALSE.
         END IF
         NR232=NR232+1
      END IF
 521  CONTINUE
C
C*          5.3 CALCULATE WORD AND BIT POINTER TO FIRST
C               ---------------------------------------
C               DATA PRESENT INDICATOR.
C               -----------------------
 530  CONTINUE
C
      IBP32=32+NBPTB
C
      IF(IB.EQ.0) THEN
         IBIT0=0
         IBIT=IBIT0+IBP32
         DO 531 I=1,I232-1
         IBIT=IBIT+NWTDW(I)
 531     CONTINUE
C
         IWPT=IBIT/NBPW
         IBPT=IBIT-IWPT*NBPW
      ELSE
         IBIT=IBP32
         IWORD=IBIT/NBPW
C         
         NWORDP(1)=NWPTB+IWORD
         NBITP (1)=IBIT-IWORD*NBPW
C
         DO 532 I=2,I232-1
         IWRD=NWORDP(I-1)
         IBTP=NBITP (I-1)
         CALL BUNPCK(NBPW,KBUFF(IWRD),IR0,IWRD,IBTP,NWTDW(I-1),KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KBUFF(IWRD),IDWINC,IWRD,IBTP,6,KERR)
         IF(KERR.NE.0) RETURN
         IF(IDWINC.GT.JBPW) THEN
            KERR=15
            WRITE(0,'(1H ,A)') 'BUPMRK :'
            CALL BUERR(KERR)
            RETURN
         END IF
         NWTIWS(I-1)=NWTDW(I-1)+6+N*IDWINC
C
         IBIT = NBITP(I-1) + NWTIWS(I-1)
         IWORD= IBIT/NBPW                  
C                                                                       
         NBITP (I)= IBIT - IWORD*NBPW          
         NWORDP(I)= NWORDP(I-1) + IWORD              
 532     CONTINUE
C                                                                       
         IBPT=NBITP (I)
         IWPT=NWORDP(I)
      END IF
C
C
C*          5.4 GET BIT MAP FROM DATA SECTION.
C               ------------------------------
 540  CONTINUE
C
      CALL GBYTES(KBUFF(IWPT),IBV,IBPT,1,0,NR232)
C
C*          5.5 DEFINE POINTER REFERING BACK TO DATA.
C               -------------------------------------
 550  CONTINUE
C
      ISUBST=MREL-NR232+1
C
C*          5.6 REPLACE MARKERS WITH CORRESPONDING DATA WIDTHS.
C               -----------------------------------------------
C
 560  CONTINUE
C
      DO 561 I=1,NR232
      IF(IBV(I).EQ.0) THEN
         NWTDW (NP232255)=NWTDW (ISUBST)
         NWTR  (NP232255)=NWTR  (ISUBST)
         CUNITS(NP232255)=CUNITS(ISUBST)
         ISUBST=ISUBST+1
         NP232255=NP232255+1
      ELSE
         ISUBST=ISUBST+1
      END IF
 561  CONTINUE
C
C     -----------------------------------------------------------------
 121  CONTINUE
C
      RETURN         
C
C
      END                         
      SUBROUTINE BUUKEY( KSEC1,KSEC2,KEY,KSUP,KERR )
C                                                                       
C**** *BUUKEY*                                                         
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Expands local ECMWF information from section 2.
C
C                                                                      
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUUKEY(KSEC1,KSEC2,KEY,KSUP,KERR)*
C                                     
C        INPUT :
C               *KSEC1*   -  array containing section 1 information
C                            KSEC1( 1)-- length of section 1 (bytes)
C                            KSEC1( 2)-- Bufr Edition number
C                            KSEC1( 3)-- originating centre
C                            KSEC1( 4)-- update sequence number
C                            KSEC1( 5)-- flag (presence of section 2)
C                            KSEC1( 6)-- bufr message type
C                            KSEC1( 7)-- bufr message subtype
C                            KSEC1( 8)-- version number of local table used
C                            KSEC1( 9)-- year
C                            KSEC1(10)-- month   
C                            KSEC1(11)-- day
C                            KSEC1(12)-- hour
C                            KSEC1(13)-- minute
C                            KSEC1(14)-- Bufr Master table
C                            KSEC1(15)-- version number of Master table used
C                            KSEC1(16) - KSEC1(JSEC1) -- local ADP centre
C                                        information(PACKED FORM)
C               *KSEC2*   -  array containing section 2 information
C                            KSEC2( 1)-- length of section 2 (bytes)
C                            KSEC2( 2) to KSEC2(JSEC2) local ADP centre
C                                        information(PACKED FORM)
C        OUTPUT:
C               *KEY*     -  array containing RDB information
C                            KEY( 1)-- length of section 2 (bytes)
C                            KEY( 2)-- RDB type
C                            KEY( 3)-- RDB subtype
C                            KEY( 4)-- year
C                            KEY( 5)-- month
C                            KEY( 6)-- day 
C                            KEY( 7)-- hour
C                            KEY( 8)-- minute
C                            KEY( 9)-- second
C                            KEY(10)-- longitude1
C                            KEY(11)-- latitude1
C                            KEY(12)-- longitude2
C                            KEY(13)-- latitude2
C                            KEY(14)-- number of subsets
C                            KEY(15)-- ident (numeric)
C                            KEY(16)-- ident ( CCITTIA5) one character 
C                            KEY(17)-- ident ( CCITTIA5) one character 
C                            KEY(18)-- ident ( CCITTIA5) one character
C                            KEY(19)-- ident ( CCITTIA5) one character
C                            KEY(20)-- ident ( CCITTIA5) one character
C                            KEY(21)-- ident ( CCITTIA5) one character
C                            KEY(22)-- ident ( CCITTIA5) one character
C                            KEY(23)-- ident ( CCITTIA5) one character
C                            KEY(24)-- ident ( CCITTIA5) one character
C                            KEY(25)-- total Bufr message length
C                            KEY(26)-- day    (RDB insertion)
C                            KEY(27)-- hour   (RDB insertion)
C                            KEY(28)-- minute (RDB insertion)
C                            KEY(29)-- second (RDB insertion)
C                            KEY(30)-- day    (MDB insertion)
C                            KEY(31)-- hour   (MDB insertion)
C                            KEY(32)-- minute (MDB insertion)
C                            KEY(33)-- second (MDB insertion)
C                            KEY(34)-- correction number
C                            KEY(35)-- part
C                            KEY(36)-- 0
C                            KEY(37)-- correction number
C                            KEY(38)-- part
C                            KEY(39)-- 0
C                            KEY(40)-- correction number    
C                            KEY(41)-- part
C                            KEY(42)-- 0
C                            KEY(43)-- correction number
C                            KEY(44)-- part
C                            KEY(45)-- 0
C                            KEY(46)-- the lowest Q/C % confidence
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          NONE.
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNPCK          - unpack bit pattern         
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       17/01/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C                                                                       
      DIMENSION KSEC1(JSEC1),KSEC2(JSEC2),KEY(JKEY)
      DIMENSION IDUM(8)     ,KSUP (JSUP )
C
C
C*          1. UNPACK LOCAL ADP CENTRE INFORMATION.
C              ------------------------------------
 100  CONTINUE
C
C
      KEY(1)=KSEC2(1)
      IW=2
      IB=0
      IF(KSEC2(1).EQ.52) THEN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(2),IW,IB, 8,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(3),IW,IB, 8,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(4),IW,IB,12,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(5),IW,IB, 4,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(6),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(7),IW,IB, 5,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(8),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(9),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY  ,IW,IB, 1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(10),IW,IB,26,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY  ,IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(11),IW,IB,25,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB, 7,KERR)
         IF(KERR.NE.0) RETURN
C
         IF(KSUP(6).GT.1.OR.KEY(2).EQ.2.OR.KEY(2).EQ.3.OR.
     1      KEY(2).EQ.12) THEN
C
            CALL BUNPCK(NBPW,KSEC2(IW),KEY(12),IW,IB,26,KERR)
            IF(KERR.NE.0) RETURN
            CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB, 6,KERR)
            IF(KERR.NE.0) RETURN
            CALL BUNPCK(NBPW,KSEC2(IW),KEY(13),IW,IB,25,KERR)
            IF(KERR.NE.0) RETURN
            CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB, 7,KERR)
            IF(KERR.NE.0) RETURN
            IF(KSUP(6).GT.256) THEN
               CALL BUNPCK(NBPW,KSEC2(IW),KEY(14),IW,IB,16,KERR)
               IF(KERR.NE.0) RETURN
               CALL BUNPCK(NBPW,KSEC2(IW),KEY(15),IW,IB,16,KERR)
               IF(KERR.NE.0) RETURN
               CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB,32,KERR)
               IF(KERR.NE.0) RETURN
               CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB, 8,KERR)
               IF(KERR.NE.0) RETURN
            ELSE
               CALL BUNPCK(NBPW,KSEC2(IW),KEY(14),IW,IB, 8,KERR)
               IF(KERR.NE.0) RETURN
               CALL BUNPCK(NBPW,KSEC2(IW),KEY(15),IW,IB,16,KERR)
               IF(KERR.NE.0) RETURN
               CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB,32,KERR)
               IF(KERR.NE.0) RETURN
               CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB,16,KERR)
               IF(KERR.NE.0) RETURN
            END IF
            GO TO 140
         ELSE
            CALL BUNPKS(NBPW,KSEC2(IW),KEY(16),IW,IB,8,0,9,KERR)
            IF(KERR.NE.0) RETURN
            CALL BUNPKS(NBPW,KSEC2(IW),IDUM(1)  ,IW,IB,8,0,8,KERR)
            IF(KERR.NE.0) RETURN
         END IF
C
C
C*          1.4 SUB KEY INFORMATION.
C               --------------------
 140  CONTINUE
C
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(25),IW,IB,16,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(26),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(27),IW,IB, 5,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(28),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(29),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB, 1,KERR)
         IF(KERR.NE.0) RETURN
C
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(30),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(31),IW,IB, 5,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(32),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(33),IW,IB, 6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),IDUMMY   ,IW,IB, 1,KERR)
         IF(KERR.NE.0) RETURN
C
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(34),IW,IB,6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(35),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(36),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(37),IW,IB,6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(38),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(39),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(40),IW,IB,6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(41),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(42),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(43),IW,IB,6,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(44),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(45),IW,IB,1,KERR)
         IF(KERR.NE.0) RETURN
         CALL BUNPCK(NBPW,KSEC2(IW),KEY(46),IW,IB,8,KERR)
         IF(KERR.NE.0) RETURN
         KSUP( 2)=46
      ELSE
         WRITE(*,'(1H )')
         WRITE(*,'(1H ,A)') 'BUUKEY : Key definition not known'
         WRITE(*,'(1H )')
      END IF
C
      RETURN
      END
      SUBROUTINE BUPKS(KBPW,KDEST,KSOURC,KWPT,KBPT,KSIZE,KSKIPB,K,KERR)
C                 
C**** *BUPKS*
C                                                                    
C                                                                    
C     PURPOSE.                                                       
C     --------                                                       
C          Purpose of this routine is to pack bit string of      
C     KSIZE bits, started at word KWPT of array KSOURC after       
C     skippinh KBPT bits. Result is put into KDEST. At the end
C     pointers KWPT and KBPT are adjusted.                   
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUPKS(KBPW,KDEST,KSOURC,KWPT,KBPT,KSIZE,KSKIPB,K,KERR)*
C                                                                       
C
C        INPUT :
C            KBPW      - number of bits per computer word
C            KSOURC    - source (continuous bit string of
C                          arbitrary length)
C            KWPT      - word pointer
C            KBPT      - bit pointer
C            KSIZE     - number of bits used for packing
C            KSKIPB    - number of bits to skip between elements
C            K         - iteration 
C
C        OUTPUT :
C            KDEST     - destination
C            KERR      - return error code
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C            NONE.                                                      
C                                                                       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C                                                                       
C          SBYTES     - pack bit pathern  
C                                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       15/01/91.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      DIMENSION KDEST(*),KSOURC(*)
C
C     ------------------------------------------------------------------
C*          1.   EXTRACT BIT PATTERN.                                   
C                --------------------                                   
 100  CONTINUE                                                          
C
      IF(KERR.NE.0) RETURN
C
      IF(KSIZE.GT.KBPW) THEN
         KERR= 13
         WRITE(0,'(1H ,A)') 'BUPKS :'
         CALL BUERR(KERR)
         RETURN
      END IF
      CALL SBYTES(KDEST,KSOURC,KBPT,KSIZE,KSKIPB,K)
C                                                                       
C     ------------------------------------------------------------------
C*          1.1  UPDATE WORD AND BIT POINTERS.                          
C                -----------------------------                          
 110  CONTINUE                                                          
C                                                                       
      KBPT = KBPT + K*(KSIZE+KSKIPB)
C                                                                       
      IF(KBPT.GE.KBPW) THEN                                             
         IW  = KBPT/ KBPW                              
         KBPT= KBPT - IW * KBPW                        
         KWPT= KWPT +IW                                
      END IF                                           
C
      RETURN
      END                                                               
      SUBROUTINE BUERR(KERR)
C
C**** *BUERR*
C
C
C     PURPOSE.
C     --------
C         PRINT ERROR CODE.
C       
C
C**   INTERFACE.
C     ----------
C
C         *CALL* *BUERR(KERR)*
C
C     METHOD.
C     -------
C
C          NONE.
C
C
C     EXTERNALS.
C     ----------
C
C          NONE.
C
C     REFERENCE.
C     ----------
C
C          NONE.
C
C     AUTHOR.
C     -------
C
C          M. DRAGOSAVAC    *ECMWF*       15/02/92.
C
C
C     MODIFICATIONS.
C     --------------
C
C          NONE.
C
C
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)
C
C
      CHARACTER*80 CERROR(100)
C
      DATA CERROR/100*' '/
C
C     ------------------------------------------------------------------
C
C*          1.   INITIALIZE MESSAGES.
C                --------------------
 100  CONTINUE
C
      CERROR( 1)=' Start of BUFR message not found.'
      CERROR( 2)=' End of BUFR message not found.'
      CERROR( 3)=' Array to receive BUFR message too small.'
      CERROR( 4)=' JSEC1 parameter too smal. Local ADP center'//
     1           ' information skipped.'
      CERROR( 5)=' JSEC2 parameter too small. Local ADP centre'//
     2           ' information skipped.'
      CERROR( 6)=' Error during read BUFR table B.'
      CERROR( 7)=' Error during read BUFR table C.'
      CERROR( 8)=' Error during read BUFR table D.'
      CERROR( 9)=' Open error.'
      CERROR(10)=' Error during close BUFR table B.'
      CERROR(11)=' Error during close BUFR table C.'
      CERROR(12)=' Error during close BUFR table D.'
      CERROR(13)=' Number of bits to be extracted greater than'//
     1           ' number of bits per computer word.'
      CERROR(14)=' Argument KVALS too small.'
      CERROR(15)=' Increment value for compressed data too big.' 
      CERROR(16)=' JSUBS parameter too small.'
      CERROR(17)=' JWORK parameter too small.'
      CERROR(18)=' Replication factor equal to zero.'
      CERROR(19)=' Delayed replication factor too big.'
      CERROR(20)=' Table D reference not found.'
      CERROR(21)=' Data descriptor operator not found.'
      CERROR(22)=' BUFR Operator name not found.'
      CERROR(23)=' Table B reference not found.'
      CERROR(24)=' Augmented table B reference not found.'
      CERROR(25)=' KELEM argument too small.'
      CERROR(26)=' Word pointer out of range.'
      CERROR(27)=' Too many subsets to be packed.'
      CERROR(28)=' Number to be packed too big.'
      CERROR(29)=' Number of descriptors KTDLEN too big.'
      CERROR(30)=' Number of elements greater than JELEM.'
      CERROR(31)=' Too few elements in KDATA array.'
      CERROR(32)=' Number of subsets equal to zero.'
      CERROR(33)=' Negative value to be packed.'
      CERROR(34)=' Number of bits to be packed greater than'//
     1           ' number of bits per computer word.'
      CERROR(35)=' '
      CERROR(36)=' Bad order of data descriptors.'
CENIV
      CERROR(91)=' Size of section 3 not greater than zero.'
C
      IF(KERR.GE.0.AND.KERR.LE.100) THEN
        WRITE(0,'(1h ,A)') CERROR(KERR)
      ELSE
        WRITE(0,'(1h ,A)') ' Error number out of range !'
      END IF
C
      RETURN
      END
      SUBROUTINE BUEXS3P(KBUFL,KBUFF,KSUP,KERR)
C
C**** *BUEXS3*                                                           
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C
C          Expand section 3 of Bufr message.
C               (preliminary items)
C
C
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUEXS3( KBUFL,KBUFF,KSUP,KERR)*
C
C        INPUT :
C               *KBUFL*   -  length of bufr message (words)
C               *KBUFF*   -  array containing bufr message
C        OUTPUT:
C               *KSUP*    -  array containing suplementary information
C                         -  KSUP( 1) -- IDIM1, dimension of KSEC1
C                         -  KSUP( 2) -- IDIM2, dimension of KSEC2
C                         -  KSUP( 3) -- IDIM3, dimension of KSEC3
C                         -  KSUP( 4) -- IDIM4, dimension of KSEC4
C                         -  KSUP( 5) -- M (number of elements in values array,
C                                           first index)
C                         -  KSUP( 6) -- N (number of subsets,second index of
C                                           values array)
C                         -  KSUP( 7) -- JVC (number of elements in CVAL array)
C                         -  KSUP( 8) -- total bufr message length in bytes
C                         -  KSUP( 9) -- IDIM0, dimension of KSEC0
C               *KERR*    -  returned error code
C                                                                       
C     METHOD.                                                           
C      -------                                                          
C                                                                       
C
C          NONE.
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          BUNEXS        - set word and bit pointers at the begining of
C                          next section
C          BUNPCK        - unpacks bit pattern
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          M. DRAGOSAVAC    *ECMWF*       01/02/91.                  
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
      INCLUDE 'PARBUF'
      INCLUDE 'COMWORK'
C
      DIMENSION KBUFF(KBUFL)
      DIMENSION KSUP(JSUP),ISEC3(JSEC3)
C                       
C     ------------------------------------------------------------------
C
C*          1.   EXPAND PRELIMINARY ITEMS OF SECTION 3.                 
C                --------------------------------------                 
 100  CONTINUE                                                          
C                                                                       
      IF( KERR.NE.0 ) RETURN                                            
C                                                                       
C*          1.1   SET THE POINTERS NWPT AND NBPT TO THE                 
C                 -------------------------------------                 
C                 BEGINING OF THE SECTION 3.                            
C                 --------------------------                            
 110  CONTINUE                                                          
C                                                                       
      NWPT = NWPTB                                                      
      NBPT = NBPTB                                                      
C                                                                       
C*          1.2   UNPACK LENGTH OF SECTION 3.                           
C                 ---------------------------                           
 120  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),ISEC3(1),NWPT,NBPT,24,KERR)
      IF(KERR.NE.0) RETURN
      if (isec3(1).le.0) then
           kerr=91
           return
      endif

C                                                                       
C*          1.2.1  SET THE POINTERS NWPTB AND NBPTB TO                  
C                  -----------------------------------                  
C                  THE BEGINNING OF THE NEXT SECTION.                   
C                  ----------------------------------                   
      CALL BUNEXS(ISEC3(1))                                             
C                                                                       
C*          1.3    UNPACK ZERO BYTE AND PUT IT IN KSEC3(2).             
C                  ----------------------------------------             
 130  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),ISEC3(2),NWPT,NBPT,8,KERR)
      IF(KERR.NE.0) RETURN
C                                                                       
C*          1.4    UNPACK NUMBER OF DATA SUB-SETS.                      
C                  -------------------------------                      
 140  CONTINUE                                                          
C                                                                       
      CALL BUNPCK(NBPW,KBUFF(NWPT),ISUBS,NWPT,NBPT,16,KERR)
      IF(KERR.NE.0) RETURN
      IF(ISUBS.GT.JSUBS) THEN
         KERR=16
         WRITE(0,'(A)')' BUEXS3P :'
         CALL BUERR(KERR)
         ksup(6)=isubs
         write(0,*)"isubs=",isubs
         RETURN 
      END IF 
C
C     -----------------------------------------------------------------
C
      KSUP(3)= 4                                                         
      KSUP(5)= 0
      KSUP(6)= ISUBS
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BUEVAR(KERR)
C
C**** *BUEVAR*
C
C
C     PURPOSE.
C     --------
C         INITIALIZE CONSTANTS AND VARIABLES.
C
C**   INTERFACE.
C     ----------
C
C         *CALL* *BUEVAR(KERR)*
C
C     METHOD.
C     -------
C
C          NONE.
C
C
C     EXTERNALS.
C     ----------
C
C         NONE.
C
C     REFERENCE.
C     ----------
C
C          NONE.
C
C     AUTHOR.
C     -------
C
C          M. DRAGOSAVAC    *ECMWF*       15/03/92.
C
C
C     MODIFICATIONS.
C     --------------
C
C          NONE.
C
C
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)
C
      INCLUDE 'PARBUF'
      INCLUDE 'COMATB'
      INCLUDE 'COMBEF'
      INCLUDE 'COMWORK'
      INCLUDE 'COMWT'
      INCLUDE 'COMROOT'
C
      CHARACTER*256 CROOT
C
C
C     ------------------------------------------------------------------
C*          1.   INITIALIZE VARIABLES AND CONSTANTS.
C                -----------------------------------
 100  CONTINUE
C
      IF(KERR.NE.0) RETURN
C
      NJA= 0
      M  =0
      MM =0
      N  =0
      JCV=0
      NBPW=JBPW
      NWPT=0
      NBPT=0
      NWPTB=0
      NBPTB=0
      NVIND=2147483647
      RVIND=1.7E38
      NBENP=0
      NLTVNP=0
      NXXP=0
      NYYP=0
      NZZP=0
      NDWINC=0
      NSCAM=0
      NAFDW=0
      NWT=0
      ODREPF=.FALSE.
      N221=0
      N263=0
      N262=0
      N261=0
      N260=0
      MREL=0
      NFCM=0
C
C Modificacoes por Waldenio em Junho/2004
C mudei o path CROOT que aponta para as tabelas BUFR
C
      CROOT='./'
C
      WRITE(*,'(1H ,A)') '             CPTEC '
      WRITE(*,'(1H )')
      WRITE(*,'(1H ,A)') '     BUFR SOFTWARE VERSION -  0.0 '
      WRITE(*,'(1H ,A)') '           JUNHO 2004. '
      WRITE(*,'(1H )')
      WRITE(*,'(1H )')
      WRITE(*,'(1H ,A)') ' Your path for Bufr tables is :'
      WRITE(*,'(1H ,A)') CROOT(1:63)
C    
      DO 101 I=1,JBPW-2
      NMASK(I)=2**I-1
 101  CONTINUE
C
      RETURN
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE BURDVMSS(KUNIT,KLEN,KBUFF,KBUFL,KERR) 
C                                                                       
C**** *BURDVMSS*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Read in one BUFR message ; returns message and its length;
C          Identical to BURDVM except that deals with Stream files; 
C          Developed by EFOFVS; 
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BURDVMSS(KUNIT,KLEN,KBUFF,KBUFL,KERR)*
C
C        INPUT :
C               *KUNIT*    -  unit number
C               *KLEN*     -  length of bufr array KBUFF
C        OUTPUT:
C               *KBUFF*    -  array (containg message)
C               *KBUFL*    -  length of bufr message in computer words
C               *KERR*     -  return code
C                               -1  end of file
C                                0  no errors detected
C                                3  length of message > KLEN                                      
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          For the unit number, array and its dimension,
C     a bufr message is returned as well as its length and 
C     return error code.
C          Interface is equal to BURDVMS; the only difference is the 
C     implementation (reads one character at a time).
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C           
C          OGETBUFR.
C          OGET7777.
C          BUERR.
C          INVBYTES.                                                                           
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          E. F. O. F.     *CPTEC*      JULY 1993.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
C
C
      DIMENSION KBUFF(KLEN)
      DIMENSION NBUFF(8192)
      CHARACTER*32768 YCHAR
      Equivalence (ychar,nbuff(1))
C
110   CONTINUE
C
      KERR=0
C
      DO 120 JJ=1,8192
         nbuff(JJ) = 0
120   CONTINUE 
C
      IF (.NOT. OGETBUFR(KUNIT)) THEN
           KERR=-1
           RETURN
      ENDIF
      YCHAR(1:4) = 'BUFR'
      ILENGTH = 4
      IF (.NOT. OGET7777(KUNIT,KLEN*4,YCHAR,ILENGTH)) THEN 
           KERR=-1
           RETURN
      ENDIF
      KBUFL=ILENGTH/4
      IOFF  = ILENGTH-KBUFL*4
      IF (IOFF .NE. 0) KBUFL=KBUFL+1
C
      IF (KBUFL .GT. KLEN) THEN
          KERR=3
          CALL BUERR(KERR)
          WRITE(0,'(A,I8)')'SUGGESTED SIZE ',KBUFL
          RETURN
      ENDIF

      DO 122 I=1,KBUFL
      KBUFF(I)=NBUFF(I)
 122  CONTINUE

C 
C     INVERTS BYTES IN MESSAGE (ENIVALDO, 28/06/93)
C
      CALL INVBYTES(ILENGTH,KBUFF) 

      RETURN
      END                                                               
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE BURDVMPB(KUNIT,KLEN,KBUFF,KBUFL,KERR) 
C                                                                       
C**** *BURDVMSS*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          Read in one BUFR message ; returns message and its length;
C          Uses PBBUFR subroutine
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BURDVMPB(KUNIT,KLEN,KBUFF,KBUFL,KERR)*
C
C        INPUT :
C               *KUNIT*    -  unit number
C               *KLEN*     -  length of bufr array KBUFF (words)
C        OUTPUT:
C               *KBUFF*    -  array (containg message)
C               *KBUFL*    -  length of bufr message in computer words
C               *KERR*     -  return code (as in PBBUFR)
C                                0  ok 
C                               -1  end of file
C                               -2  file handling problem 
C                               -3  length of message > KLEN    
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          For the unit number, array and it dimension,
C     a bufr message is returned as well as its length and 
C     return error code.
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C           
C          PBBUFR.                                                                           
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          E. F. O. F.     *CPTEC*      FEB/1995.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
C
C
      DIMENSION KBUFF(KLEN)
C
110   CONTINUE
C
      KERR=0
C      
      CALL PBBUFR(KUNIT,KBUFF,KLEN*4,KBUFLPB,KERR)
      IF (KERR .NE. 0) THEN
           RETURN
      ENDIF
      KBUFL=(KBUFLPB-1)/4 + 1
C
C 
C     INVERTS BYTES IN MESSAGE (ENIVALDO, 28/06/93)
C
      CALL INVBYTES(KBUFLPB,KBUFF) 

      RETURN
      END                                                               
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE BUWTVMPB(KUNIT,KBUFF,KBUFL,KERR) 
C                                                                       
C**** *BUWTVMPB*                                                          
C                                                                       
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C          WRITE one BUFR message ; returns message and its length;
C          Uses PBWRITE subroutine
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *CALL* *BUWTVMPB(KUNIT,KBUFF,KBUFL,KERR)*
C
C        INPUT :
C               *KUNIT*    -  unit number
C        OUTPUT:
C               *KBUFF*    -  array (containg message)
C               *KBUFL*    -  length of bufr message in computer words
C               *KERR*     -  return code (as in PBWRITE)
C                                0  ok 
C                               -1  error in writing to the file
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C           
C          PBWRITE.                                                                           
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C          NONE.                                                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C                                                                       
C          E. F. O. F.     *CPTEC*      JUN/1995.
C                                                                       
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C                                                                       
C          NONE.                                                        
C                                                                       
C                                                                       
      IMPLICIT LOGICAL(L,O,G), CHARACTER*8(C,H,Y)                       
C                                                                       
C
C
      DIMENSION KBUFF(*)
C
110   CONTINUE
C
      KERR=0
C 
C     INVERTS BYTES IN MESSAGE (ENIVALDO, 12/06/95)
C
      CALL INVBYTES(KBUFL*4,KBUFF) 
      CALL PBWRITE(KUNIT,KBUFF,KBUFL*4,KERR)
      RETURN
      END                                                               
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE GETCHAR(KUNIT,CCHAR,OEOF)
      IMPLICIT NONE
      INTEGER KUNIT
      CHARACTER*1 CCHAR
      LOGICAL OEOF
C
      OEOF=.FALSE.
      READ(KUNIT,ERR=200,END=300)CCHAR
      RETURN
200   CONTINUE
300   CONTINUE
      OEOF=.TRUE.
      RETURN
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C**   UNIT NAME: OGET7777 - GET 7777 STRING IN A BUFR MESSAGE
C
C     UNIT TYPE: LOGICAL FUNCTION
C
C     PROGRAMMING LANGUAGE: FORTRAN 77
C
C     PURPOSE:
C          Searches for a '7777' sequence of characters on stream file with 
C             logical unit number KUNIT.
C
C     SUMMARY/METHOD: 
C          Scans stream file using GETCHAR subroutine until finding '7777'. 
C          While scanning puts characters on BUFR message string YCHAR.
C
C     USAGE:
C                                                                       
C          IF (OGET7777(KUNIT,KLEN,YCHAR,ILENGTH)) THEN ...
C
C          Returns TRUE if found '7777' or FALSE if not found (searches until 
C             end of file or KLEN would be exhausted). 
C
C          INPUT PARAMETERS:
C
C                KUNIT    -  unit number
C                KLEN     -  maximum length of bufr message (in bytes)
C                YCHAR    -  current contents of message
C                ILENGTH  -  current length of message 
C
C          OUTPUT PARAMETERS:
C
C                YCHAR    -  updated contents of message 
C                ILENGTH  -  updated length of bufr message (in bytes)
C                                                            
C                                                                       
C
C     INTERFACES:
C        GLOBAL VARIABLES: NONE.
C        FILES:
C           (KUNIT)    - INPUT STREAM FILE WITH BUFR MESSAGES
C
C     EXTERNAL CODES:
C        CALLED MODULES: SUBROUTINE GETCHAR
C
C        RELATED MODULES: NONE
C 
C     REMARKS: NONE.
C   
C     KEYWORDS:
C        WMO CODE FM 94 BUFR.
C
C     ORIGIN/AUTHOR:
C        ENIVALDO.
C  
C     VERSIONS:
C        ORIGINAL: 
C
C**
C

      LOGICAL FUNCTION OGET7777(KUNIT,KLEN,YCHAR,ILENGTH)
      IMPLICIT NONE
      CHARACTER *(*) YCHAR
      INTEGER KUNIT,KLEN,ILENGTH
C
      CHARACTER*1 CCHAR
      INTEGER LIM
      LOGICAL OEOF
C
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
      LIM=KLEN-3
10    CONTINUE
      DO WHILE ((.NOT. OEOF) .AND. (CCHAR .NE. '7') 
     1                       .AND. (ILENGTH .LT. LIM))
           ILENGTH = ILENGTH + 1
           YCHAR(ILENGTH:ILENGTH) = CCHAR
           CALL GETCHAR(KUNIT,CCHAR,OEOF)
      END DO
      IF (CCHAR .NE. '7') THEN
           OGET7777=.FALSE.
           RETURN
      ENDIF
C     THERE IS A '7' CHARACTER
      ILENGTH = ILENGTH + 1
      YCHAR(ILENGTH:ILENGTH) = CCHAR
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
      IF (OEOF) THEN
          OGET7777=.FALSE.
          RETURN
      ENDIF
      IF (CCHAR .NE. '7') GO TO 10
C     THERE IS A '77' STRING
      ILENGTH = ILENGTH + 1
      YCHAR(ILENGTH:ILENGTH) = CCHAR
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
      IF (OEOF) THEN
          OGET7777=.FALSE.
          RETURN
      ENDIF
      IF (CCHAR .NE. '7') GO TO 10
C     THERE IS A '777' STRING
      ILENGTH = ILENGTH + 1
      YCHAR(ILENGTH:ILENGTH) = CCHAR
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
      IF (OEOF) THEN
          OGET7777=.FALSE.
          RETURN
      ENDIF
      IF (CCHAR .NE. '7') GO TO 10
C     THERE IS A '7777' STRING
      ILENGTH = ILENGTH + 1
      YCHAR(ILENGTH:ILENGTH) = CCHAR
      OGET7777=.TRUE.
      RETURN
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C**   UNIT NAME: OGETBUFR - GET BUFR STRING IN A BUFR MESSAGE
C
C     UNIT TYPE: LOGICAL FUNCTION
C
C     PROGRAMMING LANGUAGE: FORTRAN 77
C
C     PURPOSE:
C          Searches for a 'BUFR' sequence of characters on stream file with 
C             logical unit number KUNIT.
C
C     SUMMARY/METHOD: 
C          Scans stream file using GETCHAR subroutine until finding '7777'. 
C
C     USAGE:
C                                                                       
C          IF (OGETBUFR(KUNIT)) THEN ...
C
C          Returns TRUE if found 'BUFR' or FALSE if not found (searches until 
C             end of file). 
C
C          INPUT PARAMETERS:
C
C                KUNIT.    
C
C          OUTPUT PARAMETERS:
C                
C                NONE.
C                                                                       
C
C     INTERFACES:
C        GLOBAL VARIABLES: NONE.
C        FILES:
C           (KUNIT)    - INPUT STREAM FILE WITH BUFR MESSAGES
C
C     EXTERNAL CODES:
C        CALLED MODULES: SUBROUTINE GETCHAR
C
C        RELATED MODULES: NONE
C 
C     REMARKS: NONE.
C   
C     KEYWORDS:
C        WMO CODE FM 94 BUFR.
C
C     ORIGIN/AUTHOR:
C        ENIVALDO.
C  
C     VERSIONS:
C        ORIGINAL: 
C
C**
C

      LOGICAL FUNCTION OGETBUFR(KUNIT)
      IMPLICIT NONE
      INTEGER KUNIT
C
      CHARACTER*1 CCHAR
      LOGICAL OEOF
C
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
10    CONTINUE
      DO WHILE ((.NOT. OEOF) .AND. (CCHAR .NE. 'B'))
           CALL GETCHAR(KUNIT,CCHAR,OEOF)
      END DO
      IF (OEOF) THEN
          OGETBUFR=.FALSE.
          RETURN
      ENDIF
C     THERE IS A 'B' CHARACTER
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
      IF (OEOF) THEN
          OGETBUFR=.FALSE.
          RETURN
      ENDIF
      IF (CCHAR .NE. 'U') GO TO 10
C     THERE IS A 'BU' STRING
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
      IF (OEOF) THEN
          OGETBUFR=.FALSE.
          RETURN
      ENDIF
      IF (CCHAR .NE. 'F') GO TO 10
C     THERE IS A 'BUF' STRING
      CALL GETCHAR(KUNIT,CCHAR,OEOF)
      IF (OEOF) THEN
          OGETBUFR=.FALSE.
          RETURN
      ENDIF
      IF (CCHAR .NE. 'R') GO TO 10
C     THERE IS A 'BUFR' STRING
      OGETBUFR=.TRUE.
      RETURN
      END

C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE INVBYTES(ILEN,YBUFF)

C     INVERTS BYTE ORDERING OF STRING YBUFF (EFOF) WITHIN EACH WORD

      CHARACTER*16384 YBUFF

      CHARACTER*1 TEMP

      NWORDS=(ILEN-1)/4 +1
      DO 100 I=1,NWORDS
         IBYTE1=(I-1)*4 +1
         IBYTE2=IBYTE1+1
         IBYTE3=IBYTE1+2
         IBYTE4=IBYTE1+3
         TEMP=YBUFF(IBYTE1:IBYTE1)
         YBUFF(IBYTE1:IBYTE1)=YBUFF(IBYTE4:IBYTE4)
         YBUFF(IBYTE4:IBYTE4)=TEMP
         TEMP=YBUFF(IBYTE2:IBYTE2)
         YBUFF(IBYTE2:IBYTE2)=YBUFF(IBYTE3:IBYTE3)
         YBUFF(IBYTE3:IBYTE3)=TEMP
100   CONTINUE 
      RETURN
      END

